@@ -64,10 +64,10 @@ import qualified Darcs.Test.Patch.Examples.Set1 as Ex
 import qualified Darcs.Test.Patch.Examples.Set2Unwitnessed as ExU
 
 import Darcs.Test.Patch.Properties.Check( Check(..) )
-import qualified Darcs.Test.Patch.Properties.V1Set1 as Prop
-import qualified Darcs.Test.Patch.Properties.V1Set2 as Prop
-import qualified Darcs.Test.Patch.Properties.Generic as Prop
-import qualified Darcs.Test.Patch.Properties.Real as Prop
+import qualified Darcs.Test.Patch.Properties.V1Set1 as Prop1
+import qualified Darcs.Test.Patch.Properties.V1Set2 as Prop2
+import qualified Darcs.Test.Patch.Properties.Generic as PropG
+import qualified Darcs.Test.Patch.Properties.Real as PropR
 import qualified Darcs.Test.Patch.Properties.GenericUnwitnessed as PropU
 
 import qualified Darcs.Test.Patch.Rebase as Rebase
@@ -97,20 +97,20 @@ testCases name test datas = testCase name (assertBool assertName res)
 
 unit_V1P1:: [Test]
 unit_V1P1 =
-  [ testCases "known commutes" Prop.checkCommute Ex.knownCommutes
-  , testCases "known non-commutes" Prop.checkCantCommute Ex.knownCantCommutes
-  , testCases "known merges" Prop.checkMerge Ex.knownMerges
-  , testCases "known merges (equiv)" Prop.checkMergeEquiv Ex.knownMergeEquivs
-  , testCases "known canons" Prop.checkCanon Ex.knownCanons
-  , testCases "merge swaps" Prop.checkMergeSwap Ex.mergePairs2
-  , testCases "the patch validation works" Prop.tTestCheck Ex.validPatches
-  , testCases "commute/recommute" (Prop.recommute commute) Ex.commutePairs
-  , testCases "merge properties: merge either way valid" Prop.tMergeEitherWayValid Ex.mergePairs
-  , testCases "merge properties: merge swap" Prop.mergeEitherWay Ex.mergePairs
-  , testCases "primitive patch IO functions" (Prop.tShowRead eqFLUnsafe) Ex.primitiveTestPatches
-  , testCases "IO functions (test patches)" (Prop.tShowRead eqFLUnsafe) Ex.testPatches
-  , testCases "IO functions (named test patches)" (Prop.tShowRead unsafeCompare) Ex.testPatchesNamed
-  , testCases "primitive commute/recommute" (Prop.recommute commute) Ex.primitiveCommutePairs
+  [ testCases "known commutes" Prop1.checkCommute Ex.knownCommutes
+  , testCases "known non-commutes" Prop1.checkCantCommute Ex.knownCantCommutes
+  , testCases "known merges" Prop1.checkMerge Ex.knownMerges
+  , testCases "known merges (equiv)" Prop1.checkMergeEquiv Ex.knownMergeEquivs
+  , testCases "known canons" Prop1.checkCanon Ex.knownCanons
+  , testCases "merge swaps" Prop1.checkMergeSwap Ex.mergePairs2
+  , testCases "the patch validation works" Prop1.tTestCheck Ex.validPatches
+  , testCases "commute/recommute" (PropG.recommute commute) Ex.commutePairs
+  , testCases "merge properties: merge either way valid" Prop1.tMergeEitherWayValid Ex.mergePairs
+  , testCases "merge properties: merge swap" PropG.mergeEitherWay Ex.mergePairs
+  , testCases "primitive patch IO functions" (Prop1.tShowRead eqFLUnsafe) Ex.primitiveTestPatches
+  , testCases "IO functions (test patches)" (Prop1.tShowRead eqFLUnsafe) Ex.testPatches
+  , testCases "IO functions (named test patches)" (Prop1.tShowRead unsafeCompare) Ex.testPatchesNamed
+  , testCases "primitive commute/recommute" (PropG.recommute commute) Ex.primitiveCommutePairs
   ]
 
 unit_V2P1 :: [Test]
@@ -155,6 +155,8 @@ qc_prim :: forall prim wX wY wA model.
            , Show1 (ModelOf prim)
            , Check prim, PrimPatchBase prim, PrimOf prim ~ prim
            , FromPrim prim
+           , MightBeEmptyHunk prim
+           , MightHaveDuplicate prim
            , Show1 (prim wA)
            , Show1 ((prim :> prim) wA)
            , Show1 (WithState model prim wA)
@@ -174,7 +176,7 @@ qc_prim p =
   -- testProperty "prim inverse doesn't commute" (inverseDoesntCommute :: Prim -> Maybe Doc)
   (if runCoalesceTests p then
     [ testProperty "prim coalesce effect preserving... "
-      (unseal2 $ Prop.coalesceEffectPreserving coalesce :: Sealed2 (WithState model (prim :> prim)) -> TestResult)
+      (unseal2 $ PropG.coalesceEffectPreserving coalesce :: Sealed2 (WithState model (prim :> prim)) -> TestResult)
     ]
    else [])
     ++ concat
@@ -195,19 +197,19 @@ qc_prim p =
 qc_V2P1 :: [Test]
 qc_V2P1 =
   [ testProperty "tree flattenings are consistent... "
-    Prop.propConsistentTreeFlattenings
+    PropR.propConsistentTreeFlattenings
   , testProperty "with quickcheck that real patches are consistent... "
     (unseal $ P1.patchFromTree $ fromMaybe . isConsistent)
   -- permutivity ----------------------------------------------------------------------------
   , testConditional "permutivity"
     (unseal $ P1.commuteTripleFromTree notDuplicatestriple)
-    (unseal $ P1.commuteTripleFromTree $ Prop.permutivity commuteReals)
+    (unseal $ P1.commuteTripleFromTree $ PropG.permutivity commuteReals)
   , testConditional "partial permutivity"
     (unseal $ P1.commuteTripleFromTree notDuplicatestriple)
-    (unseal $ P1.commuteTripleFromTree $ Prop.partialPermutivity commuteReals)
+    (unseal $ P1.commuteTripleFromTree $ PropG.partialPermutivity commuteReals)
   , testConditional "nontrivial permutivity"
     (unseal $ P1.commuteTripleFromTree (\t -> nontrivialTriple t && notDuplicatestriple t))
-    (unseal $ P1.commuteTripleFromTree $ (Prop.permutivity commuteReals))
+    (unseal $ P1.commuteTripleFromTree $ (PropG.permutivity commuteReals))
   ]
 
 qc_V2 :: forall prim wXx wYy . (PrimPatch prim, Show1 (ModelOf prim), RepoModel (ModelOf prim),
@@ -216,11 +218,11 @@ qc_V2 :: forall prim wXx wYy . (PrimPatch prim, Show1 (ModelOf prim), RepoModel
       => prim wXx wYy -> [Test]
 qc_V2 _ =
   [ testProperty "readPatch and showPatch work on RealPatch... "
-    (unseal $ patchFromTree $ (Prop.show_read :: RealPatch prim wX wY -> TestResult))
+    (unseal $ patchFromTree $ (PropG.show_read :: RealPatch prim wX wY -> TestResult))
   , testProperty "readPatch and showPatch work on FL RealPatch... "
-    (unseal2 $ (Prop.show_read :: FL (RealPatch prim) wX wY -> TestResult))
+    (unseal2 $ (PropG.show_read :: FL (RealPatch prim) wX wY -> TestResult))
   , testProperty "we can do merges using QuickCheck"
-    (isNothing . (Prop.propIsMergeable ::
+    (isNothing . (PropG.propIsMergeable ::
                      Sealed (WithStartState (ModelOf prim) (Tree prim))
                      -> Maybe (Tree (RealPatch prim) wX)))
   ]
@@ -247,22 +249,28 @@ properties gen prefix genname tests =
 
 type PropList what gen = String -> TestGenerator what gen -> [Test]
 
-pair_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p, ShowPatchBasic p, MyEq p)
-                   => p x y -> PropList (p :> p) gen
+pair_properties :: forall p gen x y
+                 . ( Show1 gen, Arbitrary (Sealed gen), Patchy p, MightHaveDuplicate p
+                   , ShowPatchBasic p, MyEq p
+                   )
+                => p x y -> PropList (p :> p) gen
 pair_properties _ genname gen =
   properties gen "commute" genname
-  [ ("recommute"              , const True       , Prop.recommute commute             )
-  , ("nontrivial recommute"   , nontrivialCommute, Prop.recommute commute             )
-  , ("inverses commute"       , const True       , Prop.commuteInverses commute       )
-  , ("nontrivial inverses"    , nontrivialCommute, Prop.commuteInverses commute       )
-  , ("inverse composition"    , const True       , Prop.inverseComposition            )
+  [ ("recommute"              , const True       , PropG.recommute commute             )
+  , ("nontrivial recommute"   , nontrivialCommute, PropG.recommute commute             )
+  , ("inverses commute"       , const True       , PropG.commuteInverses commute       )
+  , ("nontrivial inverses"    , nontrivialCommute, PropG.commuteInverses commute       )
+  , ("inverse composition"    , const True       , PropG.inverseComposition            )
   ]
 
-coalesce_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p, PrimPatch p, ArbitraryPrim p)
+coalesce_properties :: forall p gen x y
+                     . ( Show1 gen, Arbitrary (Sealed gen), Patchy p, PrimPatch p
+                       , ArbitraryPrim p, MightBeEmptyHunk p
+                       )
                     => p x y -> PropList (p :> p :> p) gen
 coalesce_properties p genname gen =
   properties gen "commute" genname
-   (if runCoalesceTests p then [ ("coalesce commutes with commute", const True, Prop.coalesceCommute coalesce) ] else [])
+   (if runCoalesceTests p then [ ("coalesce commutes with commute", const True, PropG.coalesceCommute coalesce) ] else [])
 
 -- The following properties do not hold for "Real" patches (conflictors and
 -- duplicates, specifically) .
@@ -271,15 +279,15 @@ nonreal_commute_properties :: forall p gen x y
                            => p x y -> PropList (p :> p) gen
 nonreal_commute_properties _ genname gen =
   properties gen "commute" genname
-  [ ("patch & inverse commute", const True       , Prop.patchAndInverseCommute commute)
-  , ("patch & inverse commute", nontrivialCommute, Prop.patchAndInverseCommute commute)
+  [ ("patch & inverse commute", const True       , PropG.patchAndInverseCommute commute)
+  , ("patch & inverse commute", nontrivialCommute, PropG.patchAndInverseCommute commute)
   ]
 
 patch_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p, MyEq p)
                  => p x y -> PropList p gen
 patch_properties _ genname gen =
   properties gen "patch" genname
-  [ ("inverse . inverse is id"  , const True       , Prop.invertSymmetry)
+  [ ("inverse . inverse is id"  , const True       , PropG.invertSymmetry)
   ]
 
 patch_repo_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen),
@@ -289,60 +297,61 @@ patch_repo_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen),
                       => p x y -> PropList (WithState (ModelOf (PrimOf p)) p) gen
 patch_repo_properties _ genname gen =
   properties gen "patch/repo" genname
-  [ ("invert rollback"          , const True       , Prop.invertRollback)
+  [ ("invert rollback"          , const True       , PropG.invertRollback)
   ]
 
 pair_repo_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen), Patchy p,
+                                            MightBeEmptyHunk p,
                                             RepoModel (ModelOf p),
                                             RepoState (ModelOf p) ~ ApplyState p)
                       => p x y -> PropList (WithState (ModelOf p) (p :> p)) gen
 pair_repo_properties _ genname gen =
   properties gen "patch/repo" genname
-  [ ("commute is effect preserving" , const True       , Prop.effectPreserving commute )
+  [ ("commute is effect preserving" , const True       , PropG.effectPreserving commute )
   ]
 
 merge_properties :: forall p gen x y. (Show1 gen, Arbitrary (Sealed gen)
                                       , Patchy p, MyEq p, Merge p, ShowPatchBasic p
-                                      , Show2 p, Check p)
+                                      , MightHaveDuplicate p, Show2 p, Check p)
                  => p x y -> PropList (p :\/: p) gen
 merge_properties _ genname gen =
   properties gen "merge" genname
-  [ ("merge either way"           , const True     , Prop.mergeEitherWay      )
-  , ("merge either way valid"     , const True     , Prop.tMergeEitherWayValid)
-  , ("nontrivial merge either way", nontrivialMerge, Prop.mergeEitherWay      )
-  , ("merge commute"              , const True     , Prop.mergeCommute        )
+  [ ("merge either way"           , const True     , PropG.mergeEitherWay      )
+  , ("merge either way valid"     , const True     , Prop1.tMergeEitherWayValid)
+  , ("nontrivial merge either way", nontrivialMerge, PropG.mergeEitherWay      )
+  , ("merge commute"              , const True     , PropG.mergeCommute        )
   ]
 
 qc_V1P1 :: [Test]
 qc_V1P1 =
   [
-    testProperty "show and read work right" (unseal Prop.propReadShow)
+    testProperty "show and read work right" (unseal Prop2.propReadShow)
   ]
-  ++ Prop.checkSubcommutes Prop.subcommutesInverse "patch and inverse both commute"
-  ++ Prop.checkSubcommutes Prop.subcommutesNontrivialInverse "nontrivial commutes are correct"
-  ++ Prop.checkSubcommutes Prop.subcommutesFailure "inverses fail"
+  ++ Prop2.checkSubcommutes Prop2.subcommutesInverse "patch and inverse both commute"
+  ++ Prop2.checkSubcommutes Prop2.subcommutesNontrivialInverse "nontrivial commutes are correct"
+  ++ Prop2.checkSubcommutes Prop2.subcommutesFailure "inverses fail"
   ++
-  [ testProperty "commuting by patch and its inverse is ok" Prop.propCommuteInverse
+  [ testProperty "commuting by patch and its inverse is ok" Prop2.propCommuteInverse
   -- , testProperty "conflict resolution is valid... " Prop.propResolveConflictsValid
   , testProperty "a patch followed by its inverse is identity"
-    Prop.propPatchAndInverseIsIdentity
-  , testProperty "'simple smart merge'" Prop.propSimpleSmartMergeGoodEnough
-  , testProperty "commutes are equivalent" Prop.propCommuteEquivalency
-  , testProperty "merges are valid" Prop.propMergeValid
-  , testProperty "inverses being valid" Prop.propInverseValid
-  , testProperty "other inverse being valid" Prop.propOtherInverseValid
+    Prop2.propPatchAndInverseIsIdentity
+  , testProperty "'simple smart merge'" Prop2.propSimpleSmartMergeGoodEnough
+  , testProperty "commutes are equivalent" Prop2.propCommuteEquivalency
+  , testProperty "merges are valid" Prop2.propMergeValid
+  , testProperty "inverses being valid" Prop2.propInverseValid
+  , testProperty "other inverse being valid" Prop2.propOtherInverseValid
   -- The patch generator isn't smart enough to generate correct test cases for
   -- the following: (which will be obsoleted soon, anyhow)
   -- , testProperty "the order dependence of unravel... " Prop.propUnravelOrderIndependent
   -- , testProperty "the unravelling of three merges... " Prop.propUnravelThreeMerge
   -- , testProperty "the unravelling of a merge of a sequence... " Prop.propUnravelSeqMerge
-  , testProperty "the order of commutes" Prop.propCommuteEitherOrder
-  , testProperty "commute either way" Prop.propCommuteEitherWay
-  , testProperty "the double commute" Prop.propCommuteTwice
+  , testProperty "the order of commutes" Prop2.propCommuteEitherOrder
+  , testProperty "commute either way" Prop2.propCommuteEitherWay
+  , testProperty "the double commute" Prop2.propCommuteTwice
   , testProperty "merges commute and are well behaved"
-    Prop.propMergeIsCommutableAndCorrect
-  , testProperty "merges can be swapped" Prop.propMergeIsSwapable
-  , testProperty "again that merges can be swapped (I'm paranoid) " Prop.propMergeIsSwapable
+    Prop2.propMergeIsCommutableAndCorrect
+  , testProperty "merges can be swapped" Prop2.propMergeIsSwapable
+  , testProperty "again that merges can be swapped (I'm paranoid) " Prop2.propMergeIsSwapable
 
   ] -- the following properties are disabled, because they routinely lead to
     -- exponential cases, making the tests run for ever and ever; nevertheless,
