@@ -4,6 +4,7 @@
 
 module Darcs.Test.Patch.Arbitrary.Generic
        ( Tree(..), TreeWithFlattenPos(..), G2(..), ArbitraryPrim(..), NullPatch(..), RepoModel(..)
+       , MightBeEmptyHunk(..), MightHaveDuplicate(..)
        , flattenOne, flattenTree, mapTree, sizeTree
        , commutePairFromTree, mergePairFromTree
        , commuteTripleFromTree, mergePairFromCommutePair
@@ -106,6 +107,28 @@ arbitraryTree rm depth
 class NullPatch p where
   nullPatch :: p wX wY -> EqCheck wX wY
 
+class MightBeEmptyHunk p where
+  -- |V1 Prims support the value 'Hunk n [] []' that is treated specially in the
+  -- commute code and ends up breaking certain tests by behaving anomalously.
+  -- In practice they shouldn't appear in real repositories. For later,
+  -- as yet unreleased patch types, we should eliminate them completely.
+  -- An alternative to using this as a guard might be to avoid generating them.
+  isEmptyHunk :: p wX wY -> Bool
+  isEmptyHunk _ = False
+
+instance MightBeEmptyHunk (FL p)
+
+class MightHaveDuplicate p where
+  -- |"duplicates" in V2 patches (RealPatch) have lots of bugs
+  -- that break various commute/merge properties.
+  hasDuplicate :: p wX wY -> Bool
+  hasDuplicate _ = False
+
+instance MightHaveDuplicate p => MightHaveDuplicate (FL p) where
+  hasDuplicate NilFL = False
+  hasDuplicate (p :>: ps) = hasDuplicate p || hasDuplicate ps
+
+
 class ( ArbitraryState (ModelOf prim) prim
       , NullPatch prim
       , PrimPatch prim
