@@ -166,11 +166,11 @@ itemIsDir :: Item -> Bool
 itemIsDir i = unsafeHead (iDescriptor i) == c2w 'D'
 
 -- xlatePeek32 = fmap xlate32 . peek
-xlatePeek64 :: (Storable a, Bits a) => Ptr a -> IO a
+xlatePeek64 :: (Storable a, Num a, Bits a) => Ptr a -> IO a
 xlatePeek64 = fmap xlate64 . peek
 
 -- xlatePoke32 ptr v = poke ptr (xlate32 v)
-xlatePoke64 :: (Storable a, Bits a) => Ptr a -> a -> IO ()
+xlatePoke64 :: (Storable a, Num a, Bits a) => Ptr a -> a -> IO ()
 xlatePoke64 ptr v = poke ptr (xlate64 v)
 
 -- | Lay out the basic index item structure in memory. The memory location is
