@@ -4,11 +4,10 @@ module Darcs.Patch.Bracketed.Instances () where
 import Darcs.Patch.Bracketed ( Bracketed(..) )
 import Darcs.Patch.Show ( ShowPatchBasic(..) )
 
-import Darcs.Witnesses.Ordered ( FL(NilFL), mapFL )
+import Darcs.Patch.Witnesses.Ordered ( FL(NilFL), mapFL )
 
-import Printer ( vcat, blueText, ($$) )
+import Darcs.Util.Printer ( vcat, blueText, ($$) )
 
-#include "gadts.h"
 
 instance ShowPatchBasic p => ShowPatchBasic (Bracketed p) where
     showPatch (Singleton p) = showPatch p
