@@ -3,24 +3,23 @@ module Darcs.Patch.FileHunk
     )
     where
 
-import Darcs.Patch.FileName ( FileName )
+import Darcs.Util.Path ( FileName )
 import Darcs.Patch.Format ( FileNameFormat )
 import Darcs.Patch.Show ( formatFileName )
 
-import Printer
+import Darcs.Util.Printer
     ( Doc, blueText, text, lineColor, vcat, userchunkPS
     , prefix, ($$), (<+>), Color(Cyan, Magenta) )
 
 import qualified Data.ByteString as B ( ByteString )
 
-#include "gadts.h"
 
-data FileHunk C(x y) = FileHunk !FileName !Int [B.ByteString] [B.ByteString]
+data FileHunk wX wY = FileHunk !FileName !Int [B.ByteString] [B.ByteString]
 
 class IsHunk p where
-    isHunk :: p C(x y) -> Maybe (FileHunk C(x y))
+    isHunk :: p wX wY -> Maybe (FileHunk wX wY)
 
-showFileHunk :: FileNameFormat -> FileHunk C(x y) -> Doc
+showFileHunk :: FileNameFormat -> FileHunk wX wY -> Doc
 showFileHunk x (FileHunk f line old new) =
            blueText "hunk" <+> formatFileName x f <+> text (show line)
         $$ lineColor Magenta (prefix "-" (vcat $ map userchunkPS old))
