@@ -16,20 +16,20 @@
 -- Boston, MA 02110-1301, USA.
 
 module Darcs.Patch.Info ( PatchInfo(..), patchinfo, invertName,
-                          idpatchinfo, addJunk, makePatchname,
-                          makeFilename, makeAltFilename, readPatchInfo,
-                          justName, justAuthor, justLog, repopatchinfo,
-                          RepoPatchInfo, humanFriendly, toXml, piDate,
+                          rawPatchInfo,
+                          addJunk, makePatchname,
+                          makeFilename, readPatchInfo,
+                          justName, justAuthor, justLog,
+                          showPatchInfoUI, toXml, piDate,
                           setPiDate, piDateString, piDateBytestring,
                           piName, piRename, piAuthor, piTag, piLog,
                           showPatchInfo, isTag, readPatchInfos, escapeXML
                         ) where
-import Text.Html hiding (name, text, option)
 import System.Random ( randomRIO )
 import Numeric ( showHex )
-import Control.Monad ( when, unless )
+import Control.Monad ( when, unless, void )
 
-import ByteStringUtils ( unlinesPS, packStringToUTF8, unpackPSFromUTF8, decodeLocale)
+import Darcs.Util.ByteString ( unlinesPS, packStringToUTF8, unpackPSFromUTF8, decodeLocale)
 import qualified Darcs.Patch.ReadMonads as RM ( take )
 import Darcs.Patch.ReadMonads as RM ( skipSpace, char,
                                       takeTill, anyChar, ParserM,
@@ -43,21 +43,17 @@ import qualified Data.ByteString       as B  (length, splitAt, null
 import qualified Data.ByteString.Char8 as BC (index, head, unpack, pack)
 import Data.List( isPrefixOf )
 
-import Printer ( renderString, Doc, packedString,
-                 empty, ($$), (<>), (<+>), vcat, text, blueText, prefix )
+import Darcs.Util.Printer ( Doc, packedString,
+                 empty, ($$), (<>), (<+>), vcat, text, cyanText, blueText, prefix )
 import Darcs.Patch.OldDate ( readUTCDate, showIsoDateTime )
 import System.Time ( CalendarTime(ctTZ), calendarTimeToString, toClockTime,
                      toCalendarTime )
 import System.IO.Unsafe ( unsafePerformIO )
-import SHA1 ( sha1PS )
-import Darcs.Utils ( promptYorn )
+import Darcs.Util.Crypt.SHA1 ( sha1PS, SHA1 )
+import Darcs.Util.Prompt ( promptYorn )
+import Darcs.Util.Show ( appPrec )
 import Prelude hiding (pi, log)
 
-data RepoPatchInfo = RPI String PatchInfo
-
-repopatchinfo :: String -> PatchInfo -> RepoPatchInfo
-repopatchinfo r pi = RPI r pi
-
 -- | A PatchInfo value contains the metadata of a patch. The date, name, author
 -- and log fields are UTF-8 encoded text in darcs 2.4 and later, and just
 -- sequences of bytes (decoded with whatever is the locale when displayed) in
@@ -73,17 +69,30 @@ data PatchInfo = PatchInfo { _piDate    :: !B.ByteString
                            }
                  deriving (Eq,Ord)
 
-idpatchinfo :: PatchInfo
-idpatchinfo = PatchInfo myid myid myid [] False
-    where myid = BC.pack "identity"
+instance Show PatchInfo where
+    showsPrec d (PatchInfo date name author log inverted) =
+        showParen (d > appPrec) $
+            showString "rawPatchInfo " . showsPrec (appPrec + 1) date .
+            showString " " . showsPrec (appPrec + 1) name .
+            showString " " . showsPrec (appPrec + 1) author .
+            showString " " . showsPrec (appPrec + 1) log .
+            showString " " . showsPrec (appPrec + 1) inverted
 
+rawPatchInfo :: String -> String -> String -> [String] -> Bool -> PatchInfo
+rawPatchInfo date name author log inverted =
+    PatchInfo { _piDate     = BC.pack date
+              , _piName     = packStringToUTF8 name
+              , _piAuthor   = packStringToUTF8 author
+              , _piLog      = map packStringToUTF8 log
+              , isInverted  = inverted }
+
+-- | @patchinfo date name author log@ constructs a new 'PatchInfo' value
+-- with the given details, automatically assigning an Ignore-this header
+-- to guarantee the patch is unique.  The function does not verify
+-- the date string's sanity.
 patchinfo :: String -> String -> String -> [String] -> IO PatchInfo
 patchinfo date name author log =
-    addJunk $ PatchInfo { _piDate     = BC.pack date
-                         , _piName     = packStringToUTF8 name
-                         , _piAuthor   = packStringToUTF8 author
-                         , _piLog      = map packStringToUTF8 log
-                         , isInverted  = False }
+    addJunk $ rawPatchInfo date name author log False
 
 -- | addJunk adds a line that contains a random number to make the patch
 --   unique.
@@ -123,9 +132,11 @@ justAuthor =  metadataToString . _piAuthor
 justLog :: PatchInfo -> String
 justLog = unlines . map BC.unpack . _piLog
 
-humanFriendly :: PatchInfo -> Doc
-humanFriendly pi =
-    text (friendlyD $ _piDate pi) <> text "  " <> text (piAuthor pi)
+showPatchInfoUI :: PatchInfo -> Doc
+showPatchInfoUI pi =
+    cyanText "patch " <> cyanText (show $ makePatchname pi)
+ $$ text "Author: " <> text (piAuthor pi)
+ $$ text "Date:   " <> text (friendlyD $ _piDate pi)
  $$ hfn (piName pi)
  $$ vcat (map ((text "  " <>) . text) (piLog pi))
   where hfn x = case piTag pi of
@@ -189,8 +200,8 @@ piTag pinf =
 --   We try UTF-8 first because UTF-8 is clearly recognizable, widely used,
 --   and people may have UTF-8 patches even when UTF-8 is not their locale.
 metadataToString :: B.ByteString -> String
-metadataToString bs | not ('\xfffd' `elem` bsUtf8) = bsUtf8
-                    | otherwise                    = decodeLocale bs
+metadataToString bs | '\xfffd' `notElem` bsUtf8 = bsUtf8
+                    | otherwise                 = decodeLocale bs
   where bsUtf8 = unpackPSFromUTF8 bs
 
 friendlyD :: B.ByteString -> String
@@ -206,7 +217,7 @@ toXml pi =
     <+> text "date='" <> escapeXMLByteString (_piDate pi) <> text "'"
     <+> text "local_date='" <> escapeXML (friendlyD $ _piDate pi) <> text "'"
     <+> text "inverted='" <> text (show $ isInverted pi) <> text "'"
-    <+> text "hash='" <> text (makePatchname pi) <> text "'>"
+    <+> text "hash='" <> text (show $ makePatchname pi) <> text "'>"
  $$     prefix "\t" (
             text "<name>" <> escapeXMLByteString (_piName pi) <> text "</name>"
          $$ commentsAsXml (_piLog pi))
@@ -238,8 +249,8 @@ escapeXMLByteString = packedString . bstrReplace '\'' "&apos;"
 strReplace :: Char -> String -> String -> String
 strReplace _ _ [] = []
 strReplace x y (z:zs)
-  | x == z    = y ++ (strReplace x y zs)
-  | otherwise = z : (strReplace x y zs)
+  | x == z    = y ++ strReplace x y zs
+  | otherwise = z : strReplace x y zs
 
 bstrReplace :: Char -> String -> B.ByteString -> B.ByteString
 bstrReplace c s bs | B.null bs   = B.empty
@@ -249,12 +260,6 @@ bstrReplace c s bs | B.null bs   = B.empty
                                      else B.cons (B.head bs)
                                                  (bstrReplace c s (B.tail bs))
 
-makeAltFilename :: PatchInfo -> String
-makeAltFilename pi@(PatchInfo { isInverted = False }) =
-    fixUpFname (midtrunc (piName pi)++"-"++justAuthor pi++"-"++BC.unpack (_piDate pi))
-makeAltFilename pi@(PatchInfo { isInverted = True}) =
-    makeAltFilename (pi { isInverted = False }) ++ "-inverted"
-
 -- | This makes darcs-1 (non-hashed repos) filenames, and is also generally
 -- used in both in hashed and non-hashed repo code for making patch "hashes".
 --
@@ -267,11 +272,15 @@ makeAltFilename pi@(PatchInfo { isInverted = True}) =
 --  * SHA1 hash of the patch name, author, date, log, and \"inverted\"
 --    flag.
 makeFilename :: PatchInfo -> String
-makeFilename pi = makePatchname pi ++ ".gz"
+makeFilename pi = showIsoDateTime d++"-"++sha1_a++"-"++ (show $ makePatchname pi) ++ ".gz"
+    where d = readPatchDate $ _piDate pi
+          sha1_a = take 5 $ show $ sha1PS $ _piAuthor pi
 
-makePatchname :: PatchInfo -> String
-makePatchname pi =
-    showIsoDateTime d++"-"++sha1_a++"-"++sha1PS sha1_me
+-- | Hash on patch metadata (patch name, author, date, log, and \"inverted\"
+-- flag. Robust against context changes but does not garantee patch contents.
+-- Usually used as matcher or patch identifier (see Darcs.Patch.Match).
+makePatchname :: PatchInfo -> SHA1
+makePatchname pi = sha1PS sha1_me
         where b2ps True = BC.pack "t"
               b2ps False = BC.pack "f"
               sha1_me = B.concat [_piName pi,
@@ -279,48 +288,7 @@ makePatchname pi =
                                   _piDate pi,
                                   B.concat $ _piLog pi,
                                   b2ps $ isInverted pi]
-              d = readPatchDate $ _piDate pi
-              sha1_a = take 5 $ sha1PS $ _piAuthor pi
 
-midtrunc :: String -> String
-midtrunc s
-    | length s < 73 = s
-    | otherwise = (take 40 s)++"..."++(reverse $ take 30 $ reverse s)
-fixUpFname :: String -> String
-fixUpFname = map mungeChar
-
-mungeChar :: Char -> Char
-mungeChar '*' = '+'
-mungeChar '?' = '2'
-mungeChar '>' = '7'
-mungeChar '<' = '2'
-mungeChar ' ' = '_'
-mungeChar '"' = '~'
-mungeChar '`' = '.'
-mungeChar '\'' = '.'
-mungeChar '/' = '1'
-mungeChar '\\' = '1'
-mungeChar '!' = '1'
-mungeChar ':' = '.'
-mungeChar ';' = ','
-mungeChar '{' = '~'
-mungeChar '}' = '~'
-mungeChar '(' = '~'
-mungeChar ')' = '~'
-mungeChar '[' = '~'
-mungeChar ']' = '~'
-mungeChar '=' = '+'
-mungeChar '#' = '+'
-mungeChar '%' = '8'
-mungeChar '&' = '6'
-mungeChar '@' = '9'
-mungeChar '|' = '1'
-mungeChar  c  =  c
-
-instance  HTML RepoPatchInfo  where
-    toHtml = htmlPatchInfo
-instance  Show PatchInfo  where
-    show pi = renderString (showPatchInfo pi)
 
 -- |Patch is stored between square brackets.
 --
@@ -341,9 +309,8 @@ showPatchInfo pi =
                                  <> myunlines (_piLog pi) <> blueText "] "
     where inverted = if isInverted pi then "*-" else "**"
           myunlines [] = empty
-          myunlines xs = mul xs
-              where mul [] = text "\n"
-                    mul (s:ss) = text "\n " <> packedString s <> mul ss
+          myunlines xs =
+              foldr (\s -> ((text "\n " <> packedString s) <>)) (text "\n") xs
 
 -- |Parser for 'PatchInfo' as stored in patch bundles and inventory files,
 -- for example:
@@ -355,7 +322,7 @@ showPatchInfo pi =
 -- > ]
 --
 -- See 'showPatchInfo' for the inverse operation.
-readPatchInfo :: ParserM m => m (PatchInfo)
+readPatchInfo :: ParserM m => m PatchInfo
 readPatchInfo = do
   skipSpace
   char '['
@@ -364,14 +331,14 @@ readPatchInfo = do
   author <- takeTillChar '*'
   s2 <- RM.take 2
   ct <- takeTill (\c->c==']'||c=='\n')
-  option () (char '\n' >> return ()) -- consume newline char, if present
+  option () (void (char '\n')) -- consume newline char, if present
   log <- linesStartingWithEndingWith ' ' ']'
-  return $ PatchInfo { _piDate = ct
-                     , _piName = name
-                     , _piAuthor = author
-                     , _piLog = log
-                     , isInverted = BC.index s2 1 /= '*'
-                     }
+  return PatchInfo { _piDate = ct
+                   , _piName = name
+                   , _piAuthor = author
+                   , _piLog = log
+                   , isInverted = BC.index s2 1 /= '*'
+                   }
 
 readPatchInfos :: B.ByteString -> [PatchInfo]
 readPatchInfos inv | B.null inv = []
@@ -379,16 +346,3 @@ readPatchInfos inv = case parseStrictly readPatchInfo inv of
                      Just (pinfo,r) -> pinfo : readPatchInfos r
                      _ -> []
 
-htmlPatchInfo :: RepoPatchInfo -> Html
-htmlPatchInfo (RPI r pi) =
-    toHtml $ (td << patchLink r pi) `above`
-               ((td ! [align "right"] << mailLink (justAuthor pi)) `beside`
-                (td << (friendlyD $ _piDate pi)))
-
-patchLink :: String -> PatchInfo -> Html
-patchLink r pi =
-    toHtml $ hotlink
-               ("darcs?"++r++"**"++makeFilename pi)
-               [toHtml $ piName pi]
-mailLink :: String -> Html
-mailLink email = toHtml $ hotlink ("mailto:"++email) [toHtml email]
