@@ -3,16 +3,15 @@ module Darcs.Patch.Inspect
        )
        where
 
-import Darcs.Witnesses.Ordered ( FL, RL, reverseRL, mapFL )
+import Darcs.Patch.Witnesses.Ordered ( FL, RL, reverseRL, mapFL )
 
 import qualified Data.ByteString.Char8 as BC
 import Data.List ( nub )
 
-#include "gadts.h"
 
 class PatchInspect p where
-    listTouchedFiles :: p C(x y) -> [FilePath]
-    hunkMatches :: (BC.ByteString -> Bool) -> p C(x y) -> Bool
+    listTouchedFiles :: p wX wY -> [FilePath]
+    hunkMatches :: (BC.ByteString -> Bool) -> p wX wY -> Bool
 
 instance PatchInspect p => PatchInspect (FL p) where
     listTouchedFiles xs = nub $ concat $ mapFL listTouchedFiles xs
