@@ -119,8 +119,7 @@ import Darcs.Patch.Dummy ( DummyPatch )
 import Darcs.Patch.Matchable ( Matchable )
 import Darcs.Patch.MaybeInternal ( MaybeInternal, patchInternalChecker, isInternal, flIsInternal )
 import Darcs.Patch.MonadProgress ( MonadProgress )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, piap,
-                         conscientiously, hopefully, n2pia )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, conscientiously, hopefully )
 import Darcs.Patch.Set ( PatchSet(..), Tagged(..), SealedPatchSet, newset2RL, Origin )
 import Darcs.Patch.Type ( PatchType(..) )
 import Darcs.Patch.Apply( ApplyState )
@@ -538,11 +537,11 @@ matchAPatchread fs = case nonrangeMatcher fs of
 
 -- | @matchAPatch fs p@ tells whether @p@ matches the matchers in
 -- the flags @fs@
-matchAPatch :: Matchable p => [MatchFlag] -> Named p wX wY -> Bool
+matchAPatch :: Matchable p => [MatchFlag] -> PatchInfoAnd p wX wY -> Bool
 matchAPatch fs p =
     case nonrangeMatcher fs of
     Nothing -> True
-    Just m -> applyMatcher m (patch2patchinfo p `piap` p)
+    Just m -> applyMatcher m p
 
 matchPatch :: Matchable p => [MatchFlag] -> PatchSet p wStart wX -> Sealed2 (Named p)
 matchPatch fs ps =
@@ -619,7 +618,7 @@ matchSecondPatchset fs ps =
 -- the earliest patch in a sequence, as opposed to 'matchSecondPatchset' which picks up the
 -- first match starting from the latest patch
 splitSecondFL :: Matchable p
-              => (forall wA wB . q wA wB -> Sealed2 (Named p))
+              => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd p))
               -> [MatchFlag]
               -> FL q wX wY
               -> (FL q :> FL q) wX wY -- ^The first element is the patches before and including the first patch matching the second matcher,
@@ -666,10 +665,10 @@ getMatchingTag m (PatchSet (p:<:ps) ts)
             Just (patchSet :> _) -> seal patchSet
     | otherwise = getMatchingTag m (PatchSet ps ts)
 
-splitMatchFL :: Matchable p => (forall wA wB . q wA wB -> Sealed2 (Named p)) -> Matcher p -> FL q wX wY -> (FL q :> FL q) wX wY
+splitMatchFL :: Matchable p => (forall wA wB . q wA wB -> Sealed2 (PatchInfoAnd p)) -> Matcher p -> FL q wX wY -> (FL q :> FL q) wX wY
 splitMatchFL _extract m NilFL = error $ "Couldn't find patch matching " ++ show m
 splitMatchFL extract m (p :>: ps)
-   | unseal2 (applyMatcher m . n2pia) . extract $ p = (p :>: NilFL) :> ps
+   | unseal2 (applyMatcher m) . extract $ p = (p :>: NilFL) :> ps
    | otherwise = case splitMatchFL extract m ps of
                     before :> after -> (p :>: before) :> after
 
