@@ -1,4 +1,3 @@
-#include "gadts.h"
 
 -- |
 -- Module      : Darcs.Patch.Merge
@@ -9,6 +8,7 @@
 module Darcs.Patch.Merge
     (
       Merge(..)
+    , selfMerger
     , mergeFL
     ) where
 
@@ -16,16 +16,24 @@ module Darcs.Patch.Merge
 import Data.Maybe ( fromJust )
 
 import Darcs.Patch.Commute ( Commute )
-import Darcs.Witnesses.Ordered ( (:\/:)(..), (:/\:)(..),
-                                 FL(..), RL,
-                                 reverseFL, reverseRL
-                               )
-
+import Darcs.Patch.CommuteFn ( MergeFn )
+import Darcs.Patch.Witnesses.Ordered
+    ( (:\/:)(..)
+    , (:/\:)(..)
+    , FL(..)
+    , RL
+    , reverseFL
+    , reverseRL
+    )
 
 -- | Things that can always be merged
 class Commute p => Merge p where
-    merge :: (p :\/: p) C(x y)
-          -> (p :/\: p) C(x y)
+    merge :: (p :\/: p) wX wY
+          -> (p :/\: p) wX wY
+
+
+selfMerger :: Merge p => MergeFn p p
+selfMerger = merge
 
 
 instance Merge p => Merge (FL p) where
@@ -43,8 +51,8 @@ instance Merge p => Merge (RL p) where
 
 
 mergeFL :: Merge p
-        => (p :\/: FL p) C(x y)
-        -> (FL p :/\: p) C(x y)
+        => (p :\/: FL p) wX wY
+        -> (FL p :/\: p) wX wY
 mergeFL (p :\/: NilFL) = NilFL :/\: p
 mergeFL (p :\/: (x :>: xs)) = fromJust $ do
     x' :/\: p' <- return $ merge (p :\/: x)
