@@ -15,25 +15,27 @@
 -- the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 -- Boston, MA 02110-1301, USA.
 
-{-# LANGUAGE CPP, UndecidableInstances #-} -- XXX Undecidable only in GHC < 7
+{-# LANGUAGE CPP #-}
 
-#include "gadts.h"
 
-module Darcs.Patch.PatchInfoAnd ( Hopefully, PatchInfoAnd,
+module Darcs.Patch.PatchInfoAnd ( Hopefully(..), SimpleHopefully(..), PatchInfoAnd(..),
                          WPatchInfo, unWPatchInfo, compareWPatchInfo,
                          piap, n2pia, patchInfoAndPatch,
-                         fmapPIAP, fmapFL_PIAP,
+                         fmapPIAP, fmapFLPIAP,
                          conscientiously, hopefully, info, winfo,
                          hopefullyM, createHashed, extractHash,
                          actually, unavailable, patchDesc ) where
 
 import System.IO.Unsafe ( unsafeInterleaveIO )
 
-import Darcs.SignalHandler ( catchNonSignal )
-import Printer ( Doc, renderString, errorDoc, text, ($$), vcat )
-import Darcs.Patch.Info ( PatchInfo, humanFriendly, justName )
-import Darcs.Patch ( RepoPatch, Named, patch2patchinfo )
+import Darcs.Util.SignalHandler ( catchNonSignal )
+import Darcs.Util.Printer
+    ( Doc, renderString, errorDoc, text, ($$), vcat
+    , RenderMode(..) )
+import Darcs.Patch.Info ( PatchInfo, showPatchInfoUI, justName )
+import Darcs.Patch ( Named, patch2patchinfo )
 import Darcs.Patch.Conflict ( Conflict, CommuteNoConflicts )
+import Darcs.Patch.Debug ( PatchDebug(..) )
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Format ( PatchListFormat )
@@ -42,13 +44,15 @@ import Darcs.Patch.Named ( fmapNamed, fmapFL_Named )
 import Darcs.Patch.Prim ( PrimPatchBase(..) )
 import Darcs.Patch.Patchy ( Patchy, ReadPatch(..), Apply(..), Invert(..),
                             ShowPatch(..), Commute(..), PatchInspect(..) )
+import Darcs.Patch.Rebase.NameHack ( NameHack )
 import Darcs.Patch.Repair ( Repair(..), RepairToFL )
 import Darcs.Patch.Show ( ShowPatchBasic(..) )
-import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
-import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
-import Darcs.Witnesses.Ordered ( (:>)(..), (:\/:)(..), (:/\:)(..), FL, mapFL )
-import Darcs.Witnesses.Sealed ( Sealed(Sealed), seal, mapSeal )
-import Darcs.Utils ( prettyException )
+import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Ordered ( (:>)(..), (:\/:)(..), (:/\:)(..), FL, mapFL )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, mapSeal )
+import Darcs.Patch.Witnesses.Show ( Show1(..), Show2(..), ShowDict(ShowDictClass) )
+import Darcs.Util.Exception ( prettyException )
 import Storage.Hashed.Tree( Tree )
 
 import Control.Applicative ( (<$>) )
@@ -60,35 +64,44 @@ import Control.Applicative ( (<$>) )
 -- 'SimpleHopefully' does the real work of emulating
 -- 'Either'. @Hopefully sh@ represents an expected unhashed patch, and
 -- @Hashed hash sh@ represents an expected hashed patch with its hash.
-data Hopefully a C(x y) = Hopefully (SimpleHopefully a C(x y)) | Hashed String (SimpleHopefully a C(x y))
+data Hopefully a wX wY = Hopefully (SimpleHopefully a wX wY) | Hashed String (SimpleHopefully a wX wY)
+    deriving Show
 
 -- | @SimpleHopefully@ is a variant of @Either String@ adapted for
 -- type witnesses. @Actually@ is the equivalent of @Right@, while
 -- @Unavailable@ is @Left@.
-data SimpleHopefully a C(x y) = Actually (a C(x y)) | Unavailable String
+data SimpleHopefully a wX wY = Actually (a wX wY) | Unavailable String
+    deriving Show
 
--- | @'PatchInfoAnd' p C(a b)@ represents a hope we have to get a
+-- | @'PatchInfoAnd' p wA wB@ represents a hope we have to get a
 -- patch through its info. We're not sure we have the patch, but we
 -- know its info.
-data PatchInfoAnd p C(a b) = PIAP !PatchInfo (Hopefully (Named p) C(a b))
+data PatchInfoAnd p wA wB = PIAP !PatchInfo (Hopefully (Named p) wA wB)
+    deriving Show
+
+instance Show2 p => Show1 (PatchInfoAnd p wX) where
+    showDict1 = ShowDictClass
+
+instance Show2 p => Show2 (PatchInfoAnd p) where
+    showDict2 = ShowDictClass
 
 instance PrimPatchBase p => PrimPatchBase (PatchInfoAnd p) where
    type PrimOf (PatchInfoAnd p) = PrimOf p
 
--- | @'WPatchInfo' C(a b)@ represents the info of a patch, marked with
+-- | @'WPatchInfo' wA wB@ represents the info of a patch, marked with
 -- the patch's witnesses.
-newtype WPatchInfo C(a b) = WPatchInfo { unWPatchInfo :: PatchInfo }
+newtype WPatchInfo wA wB = WPatchInfo { unWPatchInfo :: PatchInfo }
 
 -- This is actually unsafe if we ever commute patches and then compare them
 -- using this function. TODO: consider adding an extra existential to WPatchInfo
 -- (as with TaggedPatch in Darcs.Patch.Choices)
-compareWPatchInfo :: WPatchInfo C(a b) -> WPatchInfo C(c d) -> EqCheck C((a, b) (c, d))
+compareWPatchInfo :: WPatchInfo wA wB -> WPatchInfo wC wD -> EqCheck (wA, wB) (wC, wD)
 compareWPatchInfo (WPatchInfo x) (WPatchInfo y) = if x == y then unsafeCoerceP IsEq else NotEq
 
 instance MyEq WPatchInfo where
    WPatchInfo x `unsafeCompare` WPatchInfo y = x == y
 
-fmapH :: (a C(x y) -> b C(w z)) -> Hopefully a C(x y) -> Hopefully b C(w z)
+fmapH :: (a wX wY -> b wW wZ) -> Hopefully a wX wY -> Hopefully b wW wZ
 fmapH f (Hopefully sh) = Hopefully (ff sh)
     where ff (Actually a) = Actually (f a)
           ff (Unavailable e) = Unavailable e
@@ -96,85 +109,88 @@ fmapH f (Hashed h sh) = Hashed h (ff sh)
     where ff (Actually a) = Actually (f a)
           ff (Unavailable e) = Unavailable e
 
-info :: PatchInfoAnd p C(a b) -> PatchInfo
+info :: PatchInfoAnd p wA wB -> PatchInfo
 info (PIAP i _) = i
 
-patchDesc :: forall p C(x y) . PatchInfoAnd p C(x y) -> String
+patchDesc :: forall p wX wY . PatchInfoAnd p wX wY -> String
 patchDesc p = justName $ info p
 
-winfo :: PatchInfoAnd p C(a b) -> WPatchInfo C(a b)
+winfo :: PatchInfoAnd p wA wB -> WPatchInfo wA wB
 winfo (PIAP i _) = WPatchInfo i
 
 -- | @'piap' i p@ creates a PatchInfoAnd containing p with info i.
-piap :: PatchInfo -> Named p C(a b) -> PatchInfoAnd p C(a b)
+piap :: PatchInfo -> Named p wA wB -> PatchInfoAnd p wA wB
 piap i p = PIAP i (Hopefully $ Actually p)
 
 -- | @n2pia@ creates a PatchInfoAnd representing a @Named@ patch.
-n2pia :: Named p C(x y) -> PatchInfoAnd p C(x y)
+n2pia :: Named p wX wY -> PatchInfoAnd p wX wY
 n2pia x = patch2patchinfo x `piap` x
 
-patchInfoAndPatch :: PatchInfo -> Hopefully (Named p) C(a b) -> PatchInfoAnd p C(a b)
+patchInfoAndPatch :: PatchInfo -> Hopefully (Named p) wA wB -> PatchInfoAnd p wA wB
 patchInfoAndPatch =  PIAP
 
-fmapPIAP :: (FORALL(a b) p C(a b) -> q C(a b)) -> PatchInfoAnd p C(x y) -> PatchInfoAnd q C(x y)
+fmapPIAP :: (forall wA wB . p wA wB -> q wA wB) -> PatchInfoAnd p wX wY -> PatchInfoAnd q wX wY
 fmapPIAP f (PIAP i hp) = PIAP i (fmapH (fmapNamed f) hp)
 
-fmapFL_PIAP :: (FL p C(x y) -> FL q C(x y)) -> PatchInfoAnd p C(x y) -> PatchInfoAnd q C(x y)
-fmapFL_PIAP f (PIAP i hp) = PIAP i (fmapH (fmapFL_Named f) hp)
+fmapFLPIAP :: (FL p wX wY -> FL q wX wY) -> PatchInfoAnd p wX wY -> PatchInfoAnd q wX wY
+fmapFLPIAP f (PIAP i hp) = PIAP i (fmapH (fmapFL_Named f) hp)
 
 -- | @'hopefully' hp@ tries to get a patch from a 'PatchInfoAnd'
 -- value. If it fails, it outputs an error \"failed to read patch:
 -- \<description of the patch>\". We get the description of the patch
 -- from the info part of 'hp'
-hopefully :: PatchInfoAnd p C(a b) -> Named p C(a b)
+hopefully :: PatchInfoAnd p wA wB -> Named p wA wB
 hopefully = conscientiously $ \e -> text "failed to read patch:" $$ e
 
 -- | @'conscientiously' er hp@ tries to extract a patch from a 'PatchInfoAnd'.
 -- If it fails, it applies the error handling function @er@ to a description
 -- of the patch info component of @hp@.
 conscientiously :: (Doc -> Doc)
-                -> PatchInfoAnd p C(a b) -> Named p C(a b)
+                -> PatchInfoAnd p wA wB -> Named p wA wB
 conscientiously er (PIAP pinf hp) =
     case hopefully2either hp of
       Right p -> p
-      Left e -> errorDoc $ er (humanFriendly pinf $$ text e)
+      Left e -> errorDoc $ er (showPatchInfoUI pinf $$ text e)
 
 -- | @hopefullyM@ is a version of @hopefully@ which calls @fail@ in a
 -- monad instead of erroring.
-hopefullyM :: Monad m => PatchInfoAnd p C(a b) -> m (Named p C(a b))
+hopefullyM :: Monad m => PatchInfoAnd p wA wB -> m (Named p wA wB)
 hopefullyM (PIAP pinf hp) = case hopefully2either hp of
                               Right p -> return p
-                              Left e -> fail $ renderString (humanFriendly pinf $$ text e)
+                              Left e -> fail $ renderString Encode
+                                                     (showPatchInfoUI pinf $$ text e)
 
 -- Any recommendations for a nice adverb to name the below?
-hopefully2either :: Hopefully a C(x y) -> Either String (a C(x y))
+hopefully2either :: Hopefully a wX wY -> Either String (a wX wY)
 hopefully2either (Hopefully (Actually p)) = Right p
 hopefully2either (Hashed _ (Actually p)) = Right p
 hopefully2either (Hopefully (Unavailable e)) = Left e
 hopefully2either (Hashed _ (Unavailable e)) = Left e
 
-actually :: a C(x y) -> Hopefully a C(x y)
+actually :: a wX wY -> Hopefully a wX wY
 actually = Hopefully . Actually
 
-createHashed :: String -> (String -> IO (Sealed (a C(x)))) -> IO (Sealed (Hopefully a C(x)))
-createHashed h f = do mapSeal (Hashed h) `fmap` unsafeInterleaveIO (f' `catchNonSignal` handler)
+createHashed :: String -> (String -> IO (Sealed (a wX))) -> IO (Sealed (Hopefully a wX))
+createHashed h f = mapSeal (Hashed h) `fmap` unsafeInterleaveIO (f' `catchNonSignal` handler)
   where
   f' = do Sealed x <- f h
           return (Sealed (Actually x))
   handler e = return $ seal $ Unavailable $ prettyException e
 
-extractHash :: PatchInfoAnd p C(a b) -> Either (Named p C(a b)) String
+extractHash :: PatchInfoAnd p wA wB -> Either (Named p wA wB) String
 extractHash (PIAP _ (Hashed s _)) = Right s
 extractHash hp = Left $ conscientiously (\e -> text "unable to read patch:" $$ e) hp
 
-unavailable :: String -> Hopefully a C(x y)
+unavailable :: String -> Hopefully a wX wY
 unavailable = Hopefully . Unavailable
 
-instance (Commute p, MyEq p) => MyEq (PatchInfoAnd p) where
+-- Equality on PatchInfoAnd is solely determined by the PatchInfo
+-- It is a global invariant of darcs that once a patch is recorded,
+-- it should always have the same representation in the same context.
+instance MyEq (PatchInfoAnd p) where
     unsafeCompare (PIAP i _) (PIAP i2 _) = i == i2
 
---instance Invert (p C(x y)) => Invert (PatchInfoAnd (p C(x y))) where
-instance (Commute p, Invert p) => Invert (PatchInfoAnd p) where
+instance Invert p => Invert (PatchInfoAnd p) where
     invert (PIAP i p) = PIAP i (invert `fmapH` p)
 
 instance PatchListFormat (PatchInfoAnd p)
@@ -182,27 +198,27 @@ instance PatchListFormat (PatchInfoAnd p)
 instance (PatchListFormat p, ShowPatchBasic p) => ShowPatchBasic (PatchInfoAnd p) where
     showPatch (PIAP n p) = case hopefully2either p of
                            Right x -> showPatch x
-                           Left _ -> humanFriendly n
+                           Left _ -> showPatchInfoUI n
 
 instance (Apply p, Conflict p, CommuteNoConflicts p, IsHunk p, PatchListFormat p, PrimPatchBase p,
           ShowPatch p, ApplyState p ~ Tree) => ShowPatch (PatchInfoAnd p) where
     showContextPatch (PIAP n p) = case hopefully2either p of
                                     Right x -> showContextPatch x
-                                    Left _ -> return $ humanFriendly n
-    description (PIAP n _) = humanFriendly n
+                                    Left _ -> return $ showPatchInfoUI n
+    description (PIAP n _) = showPatchInfoUI n
     summary (PIAP n p) = case hopefully2either p of
                          Right x -> summary x
-                         Left _ -> humanFriendly n
+                         Left _ -> showPatchInfoUI n
     summaryFL = vcat . mapFL summary
     showNicely (PIAP n p) = case hopefully2either p of
                             Right x -> showNicely x
-                            Left _ -> humanFriendly n
+                            Left _ -> showPatchInfoUI n
 
-instance Commute p => Commute (PatchInfoAnd p) where
+instance (Commute p, NameHack p) => Commute (PatchInfoAnd p) where
     commute (x :> y) = do y' :> x' <- commute (hopefully x :> hopefully y)
                           return $ (info y `piap` y') :> (info x `piap` x')
 
-instance Merge p => Merge (PatchInfoAnd p) where
+instance (Merge p, NameHack p) => Merge (PatchInfoAnd p) where
     merge (x :\/: y) = case merge (hopefully x :\/: hopefully y) of
                        y' :/\: x' -> (info y `piap` y') :/\: (info x `piap` x')
 
@@ -230,4 +246,7 @@ instance Effect p => Effect (PatchInfoAnd p) where
 instance IsHunk (PatchInfoAnd p) where
     isHunk _ = Nothing
 
-instance (RepoPatch p, ApplyState p ~ Tree) => Patchy (PatchInfoAnd p)
+instance PatchDebug p => PatchDebug (PatchInfoAnd p)
+
+instance (Patchy p, NameHack p, ApplyState p ~ Tree) => Patchy (PatchInfoAnd p)
+
