@@ -19,55 +19,55 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 {-# LANGUAGE CPP #-}
 
-#include "gadts.h"
 
 module Darcs.Patch.Permutations ( removeFL, removeRL, removeCommon,
                                   commuteWhatWeCanFL, commuteWhatWeCanRL,
-                                  genCommuteWhatWeCanRL,
+                                  genCommuteWhatWeCanRL, genCommuteWhatWeCanFL,
                                   partitionFL, partitionRL,
                                   simpleHeadPermutationsFL, headPermutationsRL,
                                   headPermutationsFL,
                                   removeSubsequenceFL, removeSubsequenceRL,
                                   partitionConflictingFL,
-                                  CommuteFn, selfCommuter,
-                                  commuterIdFL, commuterFLId,
-                                  commuterIdRL
+                                  inverseCommuter
                                 ) where
 
 import Data.Maybe ( mapMaybe )
 import Darcs.Patch.Commute ( Commute, commute, commuteFLorComplain, commuteRL )
-import Darcs.Patch.Invert ( Invert(..), invertFL, invertRL )
-import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
-import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), (+<+)
-                               , reverseFL, (+>+), (:\/:)(..), lengthFL
-                               , lengthRL, reverseRL )
+import Darcs.Patch.CommuteFn ( CommuteFn )
+import Darcs.Patch.Invert ( Invert(..) )
+import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Ordered
+    ( FL(..), RL(..), (:>)(..), (+<+)
+    , reverseFL, (+>+), (:\/:)(..), lengthFL
+    , lengthRL, reverseRL )
 #include "impossible.h"
 
 -- |split an 'FL' into "left" and "right" lists according to a predicate @p@, using commutation as necessary.
 -- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
 -- the predicate, it goes in the "middle" list; to sum up, we have: @all p left@ and @all (not.p) right@, while
 -- midddle is mixed.
+-- Note that @p@ should be invariant under commutation (i.e. if 'x1' can commute to 'x2' then 'p x1 <=> p x2').
 partitionFL :: Commute p
-            => (FORALL(u v) p C(u v) -> Bool)       -- ^predicate; if true we would like the patch in the "left" list
-            -> FL p C(x y)                          -- ^input 'FL'
-            -> ((FL p :> FL p :> FL p) C(x y))      -- ^"left", "middle" and "right"
+            => (forall wU wV . p wU wV -> Bool)  -- ^predicate; if true we would like the patch in the "left" list
+            -> FL p wX wY                        -- ^input 'FL'
+            -> (FL p :> FL p :> FL p) wX wY      -- ^"left", "middle" and "right"
 
 -- optimise by using an accumulating parameter to track all the "right" patches that we've found so far
 partitionFL' :: Commute p
-             => (FORALL(u v) p C(u v) -> Bool)
-             -> RL p C(a b)  -- the "middle" patches found so far
-             -> RL p C(b c)  -- the "right" patches found so far
-             -> FL p C(c d)
-             -> ((FL p :> FL p :> FL p) C(a d))
+             => (forall wU wV . p wU wV -> Bool)
+             -> RL p wA wB  -- the "middle" patches found so far
+             -> RL p wB wC  -- the "right" patches found so far
+             -> FL p wC wD
+             -> (FL p :> FL p :> FL p) wA wD
 
-partitionFL keepleft ps = partitionFL' keepleft NilRL NilRL ps
+partitionFL keepleft = partitionFL' keepleft NilRL NilRL
 
-partitionFL' _ middle right NilFL = (NilFL :> reverseRL middle :> reverseRL right)
+partitionFL' _ middle right NilFL = NilFL :> reverseRL middle :> reverseRL right
 partitionFL' keepleft middle right (p :>: ps)
    | keepleft p = case commuteRL (right :> p) of
      Just (p' :> right') -> case commuteRL (middle :> p') of
        Just (p'' :> middle') -> case partitionFL' keepleft middle' right' ps of
-         (a :> b :> c) -> (p'' :>: a :> b :> c)
+         (a :> b :> c) -> p'' :>: a :> b :> c
        Nothing -> partitionFL' keepleft (p' :<: middle) right' ps
      Nothing -> case commuteWhatWeCanRL (right :> p) of
        (tomiddle :> p' :> right') -> partitionFL' keepleft (p' :<: tomiddle +<+ middle) right' ps
@@ -78,16 +78,16 @@ partitionFL' keepleft middle right (p :>: ps)
 -- If a patch does satisfy the predicate but cannot be commuted past one that does not satisfy
 -- the predicate, it goes in the "left" list.
 partitionRL :: Commute p
-            => (FORALL(u v) p C(u v) -> Bool)    -- ^predicate; if true we would like the patch in the "right" list
-            -> RL p C(x y)                       -- ^input 'RL'
-            -> (RL p :> RL p) C(x y)             -- ^"left" and "right" results
+            => (forall wU wV . p wU wV -> Bool)    -- ^predicate; if true we would like the patch in the "right" list
+            -> RL p wX wY                       -- ^input 'RL'
+            -> (RL p :> RL p) wX wY             -- ^"left" and "right" results
 
 -- optimise by using an accumulating parameter to track all the "left" patches that we've found so far
 partitionRL' :: Commute p
-             => (FORALL(u v) p C(u v) -> Bool)
-             -> RL p C(x z)
-             -> FL p C(z y)   -- the "left" patches found so far
-             -> (RL p :> RL p) C(x y)
+             => (forall wU wV . p wU wV -> Bool)
+             -> RL p wX wZ
+             -> FL p wZ wY   -- the "left" patches found so far
+             -> (RL p :> RL p) wX wY
 
 partitionRL keepright ps = partitionRL' keepright ps NilFL
 
@@ -100,24 +100,30 @@ partitionRL' keepright (p :<: ps) qs
          a :> b -> a :> p' :<: b
    | otherwise = partitionRL' keepright ps (p :>: qs)
 
-commuteWhatWeCanFL :: Commute p => (p :> FL p) C(x y) -> (FL p :> p :> FL p) C(x y)
-commuteWhatWeCanFL (p :> x :>: xs) =
-    case commute (p :> x) of
-    Nothing -> case commuteWhatWeCanFL (x :> xs) of
-               xs1 :> x' :> xs2 -> case commuteWhatWeCanFL (p :> xs1) of
-                              xs1' :> p' :> xs2' -> xs1' :> p' :> xs2' +>+ x' :>: xs2
-    Just (x' :> p') -> case commuteWhatWeCanFL (p' :> xs) of
-                       a :> p'' :> c -> x' :>: a :> p'' :> c
-commuteWhatWeCanFL (y :> NilFL) = NilFL :> y :> NilFL
+commuteWhatWeCanFL :: Commute p => (p :> FL p) wX wY -> (FL p :> p :> FL p) wX wY
+commuteWhatWeCanFL = genCommuteWhatWeCanFL commute
 
-commuteWhatWeCanRL :: Commute p => (RL p :> p) C(x y) -> (RL p :> p :> RL p) C(x y)
+genCommuteWhatWeCanFL :: Commute q =>
+                         (forall wA wB . ((p:>q) wA wB -> Maybe ((q:>p)wA wB)))
+                          -> (p :> FL q) wX wY -> (FL q :> p :> FL q) wX wY
+genCommuteWhatWeCanFL com (p :> x :>: xs) =
+    case com (p :> x) of
+    Nothing -> case commuteWhatWeCanFL (x :> xs) of
+               xs1 :> x' :> xs2 -> case genCommuteWhatWeCanFL com (p :> xs1) of
+                              xs1' :> p' :> xs2' -> xs1' :> p' :> xs2' +>+ x' :>: xs2
+    Just (x' :> p') -> case genCommuteWhatWeCanFL com (p' :> xs) of
+                       a :> p'' :> c -> x' :>: a :> p'' :> c
+genCommuteWhatWeCanFL _ (y :> NilFL) = NilFL :> y :> NilFL
+
+commuteWhatWeCanRL :: Commute p => (RL p :> p) wX wY -> (RL p :> p :> RL p) wX wY
 commuteWhatWeCanRL = genCommuteWhatWeCanRL commute
 
-genCommuteWhatWeCanRL :: (FORALL(a b) ((p :> p) C(a b) -> Maybe ((p :> p) C(a b))))
-                      -> (RL p :> p) C(x y) -> (RL p :> p :> RL p) C(x y)
+genCommuteWhatWeCanRL :: Commute p =>
+                         (forall wA wB . ((p :> q) wA wB -> Maybe ((q :> p) wA wB)))
+                         -> (RL p :> q) wX wY -> (RL p :> q :> RL p) wX wY
 genCommuteWhatWeCanRL com (x :<: xs :> p) =
     case com (x :> p) of
-    Nothing -> case genCommuteWhatWeCanRL com (xs :> x) of
+    Nothing -> case commuteWhatWeCanRL (xs :> x) of
                xs1 :> x' :> xs2 -> case genCommuteWhatWeCanRL com (xs2 :> p) of
                               xs1' :> p' :> xs2' -> xs1' +<+ x' :<: xs1 :> p' :> xs2'
     Just (p' :> x') -> case genCommuteWhatWeCanRL com (xs :> p') of
@@ -125,11 +131,11 @@ genCommuteWhatWeCanRL com (x :<: xs :> p) =
 genCommuteWhatWeCanRL _ (NilRL :> y) = NilRL :> y :> NilRL
 
 
-removeCommon :: (MyEq p, Commute p) => (FL p :\/: FL p) C(x y) -> (FL p :\/: FL p) C(x y)
+removeCommon :: (MyEq p, Commute p) => (FL p :\/: FL p) wX wY -> (FL p :\/: FL p) wX wY
 removeCommon (xs :\/: NilFL) = xs :\/: NilFL
 removeCommon (NilFL :\/: xs) = NilFL :\/: xs
 removeCommon (xs :\/: ys) = rc xs (headPermutationsFL ys)
-    where rc :: (MyEq p, Commute p) => FL p C(x y) -> [(p:>FL p) C(x z)] -> (FL p :\/: FL p) C(y z)
+    where rc :: (MyEq p, Commute p) => FL p wX wY -> [(p:>FL p) wX wZ] -> (FL p :\/: FL p) wY wZ
           rc nms ((n:>ns):_) | Just ms <- removeFL n nms = removeCommon (ms :\/: ns)
           rc ms [n:>ns] = ms :\/: n:>:ns
           rc ms (_:nss) = rc ms nss
@@ -137,17 +143,17 @@ removeCommon (xs :\/: ys) = rc xs (headPermutationsFL ys)
 
 -- | 'removeFL' @x xs@ removes @x@ from @xs@ if @x@ can be commuted to its head.
 --   Otherwise it returns 'Nothing'
-removeFL :: (MyEq p, Commute p) => p C(x y) -> FL p C(x z) -> Maybe (FL p C(y z))
+removeFL :: (MyEq p, Commute p) => p wX wY -> FL p wX wZ -> Maybe (FL p wY wZ)
 removeFL x xs = r x $ headPermutationsFL xs
-    where r :: (MyEq p, Commute p) => p C(x y) -> [(p:>FL p) C(x z)] -> Maybe (FL p C(y z))
+    where r :: (MyEq p, Commute p) => p wX wY -> [(p:>FL p) wX wZ] -> Maybe (FL p wY wZ)
           r _ [] = Nothing
           r z ((z':>zs):zss) | IsEq <- z =\/= z' = Just zs
                              | otherwise = r z zss
 
 -- | 'removeRL' is like 'removeFL' except with 'RL'
-removeRL :: (MyEq p, Commute p) => p C(y z) -> RL p C(x z) -> Maybe (RL p C(x y))
+removeRL :: (MyEq p, Commute p) => p wY wZ -> RL p wX wZ -> Maybe (RL p wX wY)
 removeRL x xs = r x $ headPermutationsRL xs
-    where r :: (MyEq p, Commute p) => p C(y z) -> [RL p C(x z)] -> Maybe (RL p C(x y))
+    where r :: (MyEq p, Commute p) => p wY wZ -> [RL p wX wZ] -> Maybe (RL p wX wY)
           r z ((z':<:zs):zss) | IsEq <- z =/\= z' = Just zs
                               | otherwise = r z zss
           r _ _ = Nothing
@@ -156,27 +162,27 @@ removeRL x xs = r x $ headPermutationsRL xs
 --   @ab@ have been commuted out of it, if possible.  If this is not possible
 --   for any reason (the set of patches @ab@ is not actually a subset of @abc@,
 --   or they can't be commuted out) we return 'Nothing'.
-removeSubsequenceFL :: (MyEq p, Commute p) => FL p C(a b)
-                     -> FL p C(a c) -> Maybe (FL p C(b c))
+removeSubsequenceFL :: (MyEq p, Commute p) => FL p wA wB
+                     -> FL p wA wC -> Maybe (FL p wB wC)
 removeSubsequenceFL a b | lengthFL a > lengthFL b = Nothing
                          | otherwise = rsFL a b
-    where rsFL :: (MyEq p, Commute p) => FL p C(a b) -> FL p C(a c) -> Maybe (FL p C(b c))
+    where rsFL :: (MyEq p, Commute p) => FL p wA wB -> FL p wA wC -> Maybe (FL p wB wC)
           rsFL NilFL ys = Just ys
           rsFL (x:>:xs) yys = removeFL x yys >>= removeSubsequenceFL xs
 
 -- | 'removeSubsequenceRL' is like @removeSubsequenceFL@ except that it works
 --   on 'RL'
-removeSubsequenceRL :: (MyEq p, Commute p) => RL p C(ab abc)
-                     -> RL p C(a abc) -> Maybe (RL p C(a ab))
+removeSubsequenceRL :: (MyEq p, Commute p) => RL p wAb wAbc
+                     -> RL p wA wAbc -> Maybe (RL p wA wAb)
 removeSubsequenceRL a b | lengthRL a > lengthRL b = Nothing
                          | otherwise = rsRL a b
-    where rsRL :: (MyEq p, Commute p) => RL p C(ab abc) -> RL p C(a abc) -> Maybe (RL p C(a ab))
+    where rsRL :: (MyEq p, Commute p) => RL p wAb wAbc -> RL p wA wAbc -> Maybe (RL p wA wAb)
           rsRL NilRL ys = Just ys
           rsRL (x:<:xs) yys = removeRL x yys >>= removeSubsequenceRL xs
 
 -- | This is a minor variant of 'headPermutationsFL' with each permutation
 --   is simply returned as a 'FL'
-simpleHeadPermutationsFL :: Commute p => FL p C(x y) -> [FL p C(x y)]
+simpleHeadPermutationsFL :: Commute p => FL p wX wY -> [FL p wX wY]
 simpleHeadPermutationsFL ps = map (\ (x:>xs) -> x:>:xs) $ headPermutationsFL ps
 
 -- | 'headPermutationsFL' @p:>:ps@ returns all the permutations of the list
@@ -195,7 +201,7 @@ simpleHeadPermutationsFL ps = map (\ (x:>xs) -> x:>:xs) $ headPermutationsFL ps
 --   > y :> X h a s t c k
 --   > s :> X h a y t c k
 --   > k :> X h a y s t c
-headPermutationsFL :: Commute p => FL p C(x y) -> [(p :> FL p) C(x y)]
+headPermutationsFL :: Commute p => FL p wX wY -> [(p :> FL p) wX wY]
 headPermutationsFL NilFL = []
 headPermutationsFL (p:>:ps) =
     (p:>ps) : mapMaybe (swapfirstFL.(p:>)) (headPermutationsFL ps)
@@ -205,7 +211,7 @@ headPermutationsFL (p:>:ps) =
 -- | 'headPermutationsRL' is like 'headPermutationsFL', except that we
 --   operate on an 'RL' (in other words, we are pushing things to the end of a
 --   patch sequence instead of to the beginning).
-headPermutationsRL :: Commute p => RL p C(x y) -> [RL p C(x y)]
+headPermutationsRL :: Commute p => RL p wX wY -> [RL p wX wY]
 headPermutationsRL NilRL = []
 headPermutationsRL (p:<:ps) =
     (p:<:ps) : mapMaybe (swapfirstRL.(p:<:)) (headPermutationsRL ps)
@@ -216,66 +222,25 @@ headPermutationsRL (p:<:ps) =
 instance (MyEq p, Commute p) => MyEq (FL p) where
     a =\/= b | lengthFL a /= lengthFL b = NotEq
              | otherwise = cmpSameLength a b
-             where cmpSameLength :: FL p C(x y) -> FL p C(x z) -> EqCheck C(y z)
+             where cmpSameLength :: FL p wX wY -> FL p wX wZ -> EqCheck wY wZ
                    cmpSameLength (x:>:xs) xys | Just ys <- removeFL x xys = cmpSameLength xs ys
                    cmpSameLength NilFL NilFL = IsEq
                    cmpSameLength _ _ = NotEq
     xs =/\= ys = reverseFL xs =/\= reverseFL ys
 
-instance (Invert p, Commute p) => Invert (FL p) where
-    invert = reverseRL . invertFL
-
 instance (MyEq p, Commute p) => MyEq (RL p) where
     unsafeCompare = bug "Buggy use of unsafeCompare on RL"
     a =/\= b | lengthRL a /= lengthRL b = NotEq
              | otherwise = cmpSameLength a b
-             where cmpSameLength :: RL p C(x y) -> RL p C(w y) -> EqCheck C(x w)
+             where cmpSameLength :: RL p wX wY -> RL p wW wY -> EqCheck wX wW
                    cmpSameLength (x:<:xs) xys | Just ys <- removeRL x xys = cmpSameLength xs ys
                    cmpSameLength NilRL NilRL = IsEq
                    cmpSameLength _ _ = NotEq
     xs =\/= ys = reverseRL xs =\/= reverseRL ys
 
-instance (Commute p, Invert p) => Invert (RL p) where
-    invert = reverseFL . invertRL
-
--- |CommuteFn is the basis of a general framework for building up commutation
--- operations between different patch types in a generic manner. Unfortunately
--- type classes are not well suited to the problem because of the multiple possible
--- routes by which the commuter for (FL p1, FL p2) can be built out of the
--- commuter for (p1, p2) - and more complicated problems when we start building
--- multiple constructors on top of each other. The type class resolution machinery
--- really can't cope with selecting some route, because it doesn't know that all
--- possible routes should be equivalent.
-type CommuteFn p1 p2 = FORALL(x y) (p1 :> p2) C(x y) -> Maybe ((p2 :> p1) C(x y))
-
--- |Build a commuter between a patch and itself using the operation from the type class.
-selfCommuter :: Commute p => CommuteFn p p
-selfCommuter = commute
-
-commuterIdRL :: CommuteFn p1 p2 -> CommuteFn p1 (RL p2)
-commuterIdRL _ (x :> NilRL) = return (NilRL :> x)
-commuterIdRL commuter (x :> (y :<: ys))
-  = do ys' :> x' <- commuterIdRL commuter (x :> ys)
-       y' :> x'' <- commuter (x' :> y)
-       return ((y' :<: ys') :> x'')
-
-commuterIdFL :: CommuteFn p1 p2 -> CommuteFn p1 (FL p2)
-commuterIdFL _ (x :> NilFL) = return (NilFL :> x)
-commuterIdFL commuter (x :> (y :>: ys))
-  = do y' :> x' <- commuter (x :> y)
-       ys' :> x'' <- commuterIdFL commuter (x' :> ys)
-       return ((y' :>: ys') :> x'')
-
-commuterFLId :: CommuteFn p1 p2 -> CommuteFn (FL p1) p2
-commuterFLId _ (NilFL :> y) = return (y :> NilFL)
-commuterFLId commuter ((x :>: xs) :> y)
-  = do y' :> xs' <- commuterFLId commuter (xs :> y)
-       y'' :> x' <- commuter (x :> y')
-       return (y'' :> (x' :>: xs'))
-
--- |Partition a list into the patches that commute with the given patch and those that don't (including dependencies)
-partitionConflictingFL :: (Commute p1, Invert p1) => CommuteFn p1 p2 -> FL p1 C(x y) -> p2 C(x z) -> (FL p1 :> FL p1) C(x y)
-partitionConflictingFL _ NilFL _ = (NilFL :> NilFL)
+-- |Partition a list into the patches that merge with the given patch and those that don't (including dependencies)
+partitionConflictingFL :: (Commute p1, Invert p1) => CommuteFn p1 p2 -> FL p1 wX wY -> p2 wX wZ -> (FL p1 :> FL p1) wX wY
+partitionConflictingFL _ NilFL _ = NilFL :> NilFL
 partitionConflictingFL commuter (x :>: xs) y =
    case commuter (invert x :> y) of
      Nothing -> case commuteWhatWeCanFL (x :> xs) of
@@ -285,3 +250,9 @@ partitionConflictingFL commuter (x :>: xs) y =
      Just (y' :> _) ->
        case partitionConflictingFL commuter xs y' of
           xs_clean :> xs_conflicts -> (x :>: xs_clean) :> xs_conflicts
+
+inverseCommuter :: (Invert p, Invert q) => CommuteFn p q -> CommuteFn q p
+inverseCommuter commuter (p :> q) =
+   do invp' :> invq' <- commuter (invert q :> invert p)
+      return (invert invq' :> invert invp')
+
