@@ -1,4 +1,5 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# LANGUAGE CPP #-}
 module Darcs.Patch.Prim.V1.Commute
     ( Perhaps(..)
     , subcommutes, WrappedCommuteFunction(..)
@@ -7,13 +8,14 @@ module Darcs.Patch.Prim.V1.Commute
 
 import Prelude hiding ( pi )
 import Control.Monad ( MonadPlus, msum, mzero, mplus )
+import Control.Applicative ( Applicative(..), Alternative(..) )
 
 import qualified Data.ByteString as B (ByteString, concat)
 import qualified Data.ByteString.Char8 as BC (pack)
 
-import Darcs.Patch.FileName ( FileName, fn2fp, movedirfilename )
-import Darcs.Witnesses.Ordered ( (:<)(..) )
-import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Util.Path ( FileName, fn2fp, movedirfilename )
+import Darcs.Patch.Witnesses.Ordered ( (:<)(..) )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 import Darcs.Patch.Prim.V1.Core
      ( Prim(..), FilePatchType(..) )
 import Darcs.Patch.Invert ( Invert(..) )
@@ -22,7 +24,6 @@ import Darcs.Patch.Permutations () -- for Invert instance of FL
 import Darcs.Patch.TokenReplace ( tryTokInternal )
 
 #include "impossible.h"
-#include "gadts.h"
 
 isInDirectory :: FileName -> FileName -> Bool
 isInDirectory d f = iid (fn2fp d) (fn2fp f)
@@ -35,21 +36,35 @@ isInDirectory d f = iid (fn2fp d) (fn2fp f)
 
 data Perhaps a = Unknown | Failed | Succeeded a
 
+instance Functor Perhaps where
+    fmap _ Unknown = Unknown
+    fmap _ Failed = Failed
+    fmap f (Succeeded x) = Succeeded (f x)
+
+instance Applicative Perhaps where
+    pure = Succeeded
+    _ <*> Failed = Failed
+    _ <*> Unknown = Unknown
+    Failed <*> _ = Failed
+    Unknown <*> _ = Unknown
+    Succeeded f <*> Succeeded x = Succeeded (f x)
+
 instance  Monad Perhaps where
     (Succeeded x) >>= k =  k x
     Failed   >>= _      =  Failed
     Unknown  >>= _      =  Unknown
-    Failed   >> _       =  Failed
-    (Succeeded _) >> k  =  k
-    Unknown  >> k       =  k
     return              =  Succeeded
     fail _              =  Unknown
 
+instance Alternative Perhaps where
+    empty = Unknown
+    Unknown <|> ys    = ys
+    Failed  <|> _     = Failed
+    (Succeeded x) <|> _ = Succeeded x
+
 instance  MonadPlus Perhaps where
-    mzero                 = Unknown
-    Unknown `mplus` ys    = ys
-    Failed  `mplus` _     = Failed
-    (Succeeded x) `mplus` _ = Succeeded x
+    mzero = Unknown
+    mplus = (<|>)
 
 toMaybe :: Perhaps a -> Maybe a
 toMaybe (Succeeded x) = Just x
@@ -81,7 +96,7 @@ speedyCommute (p2@(FP f' _) :< p1@(FP f _))
 speedyCommute _other = Unknown
 
 everythingElseCommute :: CommuteFunction
-everythingElseCommute x = eec x
+everythingElseCommute = eec
     where
     eec :: CommuteFunction
     eec (ChangePref p f t :<p1) = Succeeded (unsafeCoerceP p1 :< ChangePref p f t)
@@ -114,8 +129,7 @@ commuteFiledir (FP f1 p1 :< FP f2 p2) =
   if f1 /= f2 then Succeeded ( FP f2 (unsafeCoerceP p2) :< FP f1 (unsafeCoerceP p1) )
   else commuteFP f1 (p1 :< p2)
 commuteFiledir (DP d1 p1 :< DP d2 p2) =
-  if (not $ isInDirectory d1 d2) && (not $ isInDirectory d2 d1) &&
-     d1 /= d2
+  if not (isInDirectory d1 d2 || isInDirectory d2 d1) && d1 /= d2
   then Succeeded ( DP d2 (unsafeCoerceP p2) :< DP d1 (unsafeCoerceP p1) )
   else Failed
 commuteFiledir (DP d dp :< FP f fp) =
@@ -144,7 +158,7 @@ commuteFiledir (Move d d' :< Move f f')
 
 commuteFiledir _ = Unknown
 
-type CommuteFunction = FORALL(x y) (Prim :< Prim) C(x y) -> Perhaps ((Prim :< Prim) C(x y))
+type CommuteFunction = forall wX wY . (Prim :< Prim) wX wY -> Perhaps ((Prim :< Prim) wX wY)
 newtype WrappedCommuteFunction = WrappedCommuteFunction { runWrappedCommuteFunction :: CommuteFunction }
 
 subcommutes :: [(String, WrappedCommuteFunction)]
@@ -159,8 +173,8 @@ commuteFilepatches :: CommuteFunction
 commuteFilepatches (FP f1 p1 :< FP f2 p2) | f1 == f2 = commuteFP f1 (p1 :< p2)
 commuteFilepatches _ = Unknown
 
-commuteFP :: FileName -> (FilePatchType :< FilePatchType) C(x y)
-          -> Perhaps ((Prim :< Prim) C(x y))
+commuteFP :: FileName -> (FilePatchType :< FilePatchType) wX wY
+          -> Perhaps ((Prim :< Prim) wX wY)
 commuteFP f (Hunk line1 [] [] :< p2) =
     seq f $ Succeeded (FP f (unsafeCoerceP p2) :< FP f (Hunk line1 [] []))
 commuteFP f (p2 :< Hunk line1 [] []) =
@@ -185,8 +199,8 @@ commuteFP f (TokReplace t o n :< TokReplace t2 o2 n2)
                              FP f (TokReplace t o n))
 commuteFP _ _ = Unknown
 
-commuteHunk :: FileName -> (FilePatchType :< FilePatchType) C(x y)
-            -> Maybe ((Prim :< Prim) C(x y))
+commuteHunk :: FileName -> (FilePatchType :< FilePatchType) wX wY
+            -> Maybe ((Prim :< Prim) wX wY)
 commuteHunk f (Hunk line2 old2 new2 :< Hunk line1 old1 new1)
   | seq f $ line1 + lengthnew1 < line2 =
       Just (FP f (Hunk line1 old1 new1) :<
@@ -211,5 +225,5 @@ commuteHunk _ _ = impossible
 
 tryTokReplace :: String -> String -> String
                 -> [B.ByteString] -> Maybe [B.ByteString]
-tryTokReplace t o n mss =
-  mapM (fmap B.concat . tryTokInternal t (BC.pack o) (BC.pack n)) mss
+tryTokReplace t o n =
+  mapM (fmap B.concat . tryTokInternal t (BC.pack o) (BC.pack n))
