@@ -6,7 +6,7 @@ import Darcs.Patch.Prim.V1.Core
     ( Prim(..),
       DirPatchType(..), FilePatchType(..) )
 
-import Darcs.Patch.FileName ( fn2fp )
+import Darcs.Util.Path ( fn2fp )
 import Darcs.Patch.Format ( FileNameFormat(..) )
 import Darcs.Patch.Read ( ReadPatch(..), readFileName )
 import Darcs.Patch.ReadMonads (ParserM, takeTillChar,
@@ -15,15 +15,14 @@ import Darcs.Patch.ReadMonads (ParserM, takeTillChar,
                                anyChar, char, myLex',
                                skipSpace, skipWhile, linesStartingWith)
 
-import Darcs.Witnesses.Sealed ( seal )
+import Darcs.Patch.Witnesses.Sealed ( seal )
 
-import ByteStringUtils ( fromHex2PS )
+import Darcs.Util.ByteString ( fromHex2PS )
 
 import Control.Monad ( liftM )
 import qualified Data.ByteString       as B  ( ByteString, init, tail, concat )
 import qualified Data.ByteString.Char8 as BC ( unpack, pack )
 
-#include "gadts.h"
 
 instance ReadPatch Prim where
  readPatch' = readPrim OldFormat
@@ -39,7 +38,7 @@ instance PrimRead Prim where
        , return' $ readRmDir x
        , return' $ readTok x
        , return' $ readBinary x
-       , return' $ readChangePref
+       , return' readChangePref
        ]
     where
     return'  = liftM seal
@@ -71,7 +70,7 @@ move = BC.pack "move"
 changepref :: B.ByteString
 changepref = BC.pack "changepref"
 
-readHunk :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readHunk :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readHunk x = do
   string hunk'
   fi <- myLex'
@@ -88,7 +87,7 @@ readHunk x = do
 skipNewline :: ParserM m => m Bool
 skipNewline = option False (char '\n' >> return True)
 
-readTok :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readTok :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readTok x = do
   string replace
   f <- myLex'
@@ -111,7 +110,7 @@ readTok x = do
 -- > newhex
 -- > *HEXHEXHEX
 -- > ...
-readBinary :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readBinary :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readBinary x = do
   string binary'
   fi <- myLex'
@@ -125,23 +124,23 @@ readBinary x = do
                   (fromHex2PS $ B.concat old)
                   (fromHex2PS $ B.concat new)
 
-readAddFile :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readAddFile :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readAddFile x = do string addfile
                    f <- myLex'
                    return $ FP (readFileName x f) AddFile
 
-readRmFile :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readRmFile :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readRmFile x = do string rmfile
                   f <- myLex'
                   return $ FP (readFileName x f) RmFile
 
-readMove :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readMove :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readMove x = do string move
                 d <- myLex'
                 d' <- myLex'
                 return $ Move (readFileName x d) (readFileName x d')
 
-readChangePref :: ParserM m => m (Prim C(x y))
+readChangePref :: ParserM m => m (Prim wX wY)
 readChangePref
  = do string changepref
       p <- myLex'
@@ -152,12 +151,12 @@ readChangePref
       t <- takeTillChar '\n'
       return $ ChangePref (BC.unpack p) (BC.unpack f) (BC.unpack t)
 
-readAddDir :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readAddDir :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readAddDir x = do string adddir
                   f <- myLex'
                   return $ DP (readFileName x f) AddDir
 
-readRmDir :: ParserM m => FileNameFormat -> m (Prim C(x y))
+readRmDir :: ParserM m => FileNameFormat -> m (Prim wX wY)
 readRmDir x = do string rmdir
                  f <- myLex'
                  return $ DP (readFileName x f) RmDir
