@@ -2,17 +2,14 @@
 module Darcs.Patch.Prim.V3.Coalesce () where
 
 import Darcs.Patch.Prim.Class ( PrimCanonize(..) )
-import Darcs.Witnesses.Ordered( FL(..) )
+import Darcs.Patch.Witnesses.Ordered( FL(..) )
 import Darcs.Patch.Prim.V3.Core( Prim )
 
-#include "gadts.h"
-#include "impossible.h"
-
 -- TODO
 instance PrimCanonize Prim where
    tryToShrink = error "tryToShrink"
    tryShrinkingInverse _ = error "tryShrinkingInverse"
    sortCoalesceFL = id
-   canonize = (:>: NilFL)
-   canonizeFL = id
-   join = const Nothing
+   canonize _ = (:>: NilFL)
+   canonizeFL _ = id
+   coalesce = const Nothing
