@@ -25,37 +25,36 @@ module Darcs.Patch.Read ( ReadPatch(..),
 
 import Prelude hiding ( pi )
 
-import ByteStringUtils ( dropSpace )
+import Darcs.Util.ByteString ( dropSpace )
 import qualified Data.ByteString       as B  (ByteString, null)
 
 import Darcs.Patch.Bracketed ( Bracketed(..), unBracketedFL )
-import Darcs.Patch.FileName ( FileName, fp2fn, ps2fn, decodeWhite )
+import Darcs.Util.Path ( FileName, fp2fn, ps2fn, decodeWhite )
 import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..), FileNameFormat(..) )
 import Darcs.Patch.ReadMonads (ParserM,
                                parseStrictly,
                                choice, lexChar, lexString,
                                checkConsumes )
 
-import Darcs.Witnesses.Ordered ( FL(..), RL, reverseFL )
-import Darcs.Witnesses.Sealed ( Sealed(..), mapSeal )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), RL, reverseFL )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(..), mapSeal )
 
 import Control.Applicative ( (<$>), (<|>) )
 import Control.Monad ( mzero )
 import qualified Data.ByteString.Char8 as BC ( ByteString, unpack )
 
-#include "gadts.h"
 
 class ReadPatch p where
     readPatch'
-        :: ParserM m => m (Sealed (p C(x )))
+        :: ParserM m => m (Sealed (p wX))
 
-readPatchPartial :: ReadPatch p => B.ByteString -> Maybe (Sealed (p C(x )), B.ByteString)
+readPatchPartial :: ReadPatch p => B.ByteString -> Maybe (Sealed (p wX), B.ByteString)
 readPatchPartial ps
     = case parseStrictly readPatch' ps of
          Just (p, ps') -> Just (p, ps')
          _ -> Nothing
 
-readPatch :: ReadPatch p => B.ByteString -> Maybe (Sealed (p C(x )))
+readPatch :: ReadPatch p => B.ByteString -> Maybe (Sealed (p wX))
 readPatch ps
     = case readPatchPartial ps of
          Just (p, ps') | B.null (dropSpace ps') -> Just p
@@ -78,7 +77,7 @@ instance (ReadPatch p, PatchListFormat p) => ReadPatch (FL p) where
             = mapSeal unBracketedFL <$> readPatch'
         | otherwise
             = read_patches
-     where read_patches :: ParserM m => m (Sealed (FL p C(x )))
+     where read_patches :: ParserM m => m (Sealed (FL p wX))
            read_patches = do --tracePeek "starting FL read"
                              -- checkConsumes is needed to make sure that something is read,
                              -- to avoid stack overflow when parsing FL (FL p)
@@ -95,11 +94,11 @@ instance (ReadPatch p, PatchListFormat p) => ReadPatch (RL p) where
     readPatch' = mapSeal reverseFL <$> readPatch'
 
 {-# INLINE bracketedFL #-}
-bracketedFL :: forall p m C(x) . (ParserM m) =>
-               (FORALL(y) m (Sealed (p C(y)))) -> Char -> Char -> m (Sealed (FL p C(x)))
+bracketedFL :: forall p m wX . (ParserM m) =>
+               (forall wY . m (Sealed (p wY))) -> Char -> Char -> m (Sealed (FL p wX))
 bracketedFL parser pre post =
     peekforc pre bfl mzero
-        where bfl :: FORALL(z) m (Sealed (FL p C(z)))
+        where bfl :: forall wZ . m (Sealed (FL p wZ))
               bfl = peekforc post (return $ Sealed NilFL)
                                   (do Sealed p <- parser
                                       Sealed ps <- bfl
