@@ -1,4 +1,3 @@
-{-# LANGUAGE BangPatterns #-}
 -- | This module defines our parsing monad.  In the past there have been lazy
 -- and strict parsers in this module.  Currently we have only the strict
 -- variant and it is used for parsing patch files.
@@ -10,13 +9,14 @@ module Darcs.Patch.ReadMonads (ParserM, Darcs.Patch.ReadMonads.take,
                         checkConsumes,
                         linesStartingWith, linesStartingWithEndingWith) where
 
-import ByteStringUtils ( dropSpace, breakSpace, breakFirstPS,
+import Darcs.Util.ByteString ( dropSpace, breakSpace, breakFirstPS,
                          readIntPS, breakLastPS )
 import qualified Data.ByteString as B (null, drop, length, tail, empty,
                                        ByteString)
 import qualified Data.ByteString.Char8 as BC ( uncons, dropWhile, break
                                              , splitAt, length, head )
 import Control.Applicative ( Alternative(..), Applicative(..), (<$>) )
+import Data.Foldable ( asum )
 import Control.Monad ( MonadPlus(..) )
 
 -- | 'lexChar' checks if the next space delimited token from
@@ -131,11 +131,11 @@ linesStartingWith c = work $ linesStartingWith' c
 linesStartingWith' :: Char -> B.ByteString -> Maybe (ParserState [B.ByteString])
 linesStartingWith' c thes =
     Just (lsw [] thes)
-    where lsw acc s | B.null s || BC.head s /= c = (reverse acc :*: s)
+    where lsw acc s | B.null s || BC.head s /= c = reverse acc :*: s
           lsw acc s = let s' = B.tail s
                   in case breakFirstPS '\n' s' of
                      Just (l, r) -> lsw (l:acc) r
-                     Nothing -> (reverse (s':acc) :*: B.empty)
+                     Nothing -> reverse (s':acc) :*: B.empty
 
 -- | This is a highly optimized way to read lines that start with a
 -- particular character, and stops when it reaches a particular |
@@ -147,15 +147,13 @@ linesStartingWithEndingWith st en = work $ linesStartingWithEndingWith' st en
 -- | Helper function for 'linesStartingWithEndingWith'.
 linesStartingWithEndingWith' :: Char -> Char -> B.ByteString
                              -> Maybe (ParserState [B.ByteString])
-linesStartingWithEndingWith' st en s = lswew s
+linesStartingWithEndingWith' st en = lswew
     where
-  lswew x | B.null x = Nothing
-  lswew x =
-    if BC.head x == en
-    then Just ([] :*: B.tail x)
-    else if BC.head x /= st
-         then Nothing
-         else case BC.break ((==) '\n') $ B.tail x of
+  lswew x
+    | B.null x = Nothing
+    | BC.head x == en = Just $ [] :*: B.tail x
+    | BC.head x /= st = Nothing
+    | otherwise = case BC.break ('\n' ==) $ B.tail x of
               (l,r) -> case lswew $ B.tail r of
                        Just (ls :*: r') -> Just ((l:ls) :*: r')
                        Nothing ->
@@ -177,7 +175,7 @@ option x p = p <|> pure x
 
 -- | Attempts each option until one succeeds.
 choice :: Alternative f => [f a] -> f a
-choice = foldr (<|>) empty
+choice = asum
 
 -- |Ensure that a parser consumes input when producing a result
 -- Causes the initial state of the input stream to be held on to while the
@@ -242,7 +240,7 @@ instance Monad SM where
     fail   = failSM
 
 instance ParserM SM where
-    work f = SM f
+    work = SM
     maybeWork f = SM $ \s -> case f s of
                                   Just (x :*: s') -> Just (Just x :*: s')
                                   Nothing -> Just (Nothing :*: s)
