@@ -6,7 +6,7 @@ module Darcs.Patch.Rebase.Viewing
     ( RebaseSelect(..)
     , toRebaseSelect, fromRebaseSelect, extractRebaseSelect, reifyRebaseSelect
     , partitionUnconflicted
-    , rsToNamed
+    , rsToPia
     , WithDroppedDeps(..), WDDNamed, commuterIdWDD
     , RebaseChange(..), toRebaseChanges
     ) where
@@ -129,10 +129,10 @@ instance (Show2 p, Show2 (PrimOf p)) => Show (RebaseChange p wX wY) where
             showString "RCRev " . showsPrec2 (appPrec + 1) fixups .
             showString " " . showsPrec2 (appPrec + 1) changes
 
--- |Get hold of the 'Named' patch inside a 'RebaseSelect'.
-rsToNamed :: RebaseSelect p wX wY -> Sealed2 (Named p)
-rsToNamed (RSFwd _ toEdit) = Sealed2 toEdit
-rsToNamed (RSRev _ toEdit) = Sealed2 toEdit
+-- |Get hold of the 'PatchInfoAnd' patch inside a 'RebaseSelect'.
+rsToPia :: RebaseSelect p wX wY -> Sealed2 (PatchInfoAnd p)
+rsToPia (RSFwd _ toEdit) = Sealed2 (n2pia toEdit)
+rsToPia (RSRev _ toEdit) = Sealed2 (n2pia toEdit)
 
 instance PrimPatchBase p => PrimPatchBase (RebaseSelect p) where
    type PrimOf (RebaseSelect p) = PrimOf p
