@@ -19,10 +19,10 @@
 module Darcs.Patch.RegChars ( regChars,
                 ) where
 
-(&&&) :: (a -> Bool) -> (a -> Bool) -> (a -> Bool)
+(&&&) :: (a -> Bool) -> (a -> Bool) -> a -> Bool
 (&&&) a b c = a c && b c
 
-(|||) :: (a -> Bool) -> (a -> Bool) -> (a -> Bool)
+(|||) :: (a -> Bool) -> (a -> Bool) -> a -> Bool
 (|||) a b c = a c || b c
 
 {-# INLINE regChars #-}
@@ -38,7 +38,7 @@ module Darcs.Patch.RegChars ( regChars,
 -- if it is not escaped, but the default 'filename_toks' in
 -- Darcs.Commands.Replace uses an escaped dot (WHY?).
 
-regChars :: String -> (Char -> Bool)
+regChars :: String -> Char -> Bool
 regChars ('^':cs) = not . normalRegChars (unescapeChars cs)
 regChars ('\\':'^':cs) = normalRegChars $ unescapeChars $ '^':cs
 regChars cs = normalRegChars $ unescapeChars cs
@@ -63,7 +63,7 @@ unescapeChars [] = []
 -- chars, and also unescaping of escaped special chars. If a non-special
 -- char is still escaped by now we get a failure.
 
-normalRegChars :: String -> (Char -> Bool)
+normalRegChars :: String -> Char -> Bool
 normalRegChars ('\\':'.':cs) = (=='.') ||| normalRegChars cs
 normalRegChars ('\\':'-':cs) = (=='-') ||| normalRegChars cs
 normalRegChars ('\\':'\\':cs) = (=='\\') ||| normalRegChars cs
