@@ -1,4 +1,4 @@
-{-# LANGUAGE CPP, RankNTypes, GADTs #-}
+{-# LANGUAGE CPP, RankNTypes, GADTs, ViewPatterns #-}
 {-# OPTIONS_GHC -fno-warn-unused-imports #-}
 -- Copyright (C) 2009 Ganesh Sittampalam
 --
@@ -22,25 +22,27 @@
 -- CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 -- SOFTWARE.
 
-module Darcs.Patch.Split ( Splitter(..), rawSplitter, noSplitter, primSplitter ) where
+module Darcs.Patch.Split ( Splitter(..), rawSplitter, noSplitter, primSplitter, reversePrimSplitter ) where
 
 import Data.List ( intersperse )
 
-import Darcs.Witnesses.Ordered
-import Darcs.Witnesses.Sealed
+import Darcs.Patch.Witnesses.Ordered
+import Darcs.Patch.Witnesses.Sealed
 
-import Darcs.Patch.Patchy ( ReadPatch(..), ShowPatch(..), Invert(..) )
-import Darcs.Patch.Prim ( Prim(..), FilePatchType(..), canonize, canonizeFL )
+import Darcs.Patch.FileHunk ( FileHunk(..), IsHunk(..) )
+import Darcs.Patch.Patchy ( ReadPatch(..), showPatch, ShowPatch(..), Invert(..) )
+import Darcs.Patch.Invert (invertFL)
+import Darcs.Patch.Prim ( PrimPatch, canonize, canonizeFL, primFromHunk )
 import Darcs.Patch.ReadMonads ( parseStrictly )
 import Darcs.Patch.Read ()
 import Darcs.Patch.Viewing ()
 
-import Printer ( renderPS )
+import Darcs.Util.Printer ( renderPS, RenderMode(..) )
+import qualified Darcs.Util.Diff as D ( DiffAlgorithm )
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BC
 
-#include "gadts.h"
 
 -- |A splitter is something that can take a patch and (possibly) render it
 -- as text in some format of its own choosing.
@@ -60,9 +62,9 @@ import qualified Data.ByteString.Char8 as BC
 -- splitters for Prim etc, and the generality doesn't cost anything.
 data Splitter p
   = Splitter {
-              applySplitter :: FORALL(x y) p C(x y)
+              applySplitter :: forall wX wY . D.DiffAlgorithm -> p wX wY
                               -> Maybe (B.ByteString,
-                                        B.ByteString -> Maybe (FL p C(x y)))
+                                        B.ByteString -> Maybe (FL p wX wY))
               -- canonization is needed to undo the effects of splitting
               -- Typically, the list returned by applySplitter will not
               -- be in the simplest possible form (since the user will have
@@ -72,7 +74,8 @@ data Splitter p
               -- we might record (or whatever) a rather strange looking patch.
               -- This hook allows the splitter to provide an appropriate
               -- function for doing this.
-             ,canonizeSplit :: FORALL(x y) FL p C(x y) -> FL p C(x y)
+             ,canonizeSplit :: forall wX wY . D.DiffAlgorithm -> FL p wX wY
+                              -> FL p wX wY
              }
 
 {- Some facts that probably ought to be true about splitters: should make some QC
@@ -85,7 +88,7 @@ applySplitter p = Just (bs, f) ; f bs' = Just ps ==> canonizeSplit ps = p :>: Ni
 -}
 
 -- Does not canonize as there is no generic operation to do this.
-withEditedHead :: Invert p => p C(x y) -> p C(x z) -> FL p C(x y)
+withEditedHead :: Invert p => p wX wY -> p wX wZ -> FL p wX wY
 withEditedHead p res = res :>: invert res :>: p :>: NilFL
 
 -- |This generic splitter just lets the user edit the printed representation of the patch
@@ -93,23 +96,43 @@ withEditedHead p res = res :>: invert res :>: p :>: NilFL
 rawSplitter :: (ShowPatch p, ReadPatch p, Invert p) => Splitter p
 rawSplitter = Splitter {
                   applySplitter =
-                     \p -> Just (renderPS . showPatch $ p,
-                                 \str -> case parseStrictly (readPatch' False) str of
-                                          Just (Just (Sealed res), _) -> Just (withEditedHead p res)
+                     \_ p -> Just (renderPS Standard . showPatch $ p,
+                                 \str -> case parseStrictly readPatch' str of
+                                          Just (Sealed res, _) -> Just (withEditedHead p res)
                                           _ -> Nothing
                                 )
-                 ,canonizeSplit = id
+                 ,canonizeSplit = \_ b -> id b
                 }
 
 -- |Never splits. In other code we normally pass around Maybe Splitter instead of using this
 -- as the default, because it saves clients that don't care about splitting from having to
 -- import this module just to get noSplitter.
 noSplitter :: Splitter p
-noSplitter = Splitter { applySplitter = const Nothing, canonizeSplit = id }
+noSplitter = Splitter { applySplitter = \_ -> const Nothing, canonizeSplit = \_ b -> id b }
 
 
-doPrimSplit :: Prim C(x y) -> Maybe (B.ByteString, B.ByteString -> Maybe (FL Prim C(x y)))
-doPrimSplit (FP fn (Hunk n before after))
+doPrimSplit :: PrimPatch prim => D.DiffAlgorithm -> prim wX wY
+            -> Maybe (B.ByteString, B.ByteString -> Maybe (FL prim wX wY))
+doPrimSplit da = doPrimSplit_ da True explanation
+  where
+    explanation = map BC.pack
+                   [ "Interactive hunk edit:"
+                   , " - Edit the section marked 'AFTER'"
+                   , " - Arbitrary editing is supported"
+                   , " - This will only affect the patch, not your working copy"
+                   , " - Hints:"
+                   , "   - To split added text, delete the part you want to postpone"
+                   , "   - To split removed text, copy back the part you want to retain"
+                   , ""
+                   ]
+
+doPrimSplit_ :: (PrimPatch prim, IsHunk p)
+             => D.DiffAlgorithm
+             -> Bool
+             -> [B.ByteString]
+             -> p wX wY
+             -> Maybe (B.ByteString, B.ByteString -> Maybe (FL prim wX wY))
+doPrimSplit_ da edit_before_part helptext (isHunk -> Just (FileHunk fn n before after))
  = Just (B.concat $ intersperse (BC.pack "\n") $ concat
            [ helptext
            , [mkSep " BEFORE (reference) =========================="]
@@ -121,28 +144,48 @@ doPrimSplit (FP fn (Hunk n before after))
          \bs -> do let ls = BC.split '\n' bs
                    (_, ls2) <- breakSep ls        -- before
                    (before', ls3) <- breakSep ls2 -- after 1
-                   (after', _) <- breakSep ls3    -- after 2
-                   return (hunk before before' +>+ hunk before' after' +>+ hunk after' after))
+                   (after', _) <- breakSep ls3    -- after
+                   return $
+                     if edit_before_part
+                     then hunk before before' +>+ hunk before' after' +>+ hunk after' after
+                     else hunk before after' +>+ hunk after' after)
     where sep = BC.pack "=========================="
-          helptext = map BC.pack [ "Interactive hunk edit:"
-                                 , " - Edit the section marked 'AFTER'"
-                                 , " - Arbitrary editing is supported"
-                                 , " - This will only affect the patch, not your working copy"
-                                 , " - Hints:"
-                                 , "   - To split added text, delete the part you want to postpone"
-                                 , "   - To split removed text, copy back the part you want to retain"
-                                 , ""
-                                 ]
-          hunk :: [B.ByteString] -> [B.ByteString] -> FL Prim C(a b)
-          hunk b a = canonize (FP fn (Hunk n b a))
+          hunk :: PrimPatch prim => [B.ByteString] -> [B.ByteString] -> FL prim wA wB
+          hunk b a = canonize da (primFromHunk (FileHunk fn n b a))
           mkSep s = BC.append sep (BC.pack s)
           breakSep xs = case break (sep `BC.isPrefixOf`) xs of
                            (_, []) -> Nothing
                            (ys, _:zs) -> Just (ys, zs)
-doPrimSplit _ = Nothing
+doPrimSplit_ _ _ _ _ = Nothing
 
 -- |Split a primitive hunk patch up
 -- by allowing the user to edit both the before and after lines, then insert fixup patches
 -- to clean up the mess.
-primSplitter :: Splitter Prim
-primSplitter = Splitter { applySplitter = doPrimSplit, canonizeSplit = canonizeFL }
+primSplitter :: PrimPatch p => Splitter p
+primSplitter  = Splitter { applySplitter = doPrimSplit
+                         , canonizeSplit = canonizeFL }
+
+doReversePrimSplit :: PrimPatch prim => D.DiffAlgorithm -> prim wX wY
+                   -> Maybe (B.ByteString, B.ByteString -> Maybe (FL prim wX wY))
+doReversePrimSplit da prim = do
+  (text, parser) <- doPrimSplit_ da False reverseExplanation (invert prim)
+  let parser' p = do
+        patch <- parser  p
+        return . reverseRL $ invertFL patch
+  return (text, parser')
+  where
+    reverseExplanation =
+      map BC.pack [ "Interactive hunk edit:"
+                  , " - Edit the section marked 'AFTER' (representing the state to which you'll revert)"
+                  , " - Arbitrary editing is supported"
+                  , " - Your working copy will be returned to the 'AFTER' state"
+                  , " - Do not touch the 'BEFORE' section"
+                  , " - Hints:"
+                  , "   - To revert only a part of a text addition, delete the part you want to get rid of"
+                  , "   - To revert only a part of a removal, copy back the part you want to retain"
+                  , ""
+                                 ]
+
+reversePrimSplitter :: PrimPatch prim => Splitter prim
+reversePrimSplitter = Splitter { applySplitter = doReversePrimSplit
+                               , canonizeSplit = canonizeFL}
