@@ -1,6 +1,6 @@
 module Darcs.Patch.SummaryData ( SummDetail(..), SummOp(..) ) where
 
-import Darcs.Patch.FileName ( FileName )
+import Darcs.Util.Path ( FileName )
 
 data SummDetail = SummAddDir FileName
                 | SummRmDir  FileName
