@@ -17,7 +17,6 @@
 
 {-# LANGUAGE CPP #-}
 
-#include "gadts.h"
 
 module Darcs.Patch.TouchesFiles ( lookTouch, chooseTouching, choosePreTouching,
                       selectTouching,
@@ -26,51 +25,47 @@ module Darcs.Patch.TouchesFiles ( lookTouch, chooseTouching, choosePreTouching,
 import Control.Applicative ( (<$>) )
 import Data.List ( isSuffixOf, nub )
 
-import Darcs.Patch.Choices ( PatchChoices, Tag, TaggedPatch,
-                             patchChoices, tag, getChoices,
-                      forceFirsts, forceLasts, tpPatch,
+import Darcs.Patch.Choices ( PatchChoices, Label, LabelledPatch,
+                             patchChoices, label, getChoices,
+                      forceFirsts, forceLasts, lpPatch,
                     )
 import Darcs.Patch ( Patchy, invert )
 import Darcs.Patch.Apply ( ApplyState, applyToFilePaths, effectOnFilePaths )
-import Darcs.Witnesses.Ordered ( FL(..), (:>)(..), mapFL_FL, (+>+) )
-import Darcs.Witnesses.Sealed ( Sealed, seal )
+import Darcs.Patch.Inspect ( PatchInspect )
+import Darcs.Patch.Witnesses.Ordered ( FL(..), (:>)(..), mapFL_FL, (+>+) )
+import Darcs.Patch.Witnesses.Sealed ( Sealed, seal )
 import Storage.Hashed.Tree( Tree )
 
-selectTouching :: (Patchy p, ApplyState p ~ Tree)
-               => Maybe [FilePath] -> PatchChoices p C(x y) -> PatchChoices p C(x y)
+labelTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree) => Bool
+            -> [FilePath] -> FL (LabelledPatch p) wX wY -> [Label]
+labelTouching _ _ NilFL = []
+labelTouching wantTouching fs (lp :>: lps) =
+    case lookTouchOnlyEffect fs (lpPatch lp) of
+        (doesTouch, fs') ->
+            let rest = labelTouching wantTouching fs' lps
+            in (if doesTouch == wantTouching then (label lp :) else id) rest
+
+labelNotTouchingFM :: (PatchInspect p, Patchy p, ApplyState p ~ Tree)
+                 => [FilePath] -> PatchChoices p wX wY -> [Label]
+labelNotTouchingFM files pc = case getChoices pc of
+    fc :> mc :> _ -> labelTouching False (map fix files) (fc +>+ mc)
+
+selectTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree)
+               => Maybe [FilePath] -> PatchChoices p wX wY -> PatchChoices p wX wY
 selectTouching Nothing pc = pc
 selectTouching (Just files) pc = forceFirsts xs pc
-    where ct :: (Patchy p, ApplyState p ~ Tree) => [FilePath] -> FL (TaggedPatch p) C(x y) -> [Tag]
-          ct _ NilFL = []
-          ct fs (tp:>:tps) = case lookTouchOnlyEffect fs (tpPatch tp) of
-                             (True, fs') -> tag tp:ct fs' tps
-                             (False, fs') -> ct fs' tps
-          xs = case getChoices pc of
-               _ :> mc :> lc -> ct (map fix files) (mc +>+ lc)
+    where xs = case getChoices pc of
+               _ :> mc :> lc -> labelTouching True (map fix files) (mc +>+ lc)
 
-deselectNotTouching :: (Patchy p, ApplyState p ~ Tree)
-                    => Maybe [FilePath] -> PatchChoices p C(x y) -> PatchChoices p C(x y)
+deselectNotTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree)
+                    => Maybe [FilePath] -> PatchChoices p wX wY -> PatchChoices p wX wY
 deselectNotTouching Nothing pc = pc
-deselectNotTouching (Just files) pc = forceLasts xs pc
-    where ct :: (Patchy p, ApplyState p ~ Tree) => [FilePath] -> FL (TaggedPatch p) C(x y) -> [Tag]
-          ct _ NilFL = []
-          ct fs (tp:>:tps) = case lookTouchOnlyEffect fs (tpPatch tp) of
-                             (True, fs') -> ct fs' tps
-                             (False, fs') -> tag tp:ct fs' tps
-          xs = case getChoices pc of
-               fc :> mc :> _ -> ct (map fix files) (fc +>+ mc)
+deselectNotTouching (Just files) pc = forceLasts (labelNotTouchingFM files pc) pc
 
-selectNotTouching :: (Patchy p, ApplyState p ~ Tree)
-                  => Maybe [FilePath] -> PatchChoices p C(x y) -> PatchChoices p C(x y)
+selectNotTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree)
+                  => Maybe [FilePath] -> PatchChoices p wX wY -> PatchChoices p wX wY
 selectNotTouching Nothing pc = pc
-selectNotTouching (Just files) pc = forceFirsts xs pc
-    where ct :: (Patchy p, ApplyState p ~ Tree) => [FilePath] -> FL (TaggedPatch p) C(x y) -> [Tag]
-          ct _ NilFL = []
-          ct fs (tp:>:tps) = case lookTouchOnlyEffect fs (tpPatch tp) of
-                             (True, fs') -> ct fs' tps
-                             (False, fs') -> tag tp:ct fs' tps
-          xs = case getChoices pc of
-               fc :> mc :> _ -> ct (map fix files) (fc +>+ mc)
+selectNotTouching (Just files) pc = forceFirsts (labelNotTouchingFM files pc) pc
 
 fix :: FilePath -> FilePath
 fix f | "/" `isSuffixOf` f = fix $ init f
@@ -78,25 +73,25 @@ fix "" = "."
 fix "." = "."
 fix f = "./" ++ f
 
-chooseTouching :: (Patchy p, ApplyState p ~ Tree)
-               => Maybe [FilePath] -> FL p C(x y) -> Sealed (FL p C(x))
+chooseTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree)
+               => Maybe [FilePath] -> FL p wX wY -> Sealed (FL p wX)
 chooseTouching Nothing p = seal p
 chooseTouching files p = case getChoices $ selectTouching files $ patchChoices p of
-                          fc :> _ :> _ -> seal $ mapFL_FL tpPatch fc
+                          fc :> _ :> _ -> seal $ mapFL_FL lpPatch fc
 
-choosePreTouching :: (Patchy p, ApplyState p ~ Tree)
-                  => Maybe [FilePath] -> FL p C(x y) -> Sealed (FL p C(x))
+choosePreTouching :: (Patchy p, PatchInspect p, ApplyState p ~ Tree)
+                  => Maybe [FilePath] -> FL p wX wY -> Sealed (FL p wX)
 choosePreTouching files patch = chooseTouching filesBeforePatch patch where
     filesBeforePatch = effectOnFilePaths (invert patch) <$> files
 
-lookTouchOnlyEffect :: (Patchy p, ApplyState p ~ Tree) => [FilePath] -> p C(x y)
+lookTouchOnlyEffect :: (Patchy p, PatchInspect p, ApplyState p ~ Tree) => [FilePath] -> p wX wY
     -> (Bool, [FilePath])
 lookTouchOnlyEffect fs p = (wasTouched, fs') where
     (wasTouched, _, fs', _) = lookTouch Nothing fs p
 
 
-lookTouch :: (Patchy p, ApplyState p ~ Tree) => Maybe [(FilePath, FilePath)]
-    -> [FilePath] -> p C(x y)
+lookTouch :: (Patchy p, PatchInspect p, ApplyState p ~ Tree) => Maybe [(FilePath, FilePath)]
+    -> [FilePath] -> p wX wY
     -> (Bool, [FilePath], [FilePath], [(FilePath, FilePath)])
 lookTouch renames fs p = (anyTouched, touchedFs, fs', renames')
     where
