@@ -1,7 +1,7 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
 module Darcs.Patch.V1.Apply () where
 
-import Darcs.Patch.Apply ( Apply, apply )
+import Darcs.Patch.Apply ( ApplyState, Apply, apply )
 import Darcs.Patch.Prim ( PrimPatch, applyPrimFL )
 import Darcs.Patch.Repair ( RepairToFL, applyAndTryToFixFL,
                             mapMaybeSnd )
@@ -9,11 +9,9 @@ import Darcs.Patch.Effect ( effect )
 
 import Darcs.Patch.V1.Commute ()
 import Darcs.Patch.V1.Core ( Patch(..) )
-import Darcs.Patch.Apply( ApplyState )
 
-import Darcs.Witnesses.Ordered ( mapFL_FL )
+import Darcs.Patch.Witnesses.Ordered ( mapFL_FL )
 
-#include "gadts.h"
 
 instance PrimPatch prim => Apply (Patch prim) where
     type ApplyState (Patch prim) = ApplyState prim
