@@ -10,22 +10,21 @@ import Darcs.Patch.ReadMonads ( ParserM, choice, string,
 import Darcs.Patch.V1.Core ( Patch(..) )
 import Darcs.Patch.V1.Commute ( merger )
 
-import Darcs.Witnesses.Sealed ( Sealed(..), seal, mapSeal )
-import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(..), seal, mapSeal )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
 
 import Control.Monad ( liftM )
 import qualified Data.ByteString.Char8 as BC ( unpack, pack )
 import qualified Data.ByteString       as B  (ByteString )
 
-#include "gadts.h"
 
 instance PrimPatch prim => ReadPatch (Patch prim) where
  readPatch'
    = choice [ liftM seal $ skipSpace >> readMerger True
             , liftM seal $ skipSpace >> readMerger False
-            , liftM (mapSeal PP) $ readPatch'
+            , liftM (mapSeal PP) readPatch'
             ]
-readMerger :: (ParserM m, PrimPatch prim) => Bool -> m (Patch prim C(x y))
+readMerger :: (ParserM m, PrimPatch prim) => Bool -> m (Patch prim wX wY)
 readMerger b = do string s
                   g <- myLex'
                   lexChar '('
