@@ -1,5 +1,4 @@
 {-# OPTIONS_GHC -fno-warn-orphans #-}
-{-# LANGUAGE ViewPatterns #-}
 module Darcs.Patch.V1.Viewing () where
 
 import Darcs.Patch.Prim ( PrimPatch )
@@ -10,7 +9,6 @@ import Darcs.Patch.V1.Apply ()
 import Darcs.Patch.V1.Core ( Patch(..) )
 import Darcs.Patch.V1.Show ( showPatch_ )
 
-#include "gadts.h"
 
 instance PrimPatch prim => ShowPatchBasic (Patch prim) where
     showPatch = showPatch_
