@@ -18,7 +18,6 @@
 {-# OPTIONS_GHC -fno-warn-orphans -fno-warn-name-shadowing #-}
 {-# LANGUAGE CPP #-}
 
-#include "gadts.h"
 
 module Darcs.Patch.V2.Real
     ( RealPatch(..)
@@ -29,27 +28,34 @@ module Darcs.Patch.V2.Real
     , mergeUnravelled
     ) where
 
+#if MIN_VERSION_base(4,8,0)
+import Prelude hiding ( (*>) )
+#endif
+
 import Control.Monad ( mplus, liftM )
 import qualified Data.ByteString.Char8 as BC ( ByteString, pack )
 import Data.Maybe ( fromMaybe )
 import Data.List ( partition, nub )
 
-import Darcs.ColorPrinter ( errorDoc, assertDoc )
 import Darcs.Patch.Commute ( commuteFL, commuteFLorComplain, commuteRL
                            , commuteRLFL )
 import Darcs.Patch.Conflict ( Conflict(..), CommuteNoConflicts(..)
                             , IsConflictedPrim(..), ConflictState(..) )
 import Darcs.Patch.ConflictMarking ( mangleUnravelled )
+import Darcs.Patch.Debug
 import Darcs.Patch.Effect ( Effect(..) )
 import Darcs.Patch.FileHunk ( IsHunk(..) )
 import Darcs.Patch.Format ( PatchListFormat(..), ListFormat(..)
                           , FileNameFormat(NewFormat) )
 import Darcs.Patch.Invert ( invertFL, invertRL )
+import Darcs.Patch.MaybeInternal ( MaybeInternal )
 import Darcs.Patch.Merge ( Merge(..) )
 import Darcs.Patch.Prim ( FromPrim(..), ToFromPrim(..), showPrim, showPrimFL
                         , readPrim, PrimOf, PrimPatchBase, PrimPatch )
 import Darcs.Patch.Read ( bracketedFL )
 import Darcs.Patch.ReadMonads ( skipSpace, string, choice )
+import Darcs.Patch.Rebase.NameHack ( NameHack )
+import Darcs.Patch.Rebase.Recontext ( RecontextRebase )
 import Darcs.Patch.Repair ( mapMaybeSnd, RepairToFL(..), Check(..) )
 import Darcs.Patch.Patchy ( Patchy, Apply(..), Commute(..), PatchInspect(..)
                           , ReadPatch(..), ShowPatch(..), Invert(..) )
@@ -63,17 +69,23 @@ import Darcs.Patch.V2.Non ( Non(..), Nonable(..), unNon, showNons, showNon
                           , commuteOrAddToCtx, commuteOrAddToCtxRL
                           , commuteOrRemFromCtx, commuteOrRemFromCtxFL
                           , remNons, (*>), (>*), (*>>), (>>*) )
-import Darcs.Utils ( nubsort )
-import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
-import Darcs.Witnesses.Eq ( MyEq(..), EqCheck(..) )
-import Darcs.Witnesses.Ordered ( FL(..), RL(..), (:>)(..), (+>+), (+<+)
-                               , mapFL_FL, reverseFL, (:\/:)(..), (:/\:)(..)
-                               , reverseRL, lengthFL, lengthRL, nullFL )
-import Darcs.Witnesses.Sealed ( FlippedSeal(..), Sealed(Sealed), mapSeal
-                              , unseal )
-import Darcs.Witnesses.Show ( Show2(..), ShowDict(..) )
+import Data.List.Ordered ( nubSort )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Eq ( MyEq(..), EqCheck(..) )
+import Darcs.Patch.Witnesses.Ordered
+    ( FL(..), RL(..), (:>)(..), (+>+), (+<+)
+    , mapFL_FL, reverseFL, (:\/:)(..), (:/\:)(..)
+    , reverseRL, lengthFL, lengthRL, nullFL )
+import Darcs.Patch.Witnesses.Sealed
+    ( FlippedSeal(..), Sealed(Sealed), mapSeal
+    , unseal )
+import Darcs.Patch.Witnesses.Show
+    ( Show1(..), Show2(..), ShowDict(..)
+    , showsPrec2, appPrec
+    )
+import Darcs.Util.Printer.Color ( errorDoc, assertDoc )
 
-import Printer ( Doc, renderString, blueText, redText, (<+>), ($$) )
+import Darcs.Util.Printer ( Doc, blueText, redText, (<+>), ($$) )
 
 #include "impossible.h"
 
@@ -106,30 +118,30 @@ import Printer ( Doc, renderString, blueText, redText, (<+>), ($$) )
 -- Therefore, the effect of a Conflictor is @invert xx@.
 --
 -- @InvConflictor ix xx x@: like @invert (Conflictor ix xx x)@
-data RealPatch prim C(x y) where
-    Duplicate :: Non (RealPatch prim) C(x) -> RealPatch prim C(x x)
-    Etacilpud :: Non (RealPatch prim) C(x) -> RealPatch prim C(x x)
-    Normal :: prim C(x y) -> RealPatch prim C(x y)
-    Conflictor :: [Non (RealPatch prim) C(x)] -> FL prim C(x y)
-               -> Non (RealPatch prim) C(x) -> RealPatch prim C(y x)
-    InvConflictor :: [Non (RealPatch prim) C(x)] -> FL prim C(x y)
-                  -> Non (RealPatch prim) C(x) -> RealPatch prim C(x y)
+data RealPatch prim wX wY where
+    Duplicate :: Non (RealPatch prim) wX -> RealPatch prim wX wX
+    Etacilpud :: Non (RealPatch prim) wX -> RealPatch prim wX wX
+    Normal :: prim wX wY -> RealPatch prim wX wY
+    Conflictor :: [Non (RealPatch prim) wX] -> FL prim wX wY
+               -> Non (RealPatch prim) wX -> RealPatch prim wY wX
+    InvConflictor :: [Non (RealPatch prim) wX] -> FL prim wX wY
+                  -> Non (RealPatch prim) wX -> RealPatch prim wX wY
 
 instance PrimPatch prim => PrimPatchBase (RealPatch prim) where
    type PrimOf (RealPatch prim) = prim
 
 -- | 'isDuplicate' @p@ is @True@ if @p@ is either a 'Duplicate' or 'Etacilpud'
 -- patch.
-isDuplicate :: RealPatch prim C(s y) -> Bool
+isDuplicate :: RealPatch prim wS wY -> Bool
 isDuplicate (Duplicate _) = True
 isDuplicate (Etacilpud _) = True
 isDuplicate _ = False
 
 -- | 'isForward' @p@ is @True@ if @p@ is either an 'InvConflictor' or
 -- 'Etacilpud'.
-isForward :: PrimPatch prim => RealPatch prim C(s y) -> Maybe Doc
+isForward :: PrimPatch prim => RealPatch prim wS wY -> Maybe Doc
 isForward p = case p of
-    p@(InvConflictor _ _ _) -> justRedP "An inverse conflictor" p
+    p@(InvConflictor{}) -> justRedP "An inverse conflictor" p
     p@(Etacilpud _) -> justRedP "An inverse duplicate" p
     _ -> Nothing
   where
@@ -137,8 +149,8 @@ isForward p = case p of
 
 -- |'mergeUnravelled' is used when converting from Darcs V1 patches (Mergers)
 -- to Darcs V2 patches (Conflictors).
-mergeUnravelled :: PrimPatch prim => [Sealed ((FL prim) C(x))]
-                -> Maybe (FlippedSeal (RealPatch prim) C(x))
+mergeUnravelled :: PrimPatch prim => [Sealed ((FL prim) wX)]
+                -> Maybe (FlippedSeal (RealPatch prim) wX)
 mergeUnravelled [] = Nothing
 mergeUnravelled [_] = Nothing
 mergeUnravelled ws =
@@ -147,29 +159,29 @@ mergeUnravelled ws =
         Just NilRL -> bug "found no patches in mergeUnravelled"
         Just (z :<: _) -> Just $ FlippedSeal z
   where
-    notNullS :: PrimPatch prim => Sealed ((FL prim) C(x)) -> Bool
+    notNullS :: PrimPatch prim => Sealed ((FL prim) wX) -> Bool
     notNullS (Sealed NilFL) = False
     notNullS _ = True
 
-    mergeUnravelled_private :: PrimPatch prim => [Sealed (FL prim C(x))]
-                            -> Maybe (RL (RealPatch prim) C(x x))
+    mergeUnravelled_private :: PrimPatch prim => [Sealed (FL prim wX)]
+                            -> Maybe (RL (RealPatch prim) wX wX)
     mergeUnravelled_private xs = let nonNullXs = filter notNullS xs in
         reverseFL `fmap` mergeConflictingNons (map sealed2non nonNullXs)
 
     -- | 'sealed2non' @(Sealed xs)@ converts @xs@ to a 'Non'.
     -- @xs@ must be non-empty since we split this list at the last patch,
     -- taking @init xs@ as the context of @last xs@.
-    sealed2non :: Sealed ((FL prim) C(x)) -> Non (RealPatch prim) C(x)
+    sealed2non :: Sealed ((FL prim) wX) -> Non (RealPatch prim) wX
     sealed2non (Sealed xs) =
         case reverseFL xs of
             y :<: ys -> Non (mapFL_FL fromPrim $ reverseRL ys) y
             NilRL -> bug "NilFL encountered in sealed2non"
 
-mergeConflictingNons :: PrimPatch prim => [Non (RealPatch prim) C(x)]
-                     -> Maybe (FL (RealPatch prim) C(x x))
+mergeConflictingNons :: PrimPatch prim => [Non (RealPatch prim) wX]
+                     -> Maybe (FL (RealPatch prim) wX wX)
 mergeConflictingNons ns = mcn $ map unNon ns
-    where mcn :: PrimPatch prim => [Sealed (FL (RealPatch prim) C(x))]
-              -> Maybe (FL (RealPatch prim) C(x x))
+    where mcn :: PrimPatch prim => [Sealed (FL (RealPatch prim) wX)]
+              -> Maybe (FL (RealPatch prim) wX wX)
           mcn [] = Just NilFL
           -- Apparently, the joinEffects call is a safety check "and could be
           -- removed when we're sure of the code"!
@@ -182,18 +194,18 @@ mergeConflictingNons ns = mcn $ map unNon ns
                     case merge (ps :\/: qs) of
                         qs' :/\: _ -> mcn (Sealed (c +>+ ps +>+ qs'):zs)
 
-joinEffects :: forall p C(x y) . (Effect p, Invert (PrimOf p),
-            Commute (PrimOf p), MyEq (PrimOf p)) => p C(x y)
-            -> FL (PrimOf p) C(x y)
+joinEffects :: forall p wX wY . (Effect p, Invert (PrimOf p),
+            Commute (PrimOf p), MyEq (PrimOf p)) => p wX wY
+            -> FL (PrimOf p) wX wY
 joinEffects = joinInverses . effect
-    where joinInverses :: FL (PrimOf p) C(a b) -> FL (PrimOf p) C(a b)
+    where joinInverses :: FL (PrimOf p) wA wB -> FL (PrimOf p) wA wB
           joinInverses NilFL = NilFL
           joinInverses (p :>: ps) =
               let ps' = joinInverses ps in
               fromMaybe (p :>: ps') $ removeFL (invert p) ps'
 
-assertConsistent :: PrimPatch prim => RealPatch prim C(x y)
-                 -> RealPatch prim C(x y)
+assertConsistent :: PrimPatch prim => RealPatch prim wX wY
+                 -> RealPatch prim wX wY
 assertConsistent x = flip assertDoc x $ do
     e <- isConsistent x
     Just (redText "Inconsistent patch:" $$ showPatch x $$ e)
@@ -212,15 +224,15 @@ assertConsistent x = flip assertDoc x $ do
 
 -- The first output is the set of patches from @xxx@ that are depended upon by
 -- @yyy@.
-mergeAfterConflicting :: PrimPatch prim => FL (RealPatch prim) C(x x)
-                      -> FL prim C(x y) -> Maybe ( FL (RealPatch prim) C(x x)
-                                                 , FL (RealPatch prim) C(x y))
+mergeAfterConflicting :: PrimPatch prim => FL (RealPatch prim) wX wX
+                      -> FL prim wX wY -> Maybe ( FL (RealPatch prim) wX wX
+                                                 , FL (RealPatch prim) wX wY)
 mergeAfterConflicting xxx yyy = mac (reverseFL xxx) yyy NilFL
   where
     mac :: PrimPatch prim
-        => RL (RealPatch prim) C(x y) -> FL prim C(y z)
-        -> FL (RealPatch prim) C(z a)
-        -> Maybe (FL (RealPatch prim) C(x x), FL (RealPatch prim) C(x a))
+        => RL (RealPatch prim) wX wY -> FL prim wY wZ
+        -> FL (RealPatch prim) wZ wA
+        -> Maybe (FL (RealPatch prim) wX wX, FL (RealPatch prim) wX wA)
     mac NilRL xs goneby = case joinEffects goneby of
                               NilFL -> Just (NilFL, mapFL_FL Normal xs)
                               _ -> Nothing
@@ -248,8 +260,8 @@ mergeAfterConflicting xxx yyy = mac (reverseFL xxx) yyy NilFL
                                    NilFL <- return pa
                                    return (reverseRL (p' :<: a) +>+ b', xs')
 
-geteff :: PrimPatch prim => [Non (RealPatch prim) C(x)] -> FL prim C(x y)
-       -> ([Non (RealPatch prim) C(x)], FL (RealPatch prim) C(x y))
+geteff :: PrimPatch prim => [Non (RealPatch prim) wX] -> FL prim wX wY
+       -> ([Non (RealPatch prim) wX], FL (RealPatch prim) wX wY)
 geteff _ NilFL = ([], NilFL)
 geteff ix (x :>: xs) | Just ix' <- mapM (commuteOrRemFromCtx (Normal x)) ix =
     case geteff ix' xs of
@@ -272,28 +284,28 @@ geteff ix xx =
                         redText "and xx" $$ showPatch xx $$
                         redText "and rix" $$ showPatch rix
 
-xx2nons :: PrimPatch prim => [Non (RealPatch prim) C(x)] -> FL prim C(x y)
-        -> [Non (RealPatch prim) C(x)]
+xx2nons :: PrimPatch prim => [Non (RealPatch prim) wX] -> FL prim wX wY
+        -> [Non (RealPatch prim) wX]
 xx2nons ix xx = fst $ geteff ix xx
 
-xx2patches :: PrimPatch prim => [Non (RealPatch prim) C(x)] -> FL prim C(x y)
-           -> FL (RealPatch prim) C(x y)
+xx2patches :: PrimPatch prim => [Non (RealPatch prim) wX] -> FL prim wX wY
+           -> FL (RealPatch prim) wX wY
 xx2patches ix xx = snd $ geteff ix xx
 
 -- | If @xs@ consists only of 'Normal' patches, 'allNormal' @xs@ returns
 --   @Just pxs@ those patches (so @lengthFL pxs == lengthFL xs@).
 --   Otherwise, it returns 'Nothing'.
-allNormal :: FL (RealPatch prim) C(x y) -> Maybe (FL prim C(x y))
+allNormal :: FL (RealPatch prim) wX wY -> Maybe (FL prim wX wY)
 allNormal (Normal x :>: xs) = (x  :>: ) `fmap` allNormal xs
 allNormal NilFL = Just NilFL
 allNormal _ = Nothing
 
 -- | This is used for unit-testing and for internal sanity checks
-isConsistent :: PrimPatch prim => RealPatch prim C(x y) -> Maybe Doc
+isConsistent :: PrimPatch prim => RealPatch prim wX wY -> Maybe Doc
 isConsistent (Normal _) = Nothing
 isConsistent (Duplicate _) = Nothing
 isConsistent (Etacilpud _) = Nothing
-isConsistent c@(InvConflictor _ _ _) = isConsistent (invert c)
+isConsistent c@(InvConflictor{}) = isConsistent (invert c)
 isConsistent (Conflictor im mm m@(Non deps _))
     | not $ everyoneConflicts im =
         Just $ redText "Someone doesn't conflict in im in isConsistent"
@@ -315,27 +327,31 @@ isConsistent (Conflictor im mm m@(Non deps _))
                                 showNons imnc
     where (nmm, rmm) = geteff im mm
 
-everyoneConflicts :: PrimPatch prim => [Non (RealPatch prim) C(x)] -> Bool
+everyoneConflicts :: PrimPatch prim => [Non (RealPatch prim) wX] -> Bool
 everyoneConflicts [] = True
 everyoneConflicts (x : xs) = case allConflictsWith x xs of
                                  ([], _) -> False
                                  (_, xs') -> everyoneConflicts xs'
 
-prim2real :: prim C(x y) -> RealPatch prim C(x y)
+prim2real :: prim wX wY -> RealPatch prim wX wY
 prim2real = Normal
 
+instance MaybeInternal (RealPatch prim)
+instance NameHack (RealPatch prim)
+instance RecontextRebase (RealPatch prim)
 instance PrimPatch prim => Patchy (RealPatch prim)
+instance PatchDebug prim => PatchDebug (RealPatch prim)
 
-mergeWith :: PrimPatch prim => Non (RealPatch prim) C(x)
-          -> [Non (RealPatch prim) C(x)] -> Sealed (FL prim C(x))
+mergeWith :: PrimPatch prim => Non (RealPatch prim) wX
+          -> [Non (RealPatch prim) wX] -> Sealed (FL prim wX)
 mergeWith p [] = effect `mapSeal` unNon p
 mergeWith p xs =
     mergeall . map unNon . (p :) . unconflicting_of $ nonDependsOrConflictsP xs
   where
     nonDependsOrConflictsP =
         filter (\x -> not ((p `dependsUpon` x) || (p `conflictsWith` x)))
-    mergeall :: PrimPatch prim => [Sealed (FL (RealPatch prim) C(x))]
-             -> Sealed (FL prim C(x))
+    mergeall :: PrimPatch prim => [Sealed (FL (RealPatch prim) wX)]
+             -> Sealed (FL prim wX)
     mergeall [Sealed x] = Sealed $ effect x
     mergeall [] = Sealed NilFL
     mergeall (Sealed x : Sealed y : rest) =
@@ -350,7 +366,7 @@ instance PrimPatch prim => Conflict (RealPatch prim) where
     conflictedEffect (Duplicate (Non _ x)) = [IsC Duplicated x]
     conflictedEffect (Etacilpud _) = impossible
     conflictedEffect (Conflictor _ _ (Non _ x)) = [IsC Conflicted x]
-    conflictedEffect (InvConflictor _ _ _) = impossible
+    conflictedEffect (InvConflictor{}) = impossible
     conflictedEffect (Normal x) = [IsC Okay x]
     resolveConflicts (Conflictor ix xx x) = [mangledUnravelled : unravelled]
       where
@@ -362,7 +378,7 @@ instance PrimPatch prim => Conflict (RealPatch prim) where
         -- containing a Non, in the case where we have a Normal patch at the
         -- end of the list (using the rest of the RL as context), and an empty
         -- list otherwise.
-        nonxx_ :: RL (RealPatch prim) C(x y) -> [Non (RealPatch prim) C(x)]
+        nonxx_ :: RL (RealPatch prim) wX wY -> [Non (RealPatch prim) wX]
         nonxx_ (Normal q :<: qs) = [Non (reverseRL qs) q]
         nonxx_ _ = []
         isCons = unseal (not . nullFL)
@@ -413,7 +429,7 @@ instance PrimPatch prim => CommuteNoConflicts (RealPatch prim) where
         return (Conflictor iy' (invert iyy') y' :> Normal x')
 
     -- Handle via the previous case, using the inverting commuter.
-    commuteNoConflicts c@(InvConflictor _ _ _ :> Normal _) = invertCommuteNC c
+    commuteNoConflicts c@(InvConflictor{} :> Normal _) = invertCommuteNC c
 
     -- Commuting a Conflictor past a Normal patch is the dual operation to
     -- commuting a Normal patch past a Conflictor.
@@ -423,7 +439,7 @@ instance PrimPatch prim => CommuteNoConflicts (RealPatch prim) where
         return (Normal x' :> Conflictor iy' (invertRL iyy') y')
 
     -- Handle via the previous case, using the inverting commuter.
-    commuteNoConflicts c@(Normal _ :> InvConflictor _ _ _) = invertCommuteNC c
+    commuteNoConflicts c@(Normal _ :> InvConflictor{}) = invertCommuteNC c
 
     -- Commuting two Conflictors, c1 and c2, first commutes the Conflictors'
     -- effects, then commutes the effect of c1 and c2 and the other's
@@ -438,7 +454,7 @@ instance PrimPatch prim => CommuteNoConflicts (RealPatch prim) where
         return (Conflictor iy' yy' y' :> Conflictor ix' xx' x')
 
     -- Handle via the previous case, using the inverting commuter.
-    commuteNoConflicts c@(InvConflictor _ _ _ :> InvConflictor _ _ _) =
+    commuteNoConflicts c@(InvConflictor{} :> InvConflictor{}) =
         invertCommuteNC c
 
     commuteNoConflicts (InvConflictor ix xx x :> Conflictor iy yy y) = do
@@ -495,26 +511,26 @@ minus xs [] = Just xs
 minus xs (y:ys) = do xs' <- remove1 y xs
                      xs' `minus` ys
 
-invertNon :: PrimPatch prim => Non (RealPatch prim) C(x)
-          -> Non (RealPatch prim) C(x)
+invertNon :: PrimPatch prim => Non (RealPatch prim) wX
+          -> Non (RealPatch prim) wX
 invertNon (Non c x)
     | Just rc' <- removeRL nix (reverseFL c) = Non (reverseRL rc') (invert x)
     | otherwise = commuteOrAddToCtxRL (Normal x :<: reverseFL c) $ non nix
   where
     nix = Normal $ invert x
 
-nonTouches :: PatchInspect prim => Non (RealPatch prim) C(x) -> [FilePath]
+nonTouches :: PatchInspect prim => Non (RealPatch prim) wX -> [FilePath]
 nonTouches (Non c x) = listTouchedFiles (c +>+ fromPrim x :>: NilFL)
 
 nonHunkMatches :: PatchInspect prim => (BC.ByteString -> Bool)
-               -> Non (RealPatch prim) C(x) -> Bool
+               -> Non (RealPatch prim) wX -> Bool
 nonHunkMatches f (Non c x) = hunkMatches f c || hunkMatches f x
 
-toNons :: forall p C(x y) . (Conflict p, Patchy p, PatchListFormat p,
-       ToFromPrim p, Nonable p, ShowPatchBasic (PrimOf p)) => FL p C(x y)
-       -> [Non p C(x)]
+toNons :: forall p wX wY . (Conflict p, Patchy p, PatchListFormat p,
+       ToFromPrim p, Nonable p, ShowPatchBasic (PrimOf p), ShowPatchBasic p)
+       => FL p wX wY -> [Non p wX]
 toNons xs = map lastNon $ initsFL xs
-    where lastNon :: Sealed ((p :> FL p) C(x)) -> Non p C(x)
+    where lastNon :: Sealed ((p :> FL p) wX) -> Non p wX
           lastNon (Sealed xxx) =
               case lastNon_aux xxx of
                    deps :> p :> _ ->
@@ -528,25 +544,25 @@ toNons xs = map lastNon $ initsFL xs
                                           redText "ds are" $$ showPatch ds $$
                                           redText "pp is" $$ showPatch pp
 
-          reverseFoo :: (p :> FL p) C(x z) -> (RL p :> p) C(x z)
+          reverseFoo :: (p :> FL p) wX wZ -> (RL p :> p) wX wZ
           reverseFoo (p :> ps) = rf NilRL p ps
             where
-              rf :: RL p C(a b) -> p C(b c) -> FL p C(c d)
-                 -> (RL p :> p) C(a d)
+              rf :: RL p wA wB -> p wB wC -> FL p wC wD
+                 -> (RL p :> p) wA wD
               rf rs l NilFL = rs :> l
               rf rs x (y :>: ys) = rf (x :<: rs) y ys
 
-          lastNon_aux :: (p :> FL p) C(x z) -> (RL p :> p :> RL p) C(x z)
+          lastNon_aux :: (p :> FL p) wX wZ -> (RL p :> p :> RL p) wX wZ
           lastNon_aux = commuteWhatWeCanRL . reverseFoo
 
-initsFL :: Patchy p => FL p C(x y) -> [Sealed ((p :> FL p) C(x))]
+initsFL :: Patchy p => FL p wX wY -> [Sealed ((p :> FL p) wX)]
 initsFL NilFL = []
 initsFL (x :>: xs) =
     Sealed (x :> NilFL) :
     map (\(Sealed (y :> xs')) -> Sealed (x :> y :>: xs')) (initsFL xs)
 
-filterConflictsFL :: PrimPatch prim => Non (RealPatch prim) C(x)
-                  -> FL prim C(x y) -> (FL prim :> FL prim) C(x y)
+filterConflictsFL :: PrimPatch prim => Non (RealPatch prim) wX
+                  -> FL prim wX wY -> (FL prim :> FL prim) wX wY
 filterConflictsFL _ NilFL = NilFL :> NilFL
 filterConflictsFL n (p :>: ps)
     | Just n' <- commuteOrRemFromCtx (fromPrim p) n =
@@ -584,7 +600,7 @@ instance PrimPatch prim => Commute (RealPatch prim) where
     -- Handle using the inverting commuter, and the previous case.  N.b. this
     -- is innefficient, since we'll have to also try commuteNoConflicts again
     -- (which we know will fail, since we got here).
-    commute c@(InvConflictor _ _ _ :> Normal _) = invertCommute c
+    commute c@(InvConflictor{} :> Normal _) = invertCommute c
 
     commute (Conflictor a1 n1 p1 :> Conflictor a2 n2 p2)
         | Just a2_minus_p1 <- remove1 p1' a2
@@ -623,13 +639,13 @@ instance PrimPatch prim => Commute (RealPatch prim) where
     -- Handle using the inverting commuter, and the previous case. This is also
     -- innefficient, since we'll have to also try commuteNoConflicts again
     -- (which we know will fail, since we got here).
-    commute c@(InvConflictor _ _ _ :> InvConflictor _ _ _) = invertCommute c
+    commute c@(InvConflictor{} :> InvConflictor{}) = invertCommute c
 
     commute _ = Nothing
 
 instance PrimPatch prim => Merge (RealPatch prim) where
-    merge (InvConflictor _ _ _ :\/: _) = impossible
-    merge (_ :\/: InvConflictor _ _ _) = impossible
+    merge (InvConflictor{} :\/: _) = impossible
+    merge (_ :\/: InvConflictor{}) = impossible
     merge (Etacilpud _ :\/: _) = impossible
     merge (_ :\/: Etacilpud _) = impossible
 
@@ -679,7 +695,7 @@ instance PrimPatch prim => Merge (RealPatch prim) where
                     (x' : nyy) = reverse $ xx2nons iy yyx
 
     -- Handle using the swap merge and the previous case.
-    merge m@(Conflictor _ _ _ :\/: Normal _) = swapMerge m
+    merge m@(Conflictor{} :\/: Normal _) = swapMerge m
 
     -- mH see also cH
     merge (Conflictor ix xx x :\/: Conflictor iy yy y) =
@@ -712,9 +728,9 @@ instance PatchInspect prim => PatchInspect (RealPatch prim) where
     listTouchedFiles (Etacilpud p) = nonTouches p
     listTouchedFiles (Normal p) = listTouchedFiles p
     listTouchedFiles (Conflictor x c p) =
-        nubsort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
+        nubSort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
     listTouchedFiles (InvConflictor x c p) =
-        nubsort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
+        nubSort $ concatMap nonTouches x ++ listTouchedFiles c ++ nonTouches p
 
     hunkMatches f (Duplicate p) = nonHunkMatches f p
     hunkMatches f (Etacilpud p) = nonHunkMatches f p
@@ -724,9 +740,9 @@ instance PatchInspect prim => PatchInspect (RealPatch prim) where
     hunkMatches f (InvConflictor x c p) =
         any (nonHunkMatches f) x || hunkMatches f c || nonHunkMatches f p
 
-allConflictsWith :: PrimPatch prim => Non (RealPatch prim) C(x)
-                 -> [Non (RealPatch prim) C(x)]
-                 -> ([Non (RealPatch prim) C(x)], [Non (RealPatch prim) C(x)])
+allConflictsWith :: PrimPatch prim => Non (RealPatch prim) wX
+                 -> [Non (RealPatch prim) wX]
+                 -> ([Non (RealPatch prim) wX], [Non (RealPatch prim) wX])
 allConflictsWith x ys = acw $ partition (conflictsWith x) ys
   where
     acw ([], nc) = ([], nc)
@@ -734,8 +750,8 @@ allConflictsWith x ys = acw $ partition (conflictsWith x) ys
                          (c1, nc1) -> case acw (cs, nc1) of
                                           (xs', nc') -> (c : c1 ++ xs', nc')
 
-conflictsWith :: PrimPatch prim => Non (RealPatch prim) C(x)
-              -> Non (RealPatch prim) C(x) -> Bool
+conflictsWith :: PrimPatch prim => Non (RealPatch prim) wX
+              -> Non (RealPatch prim) wX -> Bool
 conflictsWith x y | x `dependsUpon` y || y `dependsUpon` x = False
 conflictsWith x (Non cy y) =
     case commuteOrRemFromCtxFL cy x of
@@ -746,8 +762,8 @@ conflictsWith x (Non cy y) =
                 Left _ -> True
         Nothing -> True
 
-dependsUpon :: PrimPatch prim => Non (RealPatch prim) C(x)
-            -> Non (RealPatch prim) C(x) -> Bool
+dependsUpon :: PrimPatch prim => Non (RealPatch prim) wX
+            -> Non (RealPatch prim) wX -> Bool
 dependsUpon (Non xs _) (Non ys y) =
     case removeSubsequenceFL (ys +>+ fromPrim y :>: NilFL) xs of
         Just _ -> True
@@ -759,24 +775,24 @@ x +++ [] = x
 (x:xs) +++ xys | Just ys <- remove1 x xys = x : (xs +++ ys)
                | otherwise = x : (xs +++ xys)
 
-swapMerge :: PrimPatch prim => (RealPatch prim :\/: RealPatch prim) C(x y)
-          -> (RealPatch prim :/\: RealPatch prim) C(x y)
+swapMerge :: PrimPatch prim => (RealPatch prim :\/: RealPatch prim) wX wY
+          -> (RealPatch prim :/\: RealPatch prim) wX wY
 swapMerge (x :\/: y) = case merge (y :\/: x) of x' :/\: y' -> y' :/\: x'
 
-invertCommute :: PrimPatch prim => (RealPatch prim :> RealPatch prim) C(x y)
-              -> Maybe ((RealPatch prim :> RealPatch prim) C(x y))
+invertCommute :: PrimPatch prim => (RealPatch prim :> RealPatch prim) wX wY
+              -> Maybe ((RealPatch prim :> RealPatch prim) wX wY)
 invertCommute (x :> y) = do ix' :> iy' <- commute (invert y :> invert x)
                             return (invert iy' :> invert ix')
 
-invertCommuteNC :: PrimPatch prim => (RealPatch prim :> RealPatch prim) C(x y)
-                -> Maybe ((RealPatch prim :> RealPatch prim) C(x y))
+invertCommuteNC :: PrimPatch prim => (RealPatch prim :> RealPatch prim) wX wY
+                -> Maybe ((RealPatch prim :> RealPatch prim) wX wY)
 invertCommuteNC (x :> y) = do
     ix' :> iy' <- commuteNoConflicts (invert y :> invert x)
     return (invert iy' :> invert ix')
 
 -- | 'pullCommon' @xs ys@ returns the set of patches that can be commuted out
 -- of both @xs@ and @ys@ along with the remnants of both lists
-pullCommon :: Patchy p => FL p C(o x) -> FL p C(o y) -> Common p C(o x y)
+pullCommon :: (Patchy p, MyEq p) => FL p wO wX -> FL p wO wY -> Common p wO wX wY
 pullCommon NilFL ys = Common NilFL NilFL ys
 pullCommon xs NilFL = Common NilFL xs NilFL
 pullCommon (x :>: xs) xys | Just ys <- removeFL x xys =
@@ -789,12 +805,12 @@ pullCommon (x :>: xs) ys =
 
 -- | 'Common' @cs xs ys@ represents two sequences of patches that have @cs@ in
 -- common, in other words @cs +>+ xs@ and @cs +>+ ys@
-data Common p C(o x y) where
-    Common :: FL p C(o i) -> FL p C(i x) -> FL p C(i y) -> Common p C(o x y)
+data Common p wO wX wY where
+    Common :: FL p wO wI -> FL p wI wX -> FL p wI wY -> Common p wO wX wY
 
 -- | 'pullCommonRL' @xs ys@ returns the set of patches that can be commuted
 --   out of both @xs@ and @ys@ along with the remnants of both lists
-pullCommonRL :: Patchy p => RL p C(x o) -> RL p C(y o) -> CommonRL p C(x y o)
+pullCommonRL :: (Patchy p, MyEq p) => RL p wX wO -> RL p wY wO -> CommonRL p wX wY wO
 pullCommonRL NilRL ys = CommonRL NilRL ys NilRL
 pullCommonRL xs NilRL = CommonRL xs NilRL NilRL
 pullCommonRL (x :<: xs) xys | Just ys <- removeRL x xys =
@@ -808,8 +824,8 @@ pullCommonRL (x :<: xs) ys =
 
 -- | 'CommonRL' @xs ys cs@' represents two sequences of patches that have @cs@
 -- in common, in other words @xs +<+ cs@ and @ys +<+ cs@
-data CommonRL p C(x y f) where
-    CommonRL :: RL p C(x i) -> RL p C(y i) -> RL p C(i f) -> CommonRL p C(x y f)
+data CommonRL p wX wY wF where
+    CommonRL :: RL p wX wI -> RL p wY wI -> RL p wI wF -> CommonRL p wX wY wF
 
 instance PrimPatch prim => Apply (RealPatch prim) where
     type ApplyState (RealPatch prim) = ApplyState prim
@@ -884,10 +900,32 @@ instance PrimPatch prim => ReadPatch (RealPatch prim) where
                     return $ Sealed $ Normal p
                ]
 
-instance PrimPatch prim => Show (RealPatch prim C(x y)) where
-    show p = renderString $ showPatch p
+instance Show2 prim => Show (RealPatch prim wX wY) where
+    showsPrec d (Normal prim) =
+        showParen (d > appPrec) $ showString "Darcs.Patch.V2.Real.Normal " . showsPrec2 (appPrec + 1) prim
 
-instance PrimPatch prim => Show2 (RealPatch prim) where
+    showsPrec d (Duplicate x) =
+        showParen (d > appPrec) $ showString "Darcs.Patch.V2.Real.Duplicate " . showsPrec (appPrec + 1) x
+
+    showsPrec d (Etacilpud x) =
+        showParen (d > appPrec) $ showString "Darcs.Patch.V2.Etacilpud " . showsPrec (appPrec + 1) x
+
+    showsPrec d (Conflictor ix xx x) =
+        showParen (d > appPrec) $
+            showString "Darcs.Patch.V2.Real.Conflictor " . showsPrec (appPrec + 1) ix .
+            showString " " . showsPrec (appPrec + 1) xx .
+            showString " " . showsPrec (appPrec + 1) x
+
+    showsPrec d (InvConflictor ix xx x) =
+        showParen (d > appPrec) $
+            showString "Darcs.Patch.V2.Real.InvConflictor " . showsPrec (appPrec + 1) ix .
+            showString " " . showsPrec (appPrec + 1) xx .
+            showString " " . showsPrec (appPrec + 1) x
+
+instance Show2 prim => Show1 (RealPatch prim wX) where
+    showDict1 = ShowDictClass
+
+instance Show2 prim => Show2 (RealPatch prim) where
     showDict2 = ShowDictClass
 
 instance PrimPatch prim => Nonable (RealPatch prim) where
