@@ -575,7 +575,7 @@ copyCompleteRepoPacked2 fromRepo@(Repo fromDir _ _ fromCache) verb useCache clon
   allowCtrlC cloneKind cleanup $ do
     cleanDir "patches"
     putVerbose verb $ text "Using patches pack."
-    unpackPatches toCache3 (mapFL hashedPatchFileName $ newset2FL us) .
+    unpackPatches toCache3 (mapRL hashedPatchFileName $ newset2RL us) .
       Tar.read . decompress =<< fetchFileLazyPS (fromDir ++ packsDir ++ "patches.tar.gz") Uncachable
     pi <- doesPatchIndexExist toDir
     when pi $ createPIWithInterrupt toRepo
