@@ -14,21 +14,22 @@
 -- along with this program; if not, write to the Free Software Foundation,
 -- Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+{-# OPTIONS_GHC -fno-warn-missing-methods #-}
 {-# LANGUAGE CPP, TypeSynonymInstances, FlexibleInstances, MultiParamTypeClasses #-}
 
-#include "gadts.h"
 
 module Darcs.Repository.HashedIO ( HashedIO,
                                    copyHashed, copyPartialsHashed,
-                                   cleanHashdir,
+                                   cleanHashdir, getHashedFiles,
                                    RW(RW) -- only exported to make warning go away
+                                 , pathsAndContents
                                  ) where
 
-import Darcs.Global ( darcsdir )
+import Darcs.Util.Global ( darcsdir )
 import qualified Data.Set as Set
 import System.Directory ( getDirectoryContents, createDirectoryIfMissing )
-import Control.Monad.State ( StateT, runStateT, modify, get, put, gets, lift )
-import Control.Monad ( when )
+import Control.Monad.State ( StateT, runStateT, modify, get, put, gets, lift, evalStateT )
+import Control.Monad ( when, void, unless )
 import Control.Applicative ( (<$>) )
 import Data.Maybe ( isJust )
 import System.IO.Unsafe ( unsafeInterleaveIO )
@@ -36,33 +37,43 @@ import System.IO.Unsafe ( unsafeInterleaveIO )
 import Darcs.Repository.Cache ( Cache(..), fetchFileUsingCache, writeFileUsingCache,
                                 peekInCache, speculateFileUsingCache,
                                 okayHash, cleanCachesWithHint, HashedDir(..), hashedDir )
-import Darcs.RepoPath ( FilePathLike, toFilePath )
 import Darcs.Patch.ApplyMonad ( ApplyMonad(..) )
-import Darcs.Flags ( Compression( .. ) )
-import Darcs.Lock ( writeAtomicFilePS, removeFileMayNotExist )
-import Darcs.Utils ( withCurrentDirectory )
-import Progress ( debugMessage, tediousSize, finishedOneIO )
-import Darcs.Patch.FileName ( FileName, normPath, fp2fn, fn2fp, fn2niceps, niceps2fn,
-                              breakOnDir, ownName, superName )
+import Darcs.Repository.Flags ( Compression( .. ), WithWorkingDir (..) )
+import Darcs.Repository.Lock ( writeAtomicFilePS, removeFileMayNotExist )
+import Darcs.Util.File ( withCurrentDirectory )
+import Darcs.Util.Progress ( debugMessage, tediousSize, finishedOneIO )
+import Darcs.Util.Path
+    ( FileName
+    , normPath
+    , fp2fn
+    , fn2fp
+    , fn2niceps
+    , niceps2fn
+    , breakOnDir
+    , ownName
+    , superName
+    , FilePathLike
+    , toFilePath
+    , isMaliciousSubPath
+    )
 
-import ByteStringUtils ( linesPS, unlinesPS )
+import Darcs.Util.ByteString ( linesPS, unlinesPS )
 import qualified Data.ByteString       as B  (ByteString, length, empty)
 import qualified Data.ByteString.Char8 as BC (unpack, pack)
 
 import Storage.Hashed.Darcs( readDarcsHashedDir, darcsLocation,
                              decodeDarcsHash, decodeDarcsSize )
 import Storage.Hashed.Tree( ItemType(..), Tree )
-import Darcs.CommandsAux
 
 -- | @readHashFile c subdir hash@ reads the file with hash @hash@ in dir subdir,
 -- fetching it from 'Cache' @c@ if needed.
 readHashFile :: Cache -> HashedDir -> String -> IO (String,B.ByteString)
 readHashFile c subdir hash =
-    do debugMessage $ "Reading hash file "++hash++" from "++(hashedDir subdir)++"/"
+    do debugMessage $ "Reading hash file "++hash++" from "++hashedDir subdir++"/"
        fetchFileUsingCache c subdir hash
 
 data HashDir r p = HashDir { permissions :: !r, cache :: !Cache,
-                             compress :: !Compression, rootHash :: !String }
+                             rootHash :: !String }
 type HashedIO p = StateT (HashDir RW p) IO
 
 data RW = RW
@@ -173,22 +184,18 @@ withh h j = do hd <- get
                return (h',x)
 
 inh :: String -> HashedIO p a -> HashedIO p a
-inh h j = do hd <- get
-             put $ hd { rootHash = h }
-             x <- j
-             put hd
-             return x
+inh h j = snd `fmap` withh h j
 
 readroot :: HashedIO p [(ObjType, FileName, String)]
 readroot = do haveitalready <- peekroot
               cc <- gets rootHash >>= readdir
-              when (not haveitalready) $ speculate cc
+              unless haveitalready $ speculate cc
               return cc
     where speculate :: [(a,b,String)] -> HashedIO q ()
           speculate c = do cac <- gets cache
                            mapM_ (\(_,_,z) -> lift $ speculateFileUsingCache cac HashedPristineDir z) c
           peekroot :: HashedIO p Bool
-          peekroot = do HashDir _ c _ h <- get
+          peekroot = do HashDir _ c h <- get
                         lift $ peekInCache c HashedPristineDir h
 
 writeroot :: [(ObjType, FileName, String)] -> HashedIO p ()
@@ -232,44 +239,70 @@ writedir c = writeHashFile cps
 
 writeHashFile :: B.ByteString -> HashedIO p String
 writeHashFile ps = do c <- gets cache
-                      compr <- gets compress
-                      lift $ writeFileUsingCache c compr HashedPristineDir ps
+                      -- pristine files are always compressed
+                      lift $ writeFileUsingCache c GzipCompression HashedPristineDir ps
 
-copyHashed :: String -> Cache -> Compression -> String -> IO ()
--- Warning:  A do-notation statement discarded a result of type ((), HashDir RO ghc-prim
-copyHashed k c compr z = do _ <- runStateT cph $ HashDir { permissions = RW, cache = c,
-                                                      compress = compr, rootHash = z }
-                            return ()
+
+-- | Grab a whole pristine tree from a hash, and, if asked,
+--   write files in the working copy.
+copyHashed :: String -> Cache -> WithWorkingDir -> String -> IO ()
+copyHashed k c wwd z = void . runStateT cph $ HashDir { permissions = RW, cache = c, rootHash = z }
     where cph = do cc <- readroot
                    lift $ tediousSize k (length cc)
                    mapM_ cp cc
-          cp (F,n,h) = do ps <- readhash h
-                          lift $ finishedOneIO k (fn2fp n)
-                          lift $ writeAtomicFilePS (fn2fp n) ps
-          cp (D,n,h) = if isMaliciousSubPath (fn2fp n)
-                          then fail ("Caught malicious path: " ++ fn2fp n)
-                          else do
-                          lift $ createDirectoryIfMissing False (fn2fp n)
-                          lift $ finishedOneIO k (fn2fp n)
-                          lift $ withCurrentDirectory (fn2fp n) $ copyHashed k c compr h
+          cp (F,n,h) = do
+              ps <- readhash h
+              lift $ finishedOneIO k (fn2fp n)
+              case wwd of
+                WithWorkingDir -> lift $ writeAtomicFilePS (fn2fp n) ps
+                NoWorkingDir   -> ps `seq` return ()
+                                  -- force evaluation of ps to actually copy hashed file
+          cp (D,n,h) =
+              if isMaliciousSubPath (fn2fp n)
+                 then fail ("Caught malicious path: " ++ fn2fp n)
+                 else do
+                 lift $ finishedOneIO k (fn2fp n)
+                 case wwd of
+                   WithWorkingDir -> do
+                     lift $ createDirectoryIfMissing False (fn2fp n)
+                     lift $ withCurrentDirectory (fn2fp n) $ copyHashed k c WithWorkingDir h
+                   NoWorkingDir ->
+                     lift $ copyHashed k c NoWorkingDir h
+
+-- | Returns a list of pairs (FilePath, (strict) ByteString) of
+--   the pristine tree starting with the hash @root@.
+--   @path@ should be either "." or end with "/"
+--   Separator "/" is used since this function is used to generate
+--   zip archives from pristine trees.
+pathsAndContents :: FilePath -> Cache ->  String -> IO [(FilePath,B.ByteString)]
+pathsAndContents path c root = evalStateT cph $ HashDir { permissions = RW, cache = c, rootHash = root }
+    where cph = do cc <- readroot
+                   pacs <- concat <$> mapM cp cc
+                   let current = if path == "." then [] else [(path ++ "/" , B.empty)]
+                   return $ current ++ pacs
+          cp (F,n,h) = do
+              ps <- readhash h
+              let p = (if path == "." then "" else path ++ "/") ++ fn2fp n
+              return [(p,ps)]
+          cp (D,n,h) = do
+              let p = (if path == "." then "" else path) ++ fn2fp n ++ "/"
+              lift $ pathsAndContents p c h
 
 copyPartialsHashed :: FilePathLike fp =>
-                      Cache -> Compression -> String -> [fp] -> IO ()
-copyPartialsHashed c compr root = mapM_ (copyPartialHashed c compr root)
+                      Cache -> String -> [fp] -> IO ()
+copyPartialsHashed c root = mapM_ (copyPartialHashed c root)
 
-copyPartialHashed :: FilePathLike fp => Cache -> Compression -> String -> fp -> IO ()
-copyPartialHashed c compr root ff =
+copyPartialHashed :: FilePathLike fp => Cache -> String -> fp -> IO ()
+copyPartialHashed c root ff =
     do createDirectoryIfMissing True (basename $ toFilePath ff)
--- Warning:  A do-notation statement discarded a result of type ((), HashDir RO ghc-prim
-       _ <- runStateT (cp $ fp2fn $ toFilePath ff) $
+       void $ runStateT (cp $ fp2fn $ toFilePath ff)
                  HashDir { permissions = RW, cache = c,
-                           compress=compr, rootHash = root }
-       return ()
+                           rootHash = root }
  where basename = reverse . dropWhile ('/' /=) . dropWhile ('/' ==) . reverse
        cp f = do mt <- identifyThing f
                  case mt of
                    Just (D,h) -> do lift $ createDirectoryIfMissing True (fn2fp f)
-                                    lift $ withCurrentDirectory (fn2fp f) $ copyHashed "" c compr h
+                                    lift $ withCurrentDirectory (fn2fp f) $ copyHashed "" c WithWorkingDir h
                    Just (F,h) -> do ps <- readhash h
                                     lift $ writeAtomicFilePS (fn2fp f) ps
                    Nothing -> return ()
@@ -277,15 +310,9 @@ copyPartialHashed c compr root ff =
 cleanHashdir :: Cache -> HashedDir -> [String] -> IO ()
 cleanHashdir c dir_ hashroots =
    do -- we'll remove obsolete bits of "dir"
-      debugMessage $ "Cleaning out " ++ (hashedDir dir_) ++ "..."
-      let hashdir = darcsdir ++ "/" ++ (hashedDir dir_) ++ "/"
-          listone h = do let size = decodeDarcsSize $ BC.pack h
-                             hash = decodeDarcsHash $ BC.pack h
-                         x <- readDarcsHashedDir hashdir (size, hash)
-                         let subs = [ fst $ darcsLocation "" (s, h') | (TreeType, _, s, h') <- x ]
-                             hashes = h : [ fst $ darcsLocation "" (s, h') | (_, _, s, h') <- x ]
-                         (hashes++) . concat <$> mapM listone subs
-      hs <- set . concat <$> mapM listone hashroots
+      debugMessage $ "Cleaning out " ++ hashedDir dir_ ++ "..."
+      let hashdir = darcsdir ++ "/" ++ hashedDir dir_ ++ "/"
+      hs <- set <$> getHashedFiles hashdir hashroots
       fs <- set . filter okayHash <$> getDirectoryContents hashdir
       mapM_ (removeFileMayNotExist . (hashdir++)) (unset $ fs `Set.difference` hs)
       -- and also clean out any global caches.
@@ -293,3 +320,16 @@ cleanHashdir c dir_ hashroots =
       cleanCachesWithHint c dir_ (unset $ fs `Set.difference` hs)
    where set = Set.fromList . map BC.pack
          unset = map BC.unpack . Set.toList
+
+-- | getHashedFiles returns all hash files targeted by files in hashroots in
+-- the hashdir directory.
+getHashedFiles :: String -> [String] -> IO [String]
+getHashedFiles hashdir hashroots = do
+      let listone h = do let size = decodeDarcsSize $ BC.pack h
+                             hash = decodeDarcsHash $ BC.pack h
+                         x <- readDarcsHashedDir hashdir (size, hash)
+                         let subs = [ fst $ darcsLocation "" (s, h') | (TreeType, _, s, h') <- x ]
+                             hashes = h : [ fst $ darcsLocation "" (s, h') | (_, _, s, h') <- x ]
+                         (hashes++) . concat <$> mapM listone subs
+      hs <- concat <$> mapM listone hashroots
+      return hs
\ No newline at end of file
