@@ -1,4 +1,4 @@
-{-# OPTIONS_GHC -fno-warn-orphans #-}
+{-# OPTIONS_GHC -fno-warn-orphans -fno-warn-incomplete-patterns #-}
 {-# LANGUAGE CPP #-}
 
 -- Copyright (C) 2009 Petr Rockai
@@ -24,13 +24,16 @@
 -- SOFTWARE.
 
 module Darcs.Repository.State
-    ( restrictSubpaths, restrictBoring, TreeFilter(..)
+    ( restrictSubpaths, restrictBoring, TreeFilter(..), restrictDarcsdir
     -- * Diffs.
-    , unrecordedChanges, readPending
+    , unrecordedChanges, unrecordedChangesWithPatches, readPending
     -- * Trees.
     , readRecorded, readUnrecorded, readRecordedAndPending, readWorking
+    , readPendingAndWorking
     -- * Index.
-    , readIndex, invalidateIndex, UseIndex(..), ScanKnown(..) ) where
+    , readIndex, updateIndex, invalidateIndex, UseIndex(..), ScanKnown(..)
+    -- * Utilities
+    , filterOutConflicts ) where
 
 import Prelude hiding ( filter, catch )
 import Control.Monad( when )
@@ -43,28 +46,34 @@ import Text.Regex( matchRegex )
 import System.Directory( removeFile, doesFileExist, doesDirectoryExist, renameFile )
 import System.FilePath ( (</>) )
 import qualified Data.ByteString as BS
+    ( readFile, drop, writeFile, empty )
 import qualified Data.ByteString.Char8 as BSC
+    ( pack, split )
 
-import Darcs.Patch ( RepoPatch, PrimOf, sortCoalesceFL, fromPrim, effect )
+import Darcs.Patch ( RepoPatch, PrimOf, sortCoalesceFL, fromPrim, fromPrims
+                   , effect, anonymous )
 import Darcs.Patch.Apply ( ApplyState, applyToTree, effectOnFilePaths )
-import Darcs.Witnesses.Ordered ( FL(..), (+>+), mapFL_FL )
-import Darcs.Witnesses.Eq ( EqCheck(IsEq, NotEq) )
-import Darcs.Witnesses.Unsafe ( unsafeCoerceP )
-import Darcs.Witnesses.Sealed ( Sealed(Sealed), seal, unFreeLeft, mapSeal )
-import Darcs.Diff ( treeDiff )
-import Darcs.Flags ( UseIndex(..), ScanKnown(..) )
-import Darcs.Global ( darcsdir )
-import Darcs.Utils ( filterPaths )
+import Darcs.Patch.Witnesses.Ordered ( RL(..), FL(..), (+>+), mapFL_FL
+                                     , (:>)(..) )
+import Darcs.Patch.Witnesses.Eq ( EqCheck(IsEq, NotEq) )
+import Darcs.Patch.Witnesses.Unsafe ( unsafeCoerceP )
+import Darcs.Patch.Witnesses.Sealed ( Sealed(Sealed), seal, unFreeLeft, mapSeal )
+import Darcs.Patch.Commute ( selfCommuter )
+import Darcs.Patch.CommuteFn ( commuterIdRL )
+import Darcs.Patch.Permutations ( partitionConflictingFL )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, n2pia )
+
+import Darcs.Repository.Flags ( UseIndex(..), ScanKnown(..), DiffAlgorithm(..) )
+import Darcs.Util.Global ( darcsdir )
 
 import Darcs.Repository.InternalTypes ( Repository(..) )
 import Darcs.Repository.Format(formatHas, RepoProperty(NoWorkingDir))
 import qualified Darcs.Repository.LowLevel as LowLevel
 import Darcs.Repository.Prefs ( filetypeFunction, boringRegexps )
+import Darcs.Repository.Diff ( treeDiff )
 
-import Darcs.Patch.FileName ( fn2fp )
-import Darcs.RepoPath ( SubPath, sp2fn )
-
-import Storage.Hashed.AnchoredPath( AnchoredPath(..), anchorPath, floatPath, Name(..) )
+import Darcs.Util.Path( AnchoredPath(..), anchorPath, floatPath, Name(..), fn2fp,
+                   SubPath, sp2fn, filterPaths )
 import Storage.Hashed.Tree( Tree, restrict, FilterTree, expand, filter, emptyTree, overlay, find )
 import Storage.Hashed.Plain( readPlainTree )
 import Storage.Hashed.Darcs( darcsTreeHash, readDarcsHashed, decodeDarcsHash, decodeDarcsSize )
@@ -72,7 +81,6 @@ import Storage.Hashed.Hash( Hash( NoHash ) )
 import qualified Storage.Hashed.Index as I
 import qualified Storage.Hashed.Tree as Tree
 
-#include "gadts.h"
 
 newtype TreeFilter m = TreeFilter { applyTreeFilter :: forall tr . FilterTree tr m => tr m -> tr m }
 
@@ -81,20 +89,20 @@ newtype TreeFilter m = TreeFilter { applyTreeFilter :: forall tr . FilterTree tr
 -- constraint everywhere. When we have GHC 7.2 as a minimum requirement, we can
 -- lift this constraint into RepoPatch superclass context and remove this hack.
 readPendingLL :: (RepoPatch p, ApplyState p ~ Tree)
-              => Repository p C(r u t) -> IO (Sealed ((FL p) C(t)))
+              => Repository p wR wU wT -> IO (Sealed ((FL p) wT))
 readPendingLL repo = mapSeal (mapFL_FL fromPrim) `fmap` LowLevel.readPending repo
 
 -- | From a repository and a list of SubPath's, construct a filter that can be
 -- used on a Tree (recorded or unrecorded state) of this repository. This
 -- constructed filter will take pending into account, so the subpaths will be
 -- translated correctly relative to pending move patches.
-restrictSubpaths :: forall p m C(r u t). (RepoPatch p, ApplyState p ~ Tree)
-                 => Repository p C(r u t) -> [SubPath]
+restrictSubpaths :: forall p m wR wU wT. (RepoPatch p, ApplyState p ~ Tree)
+                 => Repository p wR wU wT -> [SubPath]
                  -> IO (TreeFilter m)
 restrictSubpaths repo subpaths = do
   Sealed pending <- readPendingLL repo
   let paths = map (fn2fp . sp2fn) subpaths
-      paths' = paths `union` (effectOnFilePaths pending paths)
+      paths' = paths `union` effectOnFilePaths pending paths
       anchored = map floatPath paths'
       restrictPaths :: FilterTree tree m => tree m -> tree m
       restrictPaths = filter (filterPaths anchored)
@@ -153,15 +161,83 @@ restrictDarcsdir = TreeFilter $ filter $ \p _ -> not (inDarcsDir p)
 -- is very inefficient, although in extremely rare cases, the index could go
 -- out of sync (file is modified, index is updated and file is modified again
 -- within a single second).
-unrecordedChanges :: forall p C(r u t) . (RepoPatch p, ApplyState p ~ Tree)
-                  => (UseIndex, ScanKnown) -> Repository p C(r u t)
-                  -> Maybe [SubPath] -> IO (FL (PrimOf p) C(t u))
-unrecordedChanges _ r@(Repo _ _ rf _) _
-  | (formatHas NoWorkingDir rf) = do
+unrecordedChanges :: forall p wR wU wT . (RepoPatch p, ApplyState p ~ Tree)
+                  => (UseIndex, ScanKnown, DiffAlgorithm) -> Repository p wR wU wT
+                  -> Maybe [SubPath] -> IO (FL (PrimOf p) wT wU)
+unrecordedChanges opts r paths = do
+    (pending :> working) <- readPendingAndWorking opts r paths
+    return $ sortCoalesceFL (pending +>+ working)
+
+unrecordedChangesWithPatches :: forall p wR wU wT wX. (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+                  => (UseIndex, ScanKnown, DiffAlgorithm) -> Repository p wR wU wT
+                  -> Maybe [SubPath]
+                  -> FL (PrimOf p) wX wT -- look-for-moves patches
+                  -> FL (PrimOf p) wT wT -- look-for-replaces patches
+                  -> IO (FL (PrimOf p) wT wU)
+unrecordedChangesWithPatches opts r paths movesPs replacesPs = do
+    (pending :> working) <- readPendingAndWorkingWithPatches opts r paths movesPs replacesPs
+    return $ sortCoalesceFL (pending +>+ unsafeCoerceP (movesPs +>+ replacesPs) +>+ working)
+
+-- | Mostly a helper function to 'unrecordedChangesWithPatches', returning the pending
+--   patch plus `patches` and the subsequent diff from working as two different patches
+readPendingAndWorkingWithPatches :: forall p wR wU wT wZ. (RepoPatch p, ApplyState p ~ Tree, ApplyState (PrimOf p) ~ Tree)
+                                 => (UseIndex, ScanKnown, DiffAlgorithm)
+                                 -> Repository p wR wU wT
+                                 -> Maybe [SubPath]
+                                 -> FL (PrimOf p) wZ wT  -- look-for-moves patches
+                                 -> FL (PrimOf p) wT wT  -- look-for-replaces patches
+                                 -> IO ((FL (PrimOf p) :> FL (PrimOf p)) wT wU)
+readPendingAndWorkingWithPatches _ r@(Repo _ rf _ _) _ _ _ | (formatHas NoWorkingDir rf) = do
     IsEq <- return $ workDirLessRepoWitness r
-    return NilFL
-unrecordedChanges (useidx, scan) repo mbpaths = do
-  (all_current, Sealed (pending :: FL p C(t x))) <- readPending repo
+    return (NilFL :> NilFL)
+readPendingAndWorkingWithPatches (useidx', scan, dflag) repo mbpaths movesPs replacesPs = do
+  let allPatches = movesPs +>+ replacesPs
+  let useidx = case allPatches of
+                 NilFL -> useidx'
+                 _ -> IgnoreIndex
+  (all_current, Sealed (pending :: FL p wT wX)) <- readPending repo
+  all_current_with_patches <- applyToTree allPatches all_current
+
+  relevant <- maybe (return $ TreeFilter id) (restrictSubpaths repo) mbpaths
+  let getIndex = applyToTree movesPs =<< I.updateIndex =<< (applyTreeFilter relevant <$> readIndex repo)
+      current = applyTreeFilter relevant all_current_with_patches
+
+  index <- getIndex
+  working <- applyTreeFilter restrictDarcsdir <$> case scan of
+    ScanKnown -> case useidx of
+      UseIndex -> getIndex
+      IgnoreIndex -> do
+        guide <- expand current
+        applyTreeFilter relevant <$> restrict guide <$> readPlainTree "."
+    ScanAll -> do
+      nonboring <- restrictBoring index
+      plain <- applyTreeFilter relevant <$> applyTreeFilter nonboring <$> readPlainTree "."
+      return $ case useidx of
+        UseIndex -> plain `overlay` index
+        IgnoreIndex -> plain
+    ScanBoring -> do
+      plain <- applyTreeFilter relevant <$> readPlainTree "."
+      return $ case useidx of
+        UseIndex -> plain `overlay` index
+        IgnoreIndex -> plain
+
+  ft <- filetypeFunction
+  Sealed (diff :: FL (PrimOf p) wX wY) <- (unFreeLeft `fmap` treeDiff dflag ft current working) :: IO (Sealed (FL (PrimOf p) wX))
+  IsEq <- return (unsafeCoerceP IsEq) :: IO (EqCheck wY wU)
+  return (effect pending :> diff)
+
+-- | Mostly a helper function to 'unrecordedChanges', returning the pending
+--   patch and the subsequent diff from working as two different patches
+readPendingAndWorking :: forall p wR wU wT . (RepoPatch p, ApplyState p ~ Tree)
+                      => (UseIndex, ScanKnown, DiffAlgorithm)
+                      -> Repository p wR wU wT
+                      -> Maybe [SubPath]
+                      -> IO ((FL (PrimOf p) :> FL (PrimOf p)) wT wU)
+readPendingAndWorking _ r@(Repo _ rf _ _) _ | (formatHas NoWorkingDir rf) = do
+    IsEq <- return $ workDirLessRepoWitness r
+    return (NilFL :> NilFL)
+readPendingAndWorking (useidx, scan, dflag) repo mbpaths = do
+  (all_current, Sealed (pending :: FL p wT wX)) <- readPending repo
 
   relevant <- maybe (return $ TreeFilter id) (restrictSubpaths repo) mbpaths
   let getIndex = I.updateIndex =<< (applyTreeFilter relevant <$> readIndex repo)
@@ -187,17 +263,16 @@ unrecordedChanges (useidx, scan) repo mbpaths = do
         IgnoreIndex -> plain
 
   ft <- filetypeFunction
-  Sealed (diff :: FL (PrimOf p) C(x y)) <- (unFreeLeft `fmap` treeDiff ft current working) :: IO (Sealed (FL (PrimOf p) C(x)))
-  IsEq <- return (unsafeCoerceP IsEq) :: IO (EqCheck C(y u))
-  return $ sortCoalesceFL (effect pending +>+ diff)
+  Sealed (diff :: FL (PrimOf p) wX wY) <- (unFreeLeft `fmap` treeDiff dflag ft current working) :: IO (Sealed (FL (PrimOf p) wX))
+  IsEq <- return (unsafeCoerceP IsEq) :: IO (EqCheck wY wU)
+  return (effect pending :> diff)
 
 -- | Witnesses the fact that in the absence of a working directory, we
 -- pretend that the working dir updates magically to the tentative state.
-workDirLessRepoWitness :: Repository p C(r u t)
-                          -> (EqCheck C(u t))
-workDirLessRepoWitness (Repo _ _ rf _) | formatHas NoWorkingDir rf =
-  unsafeCoerceP IsEq
-                                       | otherwise = NotEq
+workDirLessRepoWitness :: Repository p wR wU wT -> EqCheck wU wT
+workDirLessRepoWitness (Repo _ rf _ _)
+ | formatHas NoWorkingDir rf = unsafeCoerceP IsEq
+ | otherwise                 = NotEq
 
 -- | Obtains a Tree corresponding to the "recorded" state of the repository:
 -- this is the same as the pristine cache, which is the same as the result of
@@ -207,7 +282,7 @@ workDirLessRepoWitness (Repo _ _ rf _) | formatHas NoWorkingDir rf =
 -- no-pristine case, as that requires replaying patches. Cf. 'readDarcsHashed'
 -- and 'readPlainTree' in hashed-storage that are used to do the actual 'Tree'
 -- construction.
-readRecorded :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> IO (Tree IO)
+readRecorded :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO (Tree IO)
 readRecorded _repo = do
   let h_inventory = darcsdir </> "hashed_inventory"
   hashed <- doesFileExist h_inventory
@@ -236,7 +311,7 @@ readRecorded _repo = do
 -- parts of the index do not need to go through an up-to-date check (which
 -- involves a relatively expensive lstat(2) per file.
 readUnrecorded :: (RepoPatch p, ApplyState p ~ Tree)
-               => Repository p C(r u t) -> Maybe [SubPath] -> IO (Tree IO)
+               => Repository p wR wU wT -> Maybe [SubPath] -> IO (Tree IO)
 readUnrecorded repo mbpaths = do
   relevant <- maybe (return $ TreeFilter id) (restrictSubpaths repo) mbpaths
   readIndex repo >>= I.updateIndex . applyTreeFilter relevant
@@ -247,22 +322,22 @@ readUnrecorded repo mbpaths = do
 -- repositories.
 readWorking :: IO (Tree IO)
 readWorking = expand =<< (nodarcs `fmap` readPlainTree ".")
-  where nodarcs = Tree.filter (\(AnchoredPath (Name x:_)) _ -> x /= BSC.pack "_darcs")
+  where nodarcs = Tree.filter (\(AnchoredPath (Name x:_)) _ -> x /= BSC.pack darcsdir)
 
 readRecordedAndPending :: (RepoPatch p, ApplyState p ~ Tree)
-                       => Repository p C(r u t) -> IO (Tree IO)
+                       => Repository p wR wU wT -> IO (Tree IO)
 readRecordedAndPending repo = fst `fmap` readPending repo
 
 readPending :: (RepoPatch p, ApplyState p ~ Tree)
-            => Repository p C(r u t) -> IO (Tree IO, Sealed (FL p C(t)))
+            => Repository p wR wU wT -> IO (Tree IO, Sealed (FL p wT))
 readPending repo =
   do Sealed pending <- readPendingLL repo
      pristine <- readRecorded repo
      catch ((\t -> (t, seal pending)) `fmap` applyToTree pending pristine) $ \ (err :: IOException) -> do
        putStrLn $ "Yikes, pending has conflicts! " ++ show err
-       putStrLn $ "Stashing the buggy pending as _darcs/patches/pending_buggy"
-       renameFile "_darcs/patches/pending"
-                  "_darcs/patches/pending_buggy"
+       putStrLn "Stashing the buggy pending as _darcs/patches/pending_buggy"
+       renameFile (darcsdir </> "patches" </> "pending")
+                  (darcsdir </> "patches" </> "pending_buggy")
        return (pristine, seal NilFL)
 
 -- | Mark the existing index as invalid. This has to be called whenever the
@@ -271,25 +346,59 @@ readPending repo =
 -- pristine: changes to file content in either pristine or working are handled
 -- transparently by the index reading code.)
 invalidateIndex :: t -> IO ()
-invalidateIndex _ = do
-  BS.writeFile "_darcs/index_invalid" BS.empty
+invalidateIndex _ = BS.writeFile (darcsdir </> "index_invalid") BS.empty
 
-readIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p C(r u t) -> IO I.Index
+readIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO I.Index
 readIndex repo = do
-  invalid <- doesFileExist "_darcs/index_invalid"
-  exist <- doesFileExist "_darcs/index"
-  format_valid <- if exist
-                     then I.indexFormatValid "_darcs/index"
+  invalid <- doesFileExist $ darcsdir </> "index_invalid"
+  exists <- doesFileExist $ darcsdir </> "index"
+  formatValid <- if exists
+                     then I.indexFormatValid $ darcsdir </> "index"
                      else return True
-  when (exist && not format_valid) $
+  when (exists && not formatValid) $
 #if mingw32_HOST_OS
-       renameFile "_darcs/index" "_darcs/index.old"
+       renameFile (darcsdir </> "index") (darcsdir </> "index.old")
 #else
-       removeFile "_darcs/index"
+       removeFile $ darcsdir </> "index"
 #endif
-  if (not exist || invalid || not format_valid)
+  if not exists || invalid || not formatValid
      then do pris <- readRecordedAndPending repo
-             idx <- I.updateIndexFrom "_darcs/index" darcsTreeHash pris
-             when invalid $ removeFile "_darcs/index_invalid"
+             idx <- I.updateIndexFrom (darcsdir </> "index") darcsTreeHash pris
+             when invalid $ removeFile $ darcsdir </> "index_invalid"
              return idx
-     else I.readIndex "_darcs/index" darcsTreeHash
+     else I.readIndex (darcsdir </> "index") darcsTreeHash
+
+updateIndex :: (RepoPatch p, ApplyState p ~ Tree) => Repository p wR wU wT -> IO ()
+updateIndex repo = do
+    invalid <- doesFileExist $ darcsdir </> "index_invalid"
+    exists <- doesFileExist $ darcsdir </> "index"
+    formatValid <- if exists
+                     then I.indexFormatValid $ darcsdir </> "index"
+                     else return True
+    when (exists && not formatValid) $
+#if mingw32_HOST_OS
+       renameFile (darcsdir </> "index") (darcsdir </> "index.old")
+#else
+       removeFile $ darcsdir </> "index"
+#endif
+    pris <- readRecordedAndPending repo
+    _ <- I.updateIndexFrom (darcsdir </> "index") darcsTreeHash pris
+    when invalid $ removeFile $ darcsdir </> "index_invalid"
+
+-- |Remove any patches (+dependencies) from a sequence that
+-- conflict with the recorded or unrecorded changes in a repo
+filterOutConflicts :: (RepoPatch p, ApplyState p ~ Tree)
+     => RL (PatchInfoAnd p) wX wT                  -- ^Recorded patches from repository, starting from
+                                                   -- same context as the patches to filter
+     -> Repository p wR wU wT                      -- ^Repository itself, used for grabbing unrecorded changes
+     -> FL (PatchInfoAnd p) wX wZ                  -- ^Patches to filter
+     -> IO (Bool, Sealed (FL (PatchInfoAnd p) wX)) -- ^(True iff any patches were removed, possibly filtered patches)
+filterOutConflicts us repository them
+     = do let commuter = commuterIdRL selfCommuter
+          unrec <- fmap n2pia . anonymous . fromPrims
+                     =<< unrecordedChanges (UseIndex, ScanKnown, MyersDiff) repository Nothing
+          them' :> rest <- return $ partitionConflictingFL commuter them (unrec :<: us)
+          return (check rest, Sealed them')
+  where check :: FL p wA wB -> Bool
+        check NilFL = False
+        check _ = True
