@@ -98,7 +98,7 @@ applyHelp = unlines
  [ "The `darcs apply` command takes a patch bundle and attempts to insert"
  , "it into the current repository.  In addition to invoking it directly"
  , "on bundles created by `darcs send`, it is used internally by `darcs"
- , "push` and `darcs put` on the remote end of an SSH connection."
+ , "push` on the remote end of an SSH connection."
  , ""
  , "If no file is supplied, the bundle is read from standard input."
  , ""
