@@ -241,7 +241,7 @@ showLog opts files =
     else do let header = if isJust files && XMLOutput `notElem` opts
                           then text $ "Changes to "++unwords (fromJust recFiles)++":\n"
                           else empty
-            debugMessage "About to print the changes..."
+            debugMessage "About to print the patches..."
             let printers = if XMLOutput `elem` opts then simplePrinters else fancyPrinters
             ps <- readRepo repository -- read repo again to prevent holding onto
                                        -- values forced by filtered_changes
@@ -257,10 +257,10 @@ logHelp' :: String
 logHelp' =
  "When given one or more files or directories as arguments, only\n" ++
  "patches which affect those files or directories are listed. This\n" ++
- "includes changes that happened to files before they were moved or\n" ++
+ "includes patches that happened to files before they were moved or\n" ++
  "renamed.\n" ++
  "\n" ++
- "When given a `--from-tag`, `--from-patch` or `--from-match`, only changes\n" ++
+ "When given a `--from-tag`, `--from-patch` or `--from-match`, only patches\n" ++
  "since that tag or patch are listed.  Similarly, the `--to-tag`,\n" ++
  "`--to-patch` and `--to-match` options restrict the list to older patches.\n" ++
  "\n" ++
@@ -348,7 +348,7 @@ filterPatchesByNames maxcount fns patches = removeNonRenames $
                 then returnFinal
                 else case hopefullyM hp of
                     Nothing -> do
-                        let err = text "Can't find changes prior to:"
+                        let err = text "Can't find patches prior to:"
                                   $$ showPatchInfo (info hp)
                         return ([], renames, Just err)
                     Just p ->
@@ -441,7 +441,7 @@ logHelp'' :: String
 logHelp'' =
  "Three output formats exist.  The default is `--human-readable`.  You can\n" ++
  "also select `--context`, which is the internal format (as seen in patch\n" ++
- "bundles) that can be re-read by Darcs (e.g. `darcs get --context`).\n" ++
+ "bundles) that can be re-read by Darcs (e.g. `darcs clone --context`).\n" ++
  "\n" ++
  "Finally, there is `--xml-output`, which emits valid XML... unless a the\n" ++
  "patch metadata (author, name or description) contains a non-ASCII\n" ++
@@ -449,7 +449,7 @@ logHelp'' =
  "\n" ++
  -- FIXME: can't we just disallow the following usage?
  "Note that while the `--context` flag may be used in conjunction with\n" ++
- "`--xml-output` or `--human-readable`, in neither case will darcs get be\n" ++
+ "`--xml-output` or `--human-readable`, in neither case will darcs clone be\n" ++
  "able to read the output.  On the other hand, sufficient information\n" ++
  "WILL be output for a knowledgeable human to recreate the current state\n" ++
  "of the repository.\n"
@@ -486,7 +486,7 @@ logPatchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveralOrRange flags
     , S.diffAlgorithm = diffAlgorithm flags
-    , S.interactive = isInteractive flags
+    , S.interactive = isInteractive False flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = hasSummary O.NoSummary flags
     , S.withContext = isUnified flags
