@@ -63,8 +63,7 @@ import Darcs.Repository.Old ( oldRepoFailMsg )
 import Darcs.Repository.InternalTypes ( Repository(..) )
 import Darcs.Repository.Util ( getRecursiveDarcsRepos )
 import Darcs.Patch.Witnesses.Ordered
-     ( mapRL
-     , mapFL
+     ( mapFL
      , bunchFL
      , lengthRL
      )
@@ -427,7 +426,7 @@ optimizeHelpRelink =
  "`--sibling` option (which can be used multiple times).\n" ++
  "\n" ++
  "Darcs uses hard-links automatically, so this command is rarely needed.\n" ++
- "It is most useful if you used `cp -r` instead of `darcs get` to copy a\n" ++
+ "It is most useful if you used `cp -r` instead of `darcs clone` to copy a\n" ++
  "repository, or if you pulled the same patch from a remote repository\n" ++
  "into multiple local repositories."
 
@@ -532,7 +531,7 @@ doOptimizeHTTP repo = flip finally (mapM_ removeFileIfExists
   Just hash <- readHashedPristineRoot repo
   writeFile ( packsDir </> "pristine" ) hash
   -- pack patchesTar
-  ps <- mapRL hashedPatchFileName . newset2RL <$> readRepo repo
+  ps <- mapFL hashedPatchFileName . newset2FL <$> readRepo repo
   is <- map ((darcsdir </> "inventories") </>) <$> HashedRepo.listInventories
   writeFile (darcsdir </> "meta-filelist-inventories") . unlines $
     map takeFileName is
