@@ -122,7 +122,7 @@ fetchDescription =
 
 pullHelp :: String
 pullHelp = unlines
- [ "Pull is used to bring changes made in another repository into the current"
+ [ "Pull is used to bring patches made in another repository into the current"
  , "repository (that is, either the one in the current directory, or the one"
  , "specified with the `--repodir` option). Pull allows you to bring over all or"
  , "some of the patches that are in that repository but not in this one. Pull"
@@ -145,7 +145,7 @@ pullHelp = unlines
 
 fetchHelp :: String
 fetchHelp = unlines
- [ "Fetch is similar to `pull` except that it does not apply any changes"
+ [ "Fetch is similar to `pull` except that it does not apply any patches"
  , "to the current repository. Instead, it generates a patch bundle that"
  , "you can apply later with `apply`."
  , ""
@@ -410,7 +410,7 @@ fetchPatches o opts unfixedrepodirs@(_:_) jobname repository = do
         then filterOutConflicts (reverseFL us') repository ps
         else return (False, Sealed ps)
   when hadConflicts $ putStrLn "Skipping some patches which would cause conflicts."
-  when  (nullFL psFiltered) $ do putInfo opts $ text "No remote changes to pull in!"
+  when  (nullFL psFiltered) $ do putInfo opts $ text "No remote patches to pull in!"
                                  setEnvDarcsPatches psFiltered
                                  when (reorder opts /= O.Reorder) exitSuccess
   let direction = if doReverse opts then FirstReversed else First
@@ -482,7 +482,7 @@ pullPatchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveral flags
     , S.diffAlgorithm = diffAlgorithm flags
-    , S.interactive = isInteractive flags
+    , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
     , S.withContext = isUnified flags
