@@ -540,7 +540,7 @@ recordHelp'' =
  "\n" ++
  "The `--set-scripts-executable` option causes scripts to be made\n" ++
  "executable in the clean copy of the working tree, prior to running the\n" ++
- "test.  See `darcs get` for an explanation of the script heuristic.\n" ++
+ "test.  See `darcs clone` for an explanation of the script heuristic.\n" ++
  "\n" ++
  "If your test command is tediously slow (e.g. `make all`) and you are\n" ++
  "recording several patches in a row, you may wish to use `--no-test` to\n" ++
