@@ -131,6 +131,7 @@ import Darcs.Util.Printer
     ( Doc, vsep, text, ($$), (<+>), (<>), putDoc, putDocLn
     , renderPS, RenderMode(..)
     )
+import Darcs.Util.English ( englishNum, Noun(..) )
 import Darcs.Util.Path ( FilePathLike, toFilePath, AbsolutePath, AbsolutePathOrStd,
                         getCurrentDirectory, useAbsoluteOrStd, makeAbsoluteOrStd )
 import Darcs.Util.Download.HTTP ( postUrl )
@@ -233,7 +234,7 @@ patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveral flags
     , S.diffAlgorithm = O.PatienceDiff
-    , S.interactive = isInteractive flags
+    , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
     , S.withContext = O.NoContext
@@ -318,7 +319,7 @@ sendToThem repo opts wtds their_name them = do
       (hasSummary O.NoSummary opts)
       (dryRun opts)
       O.NoXml
-      (isInteractive opts)
+      (isInteractive True opts)
       to_be_sent
   when (nullFL to_be_sent) $ do
       putInfo opts Msg.selectionIsNull
@@ -541,12 +542,13 @@ getDescription opts their_name patches =
                      
                      return (doc, Just updatedFile, tryGetCharset doc)
         Nothing -> return (patchdesc, Nothing, tryGetCharset patchdesc)
-    where patchdesc = text (if lengthFL patches == 1
-                               then "1 patch"
-                               else show (lengthFL patches) ++ " patches")
+    where patchdesc = text (show len)
+                      <+> text (englishNum len (Noun "patch") "")
                       <+> text "for repository" <+> text their_name <> text ":"
                       $$ text ""
                       $$ vsep (mapFL description patches)
+            where
+              len = lengthFL patches
           updateFilename file = 
                 maybe (renameFile file darcsSendMessageFinal >>
                        return darcsSendMessageFinal) (return . toFilePath) $ hasLogfile opts
