@@ -351,7 +351,7 @@ genericObliterateCmd cmdname _ opts _ =
                       (hasSummary O.NoSummary opts)
                       (dryRun opts)
                       (hasXmlOutput opts)
-                      (isInteractive opts)
+                      (isInteractive True opts)
                       removed
                     setEnvDarcsPatches removed
                     when (isJust $ getOutput opts "") $
@@ -438,7 +438,7 @@ patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = parseFlags O.matchSeveralOrLast flags
     , S.diffAlgorithm = diffAlgorithm flags
-    , S.interactive = isInteractive flags
+    , S.interactive = isInteractive True flags
     , S.selectDeps = selectDeps flags
     , S.summary = hasSummary O.NoSummary flags
     , S.withContext = O.NoContext
