@@ -148,7 +148,7 @@ patchSelOpts flags = S.PatchSelectionOptions
     { S.verbosity = verbosity flags
     , S.matchFlags = []
     , S.diffAlgorithm = diffAlgorithm flags
-    , S.interactive = isInteractive flags
+    , S.interactive = isInteractive True flags
     , S.selectDeps = O.PromptDeps -- option not supported, use default
     , S.summary = hasSummary (defaultSummary flags) flags
     , S.withContext = isUnified flags
