@@ -182,9 +182,9 @@ wantGuiPause fs = if (hasDiffCmd fs || hasExternalMerge fs) && hasPause fs then
     hasPause = (== O.YesWantGuiPause) . parseFlags O.pauseForGui
 
 -- | Non-trivial interaction between options. Explicit @-i@ or @-a@ dominates,
--- else @--count@, @--xml@, or @--dry-run@ imply @-a@, else default to interactive.
-isInteractive :: Config -> Bool
-isInteractive = oparse (O.dryRunXml ^ O.changesFormat ^ O.interactive) decide
+-- else @--count@, @--xml@, or @--dry-run@ imply @-a@, else use the def argument.
+isInteractive :: Bool -> Config -> Bool
+isInteractive def = oparse (O.dryRunXml ^ O.changesFormat ^ O.interactive) decide
   where
     decide :: O.DryRun -> O.XmlOutput -> Maybe O.ChangesFormat -> Maybe Bool -> Bool
     decide _           _        _                     (Just True)  = True
@@ -192,7 +192,7 @@ isInteractive = oparse (O.dryRunXml ^ O.changesFormat ^ O.interactive) decide
     decide _           _        (Just O.CountPatches) Nothing      = False
     decide _           O.YesXml _                     Nothing      = False
     decide O.YesDryRun _        _                     Nothing      = False
-    decide _           _        _                     Nothing      = True
+    decide _           _        _                     Nothing      = def
 
 maxCount :: Config -> Maybe Int
 maxCount = parseFlags O.matchMaxcount
