@@ -95,8 +95,7 @@ import Darcs.Patch.Info ( PatchInfo )
 import Darcs.Patch.Inspect ( PatchInspect )
 import Darcs.Patch.Invert ( Invert )
 import Darcs.Patch.Match ( haveNonrangeMatch, matchAPatch, matchAPatchread )
-import Darcs.Patch.Named ( Named )
-import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, hopefully, info, n2pia )
+import Darcs.Patch.PatchInfoAnd ( PatchInfoAnd, info, n2pia )
 import Darcs.Patch.Set ( PatchSet(..), newset2RL )
 import Darcs.Patch.Show ( ShowPatch )
 import Darcs.Patch.Split ( Splitter (..) )
@@ -213,7 +212,7 @@ selectionContext whch jn o spl fs =
  PSC { opts = o
      , splitter = spl
      , files = fs
-     , matchCriterion = iswanted (seal2 . hopefully) (matchFlags o)
+     , matchCriterion = iswanted seal2 (matchFlags o)
      , jobname = jn
      , allowSkipAll = True
      , pristine = Nothing
@@ -222,7 +221,7 @@ selectionContext whch jn o spl fs =
 
 -- | A generic 'PatchSelectionContext'.
 selectionContextGeneric :: (RepoPatch p, Invert q)
-                        => (forall wX wY . q wX wY -> Sealed2 (Named p))
+                        => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd p))
                         -> WhichChanges
                         -> String
                         -> PatchSelectionOptions
@@ -268,7 +267,7 @@ triv = MatchCriterion { mcHasNonrange = False, mcFunction = \ _ _ -> True }
 -- | 'iswanted' selects patches according to the given match flags
 iswanted :: forall p q
           . (RepoPatch p, Invert q)
-         => (forall wX wY . q wX wY -> Sealed2 (Named p))
+         => (forall wX wY . q wX wY -> Sealed2 (PatchInfoAnd p))
          -> [MatchFlag]
          -> MatchCriterion q
 iswanted extract mflags = MatchCriterion
