@@ -67,11 +67,14 @@ import qualified Data.ByteString.Internal   as BI
 import Data.ByteString (intercalate)
 import Data.ByteString.Internal (fromForeignPtr)
 
+#if mingw32_HOST_OS
+#else
 import Control.Exception ( catch, SomeException )
+#endif
 import System.IO
 import System.IO.Unsafe         ( unsafePerformIO )
 
-import Foreign.Storable         ( peekElemOff, peek )
+import Foreign.Storable         ( peek )
 import Foreign.Marshal.Array    ( advancePtr )
 import Foreign.C.Types          ( CInt(..) )
 
@@ -101,9 +104,12 @@ import qualified Codec.Compression.Zlib.Internal as ZI
 import Darcs.Util.Encoding ( decode, encode )
 import Darcs.Util.Global ( addCRCWarning )
 
+#if mingw32_HOST_OS
+#else
 import System.IO.MMap( mmapFileByteString )
 import System.Mem( performGC )
 import System.Posix.Files( fileSize, getSymbolicLinkStatus )
+#endif
 
 -- -----------------------------------------------------------------------------
 -- obsolete debugging code
@@ -405,6 +411,9 @@ gzReadStdin = do
 -- the file is assumed to be ISO-8859-1.
 
 mmapFilePS :: FilePath -> IO B.ByteString
+#if mingw32_HOST_OS
+mmapFilePS = B.readFile
+#else
 mmapFilePS f =
   mmapFileByteString f Nothing
    `catch` (\(_ :: SomeException) -> do
@@ -412,6 +421,7 @@ mmapFilePS f =
                      if size == 0
                         then return B.empty
                         else performGC >> mmapFileByteString f Nothing)
+#endif
 
 -- -------------------------------------------------------------------------
 -- fromPS2Hex
@@ -461,21 +471,13 @@ betweenLinesPS start end ps
 breakAfterNthNewline :: Int -> B.ByteString
                         -> Maybe (B.ByteString, B.ByteString)
 breakAfterNthNewline 0 the_ps | B.null the_ps = Just (B.empty, B.empty)
+                              | otherwise     = Just (B.empty, the_ps)
 breakAfterNthNewline n the_ps =
-  case BI.toForeignPtr the_ps of
-  (fp,the_s,l) ->
-   unsafePerformIO $ withForeignPtr fp $ \p ->
-   do let findit 0 s | s == end = return $ Just (the_ps, B.empty)
-          findit _ s | s == end = return Nothing
-          findit 0 s = let left_l = s - the_s
-                       in return $ Just (fromForeignPtr fp the_s left_l,
-                                         fromForeignPtr fp s (l - left_l))
-          findit i s = do w <- peekElemOff p s
-                          if w == nl then findit (i-1) (s+1)
-                                     else findit i (s+1)
-          nl = BI.c2w '\n'
-          end = the_s + l
-      findit n the_s
+  go n (B.elemIndices (BI.c2w '\n') the_ps)
+  where  go 0 []      = Just (the_ps, B.empty)
+         go _ []      = Nothing
+         go 1 (i:_)   = Just $ B.splitAt (i+1) the_ps
+         go !m (_:is) = go (m-1) is
 
 -- -------------------------------------------------------------------------
 -- breakBeforeNthNewline
@@ -484,21 +486,10 @@ breakBeforeNthNewline :: Int -> B.ByteString -> (B.ByteString, B.ByteString)
 breakBeforeNthNewline 0 the_ps
  | B.null the_ps = (B.empty, B.empty)
 breakBeforeNthNewline n the_ps =
- case BI.toForeignPtr the_ps of
- (fp,the_s,l) ->
-   unsafePerformIO $ withForeignPtr fp $ \p ->
-   do let findit _ s | s == end = return (the_ps, B.empty)
-          findit i s = do w <- peekElemOff p s
-                          if w == nl
-                            then if i == 0
-                                 then let left_l = s - the_s
-                                      in return (fromForeignPtr fp the_s left_l,
-                                                 fromForeignPtr fp s (l - left_l))
-                                 else findit (i-1) (s+1)
-                            else findit i (s+1)
-          nl = BI.c2w '\n'
-          end = the_s + l
-      findit n the_s
+  go n (B.elemIndices (BI.c2w '\n') the_ps)
+  where  go _ []      = (the_ps, B.empty)
+         go 0 (i:_)   = B.splitAt i the_ps
+         go !m (_:is) = go (m-1) is
 
 -- | Test if a ByteString is made of ascii characters
 isAscii :: B.ByteString -> Bool
