@@ -1,4 +1,4 @@
-import qualified Darcs.Bug as Bug_
+import qualified Darcs.Util.Bug as Bug_
 
 #define darcsBug (\imp_funny_name -> imp_funny_name (__FILE__,__LINE__,__TIME__,__DATE__))
 
