@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # This checks for a possible bug in patch selection where the no available
 # patches case is hit.
@@ -13,7 +13,7 @@ darcs add A
 darcs record -am A
 echo 'l1' >> A
 darcs record -am l1
-darcs amend-record -a --patch 'A'
+darcs amend -a --patch 'A'
 
 cd ..
 rm -rf temp1
