@@ -34,6 +34,6 @@ echo 'file2' > file2
 darcs record -lam 'file2'
 
 echo 'file2:amended' > file2
-echo 'nkya' | darcs amend-record
+echo 'nkya' | darcs amend
 
 darcs changes -p 'file2' -v | grep amended
