@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp0 temp1 temp2
 
