@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp
 mkdir temp
@@ -35,19 +35,19 @@ darcs ann -p a2
 touch b
 darcs add b
 # test 0
-echo n/n/n/q | tr / \\012 | darcs rec -am b0
+echo nnnY | tr '[A-Z]' '[a-z]' | darcs rec -am b0
 darcs ann -p b0
 # test 1
 echo 1 > b
-echo n/n/y/q | tr / \\012 | darcs rec -am b1
+echo nnyY | tr '[A-Z]' '[a-z]' | darcs rec -am b1
 darcs ann -p b1 | grep '^\[a0'
 # test 2
 echo 2 > b
-echo n/y/q | tr / \\012 | darcs rec -am b2
+echo nyY | tr '[A-Z]' '[a-z]' | darcs rec -am b2
 darcs ann -p b2 | grep '^\[a1'
 # test 3
 echo 3 > b
-echo y/q | tr / \\012 | darcs rec -am b3
+echo yY | tr '[A-Z]' '[a-z]' | darcs rec -am b3
 darcs ann -p b3 | grep '^\[a2'
 
 cd ..
