@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 rm -fr temp1 temp2
 
@@ -23,7 +23,7 @@ mkdir temp2
 cd temp2
 darcs init
 darcs pull --all ../temp1
-darcs whatsnew | grep 'No changes'
+not darcs whatsnew
 cd ..
 
 rm -fr temp1 temp2
@@ -45,7 +45,7 @@ mkdir temp2
 cd temp2
 darcs init
 darcs pull --all ../temp1
-darcs whatsnew | grep 'No changes'
+not darcs whatsnew
 cd ..
 
 rm -fr temp1 temp2
