@@ -1,7 +1,7 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
-# Some tests for 'darcs changes'
+# Some tests for 'darcs changes -a'
 
 rm -rf temp1
 mkdir temp1
