@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # RT#544 using context created with 8-bit chars;
 rm -rf temp1
