@@ -1,10 +1,10 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp-$$
 mkdir temp-$$
 cd temp-$$
-set -e
+
 darcs initialize
 echo text > afile.txt
 darcs add afile.txt
@@ -13,6 +13,11 @@ darcs diff
 darcs diff --no-unified -p . --store-in-mem > diffinmem
 darcs diff --no-unified -p . --no-store-in-mem > diffondisk
 diff diffinmem diffondisk
-cd ..
 
+echo text >> afile.txt
+darcs diff | sed 's/afile\.txt.*//'> diffnoarg
+darcs diff . | sed 's/afile\.txt.*//' > diffdot
+diff diffnoarg diffdot
+
+cd ..
 rm -rf temp-$$
