@@ -1,12 +1,12 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 T=temp
 rm -rf "$T"
 mkdir "$T"
 echo "$T"
 cd "$T"
-set -e
+
 darcs initialize
 echo text > afile.txt
 darcs add afile.txt
