@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # tests for "darcs dist"
 
