@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # This demonstrates a bug that happens if you revert followed by
 # a partial unrevert and a full unrevert.  It requires that
@@ -22,7 +22,7 @@ sed 's/line2/Line2/' A  > A1; rm A; mv A1 A
 sed '4d' A > A1; rm A; mv A1 A
 sed 's/line6/Line6/' A > A1; rm A; mv A1 A
 darcs revert -a
-echo nyn | darcs unrev
+echo nyny | darcs unrev
 darcs unrev -a
 
 cd ..
