@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # Set up a base repo. Our experiment will start from this point
 mkdir base
