@@ -2,7 +2,7 @@
 
 # For issue68, 'don't report "resource vanished" when stdout pipe is broken.'
 
-set -ev
+. ./lib
 
 rm -rf temp1 # Another script may have left a mess.
 
