@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # Note that this is fixed, the lines marked # BUG HERE
 # should be moved back into merging_newlines.sh
