@@ -1,6 +1,6 @@
 #!/bin/sh
 
-set -ev
+. ./lib
 
 mkdir temp1
 cd temp1
