@@ -37,5 +37,5 @@ serve_http # sets baseurl
 darcs get $baseurl/R S
 cd S
 darcs pull ../R | tee log
-grep "No remote changes" log
+grep "No remote" log
 darcs check
