@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp1 temp2
 mkdir temp1
@@ -54,7 +54,7 @@ cd ..
 mkdir temp2
 cd temp2
 darcs init
-echo y/n/y | tr / \\012 | darcs pull ../temp1
+echo ynyy | darcs pull ../temp1
 darcs tag -Ax -m Tag
 darcs push -a ../temp1
 cd ..
