@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 rm -rf temp1
 mkdir temp1
 cd temp1
@@ -10,9 +10,9 @@ darcs add File.hs
 darcs record File.hs -a -m "add File"
 rm File.hs
 darcs record -a -m "rm File"
-darcs cha
+darcs changes
 darcs unrecord -p "rm File" -a
-darcs cha
+darcs changes
 darcs record -a -m "re-rm File"
 cd ..
 rm -rf temp1
