@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 rm -rf temp1 temp1-h
 
 gunzip -c $TESTDATA/many-files--old-fashioned-inventory.tgz | tar xf -
