@@ -1,10 +1,10 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp-$$
 mkdir temp-$$
 cd temp-$$
-set -e
+
 darcs initialize
 echo text > foo
 darcs add foo
