@@ -8,5 +8,5 @@
 
 gunzip -c  $TESTDATA/oldfashioned-compressed.tgz | tar xf -
 cd oldfashioned-compressed
-darcs optimize --upgrade
+darcs optimize upgrade
 darcs check
