@@ -1,7 +1,6 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
-. lib
 rm -rf temp
 mkdir temp
 cd temp
