@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-## Test for issue1739 - "Char.intToDigit: not a digit" in darcs changes 
+## Test for issue1739 - "Char.intToDigit: not a digit" in darcs changes
 ##
 ## Copyright (C) 2010  Reinier Lamers
 ##
@@ -25,6 +25,11 @@
 
 . lib
 
+# TODO: get this working on Windows
+# Seems to be a real bug - darcs doesn't record a patch with the
+# right characters.
+abort_windows
+
 # First, try to see if character set is UTF-8. If we can't find out or if it
 # isn't, we skip this test.
 if ! which locale ; then
@@ -45,6 +50,7 @@ fi
 
 # we want escaping, otherwise output of non-ASCII characters is unreliable
 export DARCS_DONT_ESCAPE_ANYTHING=0
+export DARCS_DONT_ESCAPE_8BIT=0
 
 rm -rf R
 mkdir R
