@@ -34,7 +34,7 @@ touch dir/file
 darcs add dir/file              # adds dir too (which is fine)
 darcs rec -a -m"add dir and file"
 
-darcs remove dir
+not darcs remove dir
 # removed dir but not file - should be nothing to add
 not darcs rec -a -m"remove dir"
 
