@@ -11,6 +11,6 @@ cd R
 echo 'Example content.' > f
 darcs rec -lam "first"
 rm -f f
-echo n | darcs record f 2>&1 | tee log
+echo ny | darcs record f 2>&1 | tee log
 not grep "None of the files" log
 cd ..
