@@ -23,8 +23,6 @@
 ## CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 ## SOFTWARE.
 
-TESTDATA=repos # remove this line in HEAD
-
 . lib                  # Load some portability helpers.
-tar zxf $TESTDATA/badrepo.tgz
+gunzip -c $TESTDATA/badrepo.tgz | tar xf -
 not darcs get badrepo
