@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # Issue244
 # darcs changes should be able to pull up the history for a file
