@@ -21,11 +21,11 @@ darcs record -a -m 'adding b' b.txt
 # extra confirmation for --all
 echo an | darcs obliterate -p add | grep -i "really obliterate"
 # --last=1
-echo n  | darcs obliterate --last 1 | grep -i adding
+echo nyy | darcs obliterate --last 1 | grep -i adding
 # automatically getting dependencies
 date >> a.txt
 darcs record -a -m 'modifying a' a.txt
-echo n | darcs obliterate -p 'adding a' > log
+echo ny | darcs obliterate -p 'adding a' > log
 grep -i "modifying a" log
 grep -i "No patches selected" log
 cd ..
