@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf tempA
 mkdir tempA
@@ -28,21 +28,21 @@ darcs mv baz world
 darcs replace baz world foo
 darcs record -a -m baz2world
 
-darcs whatsnew | grep 'No changes'
+not darcs whatsnew
 
 grep 'love the world' foo
 
-echo y | darcs obliterate -p baz2world
+echo yy | darcs obliterate -p baz2world
 
-darcs whatsnew | grep 'No changes'
+not darcs whatsnew
 
 grep 'love the baz' foo
 
-echo y | darcs obliterate -p bar2baz
+echo yy | darcs obliterate -p bar2baz
 
 grep 'love the bar' foo
 
-echo y | darcs obliterate -p nolove
+echo yy | darcs obliterate -p nolove
 
 grep 'love' foo && exit 1 || true
 
