@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # tests for "darcs optimize"
 
@@ -10,7 +10,7 @@ darcs init
 touch foo
 darcs add foo
 darcs record -a -m add_foo
-darcs optimize --reorder-patches| grep -i "done optimizing"
+darcs optimize reorder| grep -i "done"
 cd ..
 
 rm -rf temp1
