@@ -18,7 +18,7 @@ cd one
 mkdir two
 cd two
 echo darcs pull should work relative to the current directory
-darcs pull -a ../../../temp1 | grep -i 'No remote changes to pull in'
+darcs pull -a ../../../temp1 | grep -i 'No remote patches to pull in'
 
 echo -- darcs pull should pull into repo specified with --repo
 cd ../..  # now in temp2
