@@ -124,7 +124,7 @@ cat > p7.req <<EOF
 EOF
 
 darcs pull --dry-run --complement ../temp2 ../temp3 > p8.out
-grep "No remote changes to pull in!" p8.out
+grep "No remote patches to pull in!" p8.out
 
 # because duplicates are stripped before performing action,
 # this is the same as: darcs pull ../temp1
@@ -136,7 +136,7 @@ diff p1.req p9.out
 # so the "null" pull must be tested this way:
 darcs get ../temp1 ../temp5
 darcs pull --dry-run --complement ../temp1 ../temp5 > p9.out
-grep "No remote changes to pull in!" p9.out
+grep "No remote patches to pull in!" p9.out
 
 darcs pull -av --complement ../temp1 ../temp3
 darcs check
