@@ -3,7 +3,7 @@
 # This test script, originally written by David Roundy is in the public
 # domain.
 
-set -ev
+. ./lib
 
 rm -rf temp1 temp2
 
