@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 # For issue855: wish: avoid taking lock if using --dry-run
 chmod -R u+w temp2 || :
