@@ -33,7 +33,7 @@ cd ..
 mkdir -p temp2/one/two
 cd temp2/one/two
 # darcs push should work relative to the current directory
-darcs push -a ../../../temp1 | grep -i 'No recorded local changes to push'
+darcs push -a ../../../temp1 | grep -i 'No recorded local patches to push'
 cd ../../../
 
 # darcs push should push into repo specified with --repo
