@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp temp_0
 mkdir temp
