@@ -2,7 +2,7 @@
 
 # For issue257: push => incorrect return code when couldn't get lock
 
-set -ev
+. ./lib
 
 rm -rf tempc
 mkdir tempc
