@@ -4,8 +4,6 @@
 
 . lib
 
-abort_windows
-
 rm -rf temp1
 
 export DARCS_EDITOR="cat -n"
@@ -15,7 +13,7 @@ cd temp1
 darcs init
 touch file.t
 darcs add file.t
-echo y | darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep '# Please enter'
 cd ..
 rm -rf temp1
 
@@ -25,7 +23,7 @@ cd temp2\ dir
 darcs init
 touch file.t
 darcs add file.t
-echo y | darcs record --edit-long-comment -a -m foo file.t | grep '2.*END OF DESCRIPTION'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep '# Please enter'
 cd ..
 rm -rf temp2\ dir
 
@@ -51,14 +49,14 @@ echo y | darcs record --edit-long-comment -a -m edit | grep -c "./file" | grep 1
 cd ..
 rm -rf temp2\ dir
 
-export DARCS_EDITOR='grep "END OF"'
+export DARCS_EDITOR='grep "# Please enter"'
 # editor: quoting in command
 mkdir temp1
 cd temp1
 darcs init
 touch file.t
 darcs add file.t
-echo y | darcs record --edit-long-comment -a -m foo file.t | grep 'END OF'
+echo y | darcs record --edit-long-comment -a -m foo file.t | grep '# Please enter'
 cd ..
 rm -rf temp1
 
