@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf foo1 foo2
 mkdir foo1 foo2
