@@ -3,7 +3,7 @@
 rm -rf bad
 mkdir bad
 cd bad
-darcs init
+darcs init --no-patch-index
 
 echo foo > bar
 darcs add bar
