@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 rm -fr temp1
 
