@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 rm -rf temp
 mkdir temp
@@ -11,6 +11,6 @@ darcs add file1
 darcs record -am "test"
 rm file1
 darcs record -am "rm"
-echo yd | darcs rollback --last=1 -m 'oops' | grep 'No changes selected'
+echo yYd | tr [A-Z] [a-z] | darcs rollback --last=1 | grep 'No changes selected'
 cd ..
 rm -rf temp
