@@ -1,5 +1,5 @@
 #!/usr/bin/env bash
-set -ev
+. ./lib
 
 rm -rf temp1
 mkdir temp1
@@ -17,7 +17,7 @@ three
 one
 EOF
 darcs rec -Ax -am foo
-echo y | darcs unpull -p foo
+echo yy | darcs unpull -p foo
 cmp f g
 cd ..
 rm -rf temp1
