@@ -1,6 +1,6 @@
 #!/usr/bin/env bash
 
-set -ev
+. ./lib
 
 # Check that the right patches get unrecorded using --dont-prompt-for-dependencies
 
@@ -19,7 +19,7 @@ darcs record -Ax -alm bar2
 darcs unrec --no-deps -p foo1
 darcs changes -p foo --count | grep 2
 #foo1 is depended upon, we don't unpull it
-echo y | darcs unrec --dont-prompt-for-dependencies -p foo1
+echo yy | darcs unrec --dont-prompt-for-dependencies -p foo1
 #on the previous line, we don't get asked about foo2.
 darcs changes -p foo --count | grep 0
 #yet, it is unrecorded.
